/*
 * @(#)AbstractActionDefault.java 1.2 29.01.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

import java.awt.Component;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;

/** An abstract JGraphpad action, and is the base class for all JGraphpad actions.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public abstract class AbstractActionDefault
 extends AbstractAction
 implements GPBarConstants {

 /** A reference back to the graphpad.
  *  If an action was performed the
  *  Action applies the changes to the current
  *  Document at the graphpad.
  * 
  * 
  */
 protected GPGraphpad graphpad;

 /**
  * Constructor for AbstractActionDefault.
  * The Abstract action uses the class name
  * without package prefix as action name.
  * 
  * @see Action#NAME
  * 
  */
 public AbstractActionDefault() {
  this((GPGraphpad)null);
 }

 /**
  * Constructor for AbstractActionDefault.
  * The Abstract action uses the class name
  * without package prefix as action name.
  * 
  * 
  * @param graphpad The reference to the graphpad for this action.
  * @see Action#NAME
  * 
  */
 public AbstractActionDefault(GPGraphpad graphpad) {
  this.graphpad = graphpad;

  // build the name for this action
  // without the package prefix
  putValue(Action.NAME, Utilities.getClassNameWithoutPackage(getClass()));
 }


 /**
  * Constructor for AbstractActionDefault.
  * 
  * @param name This string is the key for the name of this action, and it is
  * sent to super().
  */

 public AbstractActionDefault(String name) {
  super(name);
 }
 /**
  * Constructor for AbstractActionDefault.
  * 
  * @param graphpad This GPGraphpad is a reference to the graphpad for this action.
  * @param name This string is the key for the name of this action, and it is
  * sent to super().
  */

 public AbstractActionDefault(GPGraphpad graphpad, String name) {
  super(name);
  this.graphpad = graphpad;
 }

 /**
  * Constructor for AbstractActionDefault.
  * 
  * @param graphpad This GPGraphpad is a reference to the graphpad for this action.
  * @param name This string is the key for the name of the action, and it is
  * sent to super().
  * @param icon This Icon is the icon for this action, and it is
  * sent to super().
  */

 public AbstractActionDefault(GPGraphpad graphpad, String name, Icon icon) {
  super(name, icon);
  this.graphpad = graphpad;
 }

 /**
  * Constructor for AbstractActionDefault.
  * 
  * @param name This is the key for the name of this action, and it is
  * sent to super().
  * @param icon This Icon is the icon for this action, and it is
  * sent to super().
  */

 public AbstractActionDefault(String name, Icon icon) {
  super(name, icon);
 }

 /** Returns the name of the action.
  * @return Returns the name of the action */
 public String getName() {
  return (String) getValue(NAME);
 }

 /** This gets the current graph.
  * @return Returns the current graph */        
 public GPGraph getCurrentGraph() {
   return graphpad.getCurrentGraph();
 }

 /** This gets the current graph layout cache.
  * @return Returns the current graph layout cache */        
 public GraphLayoutCache getCurrentGraphLayoutCache() {
   return graphpad.getCurrentDocument().getGraphLayoutCache();
 }
 
 /** This sets the selection attributes.
  * @param map This Map holds the selection attributes to be set. */
 public void setSelectionAttributes(Map map) {
   if (graphpad != null && graphpad.getCurrentDocument() != null)
     graphpad.getCurrentDocument().setSelectionAttributes(map);
 }

 /** This gets the current document.
  * @return Returns the current document. */        
 public GPDocument getCurrentDocument() {
  return graphpad.getCurrentDocument();
 }

 /** Creates by default an array with one
  *  entry. The entry contains a JMenuItem
  *  which joins the instance of this Action.
  * @return Returns a new menu component. */
 public Component[] getMenuComponents() {
   return new Component[] { getMenuComponent(null)};
 }
 
 /** Returns by default a list with one JButton.
  *  The button joints this action.
  * @return Returns a new tool component */
 public Component[] getToolComponents() {
   return new Component[] { getToolComponent(null)};
 }
 
 /** Returns a JMenuItem with a link to this action.
  * @param actionCommand This action is associated with a menu component.
  * @return Returns the JMenuItem component. */
 protected Component getMenuComponent(String actionCommand) {
   JMenuItem item = new JMenuItem(this);
   GPBarFactory.fillAbstractButton(item, getName(), actionCommand);
   String presentationText = getPresentationText(actionCommand);
   if (presentationText != null)
     item.setText(presentationText);
   
   return item;
 }
 
 /** Returns a clean JButton which has a link to this action.
  * @param actionCommand This action is associated with a tool component.
  * @return Returns a JButton component. */
 protected Component getToolComponent(String actionCommand) {
   AbstractButton b = new JButton(this) {
     public float getAlignmentY() {
       return 0.5f;
     }
   };
   return GPBarFactory.fillAbstractButton(b,
                                          getName(),
                                          actionCommand,
                                          false);
 }
 
 /** Empty implementation for this type of action.  */
 public void update() {
   if (graphpad.getCurrentDocument() == null)
     setEnabled(false);
  else
    setEnabled(true);
 }
 
 /** Should return presentation Text for the
  *  action command or null
  *  for the default.
  * @param actionCommand This string is the action command.
  * @return Returns presentation text or null. */
 public String getPresentationText(String actionCommand) {
   return null;
 }

 /**
  * Sets the graphpad.
  * @param graphpad The graphpad to set current Object to.
  */
 public void setGraphpad(GPGraphpad graphpad) {
  this.graphpad = graphpad;
 }

 /**
  * Returns the graphpad.
  * @return Returns current Object's graphpad field, of type GPGraphpad.
  */
 public GPGraphpad getGraphpad() {
  return graphpad;
 }

}
