import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import javax.swing.JFileChooser;



/**  An abstract action for a file.
 * @author sven.luzar
 * @version 2.0
 *
 */
public abstract class AbstractActionFile extends AbstractActionDefault {

 /** If the library files should be compressed (zipped)
  */
 public static final boolean COMPRESS_FILES = true;


 /**
  * Constructor for AbstractActionFile.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  */
 public AbstractActionFile(GPGraphpad graphpad) {
  super(graphpad);
 }
 
 
 ///**
  //* Constructor for AbstractActionFile.
  //* @param graphpad
  //* @param name
  //* @param icon
  //*/
 /*
 public AbstractActionFile(GPGraphpad graphpad, String name, Icon icon) {
  super(graphpad, name, icon);
 }*/

 /** Shows a file open dialog and returns the filename. 
  * @param message This string is the title of the file.
  * @param extension This string is the extension of the file.
  * @param desc This string is a description for the file.
  * @return Returns the filename as a String.
  */
 public String openDialog(String message, String extension, String desc) {
  return dialog(message, true, extension, desc);
 }

 /** Shows a file save dialog and returns the filename.
  * @param message This string is the title of the file.
  * @param extension This string is the extension of the file.
  * @param desc This string is a description for the file.
  * @return Returns the filename as a String.
  */
 public String saveDialog(String message, String extension, String desc) {
   return dialog(message, false, extension, desc);
 }
 
 /** Opens a dialog and returns the filename. 
  *  Returns <tt>null</tt> if cancelled. 
  * @param message This string is the title of the file.
  * @param open This boolean indicates whether or not the current file is open.
  * @param extension This string is the extension of the file.
  * @param desc This string is a description for the file.
  * @return Returns the filename as a String.
  * */
 protected String dialog(String message, boolean open, String extension, String desc) {
   //FileDialog f = new FileDialog(getFrame(), message, mode);
   JFileChooser f = new JFileChooser();
   f.setDialogTitle(message);
   String ext = (extension != null) ? "."+extension.toLowerCase() : "";
   if (extension != null)
     f.setFileFilter(new MyFileFilter(extension, desc));
   if (open)
     f.showOpenDialog(graphpad.getFrame());
   else
     f.showSaveDialog(graphpad.getFrame());
   if (f.getSelectedFile() != null) {
     String tmp = f.getSelectedFile().getAbsolutePath();
     if (extension != null && !tmp.toLowerCase().endsWith(ext))
       tmp += ext;
     return tmp;
   } else
     return null;
 }
 
 /** Creates an object input stream. 
  * @param filename This string is the name of the file.
  * @return Returns the ObjectInputStream that is created.
  */
 public static ObjectInputStream createInputStream(String filename)
   throws Exception {
   InputStream f = new FileInputStream(filename);
   if (COMPRESS_FILES)
     f = new GZIPInputStream(f);
   return new ObjectInputStream(f);
 }
 
 /** Creates an object output stream. 
  * @param filename This string is the name of the file.
  * @return Returns ObjectOutputStream that is created.
  */
 public static ObjectOutputStream createOutputStream(String filename)
  throws Exception {
  OutputStream f = new FileOutputStream(filename);
  if (COMPRESS_FILES)
   f = new GZIPOutputStream(f);
  return new ObjectOutputStream(f);
 }

 /** 
  * Filter for the jgraphpad file format (*.pad or *.lib).
  */
 protected class MyFileFilter extends javax.swing.filechooser.FileFilter {
   
   /** Extension for this file format.
    */     
   protected String ext; 
   /** Full extension for this file format (the Point and the extension)
    */
   protected String fullExt;
   /** Descrption of the File format.
    */
   protected String desc;
   
   /** Constructor for the Graphpad specific file format.
    * @param extension This string is the extension of the file.
    * @param description This string is a description of the file.
    */     
   public MyFileFilter(String extension, String description) {
     ext = extension.toLowerCase();
     fullExt = "."+ext;
     desc = description;
   }
   
   /** Returns true if the file ends with the full extension or 
    * if the file is a directory.
    * @param file This File object holds the information for the current file.
    */
   public boolean accept(File file) {
     return file.isDirectory() || file.getName().toLowerCase().endsWith(fullExt);
   }
   
   /** This gets the description of the file.
    * @return Returns the desc of the file.
    */
   public String getDescription() {
     return desc;
   }
   
 }
  
}
