/*
 * @(#)AbstractActionList.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;

import javax.swing.AbstractButton;
//import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.ListCellRenderer;


/** An abstract action for a list (of Objects).
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public abstract class AbstractActionList extends AbstractActionDefault {

 /** Item key for the user item.
  * 
  */
 public static final String ITEM_KEY = "itemKey";
 
 /**
  * Constructor for AbstractActionList.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  */
 public AbstractActionList(GPGraphpad graphpad) {
  super(graphpad);
 }

 /**
  * Constructor for AbstractActionList.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This string is the key for the name of the action, and it is
  * sent to super().
  */
 public AbstractActionList(GPGraphpad graphpad, String name) {
  super(graphpad, name);
 }

 /**
  * Constructor for AbstractActionList.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This string is the key for the name of the action, and it is
  * sent to super().
  * @param icon This Icon is the icon for this action, and it is
  * sent to super().
  */
 public AbstractActionList(GPGraphpad graphpad, String name, Icon icon) {
  super(graphpad, name, icon);
 }

 /** Gets all of the JMenuItem components for this list.
  * @see com.jgraph.pad.actions.AbstractActionDefault#getMenuBarComponents()
  * @return Returns an array of the JMenuItem components.
  */
 public Component[] getMenuComponents() {
  AbstractButton menu = getMenuBarComponent();
  GPBarFactory.fillAbstractButton(menu, getName(), null);

  Object[] items = getItems();

  for (int i = 0; i < items.length; i++) {
   JMenuItem item = new JMenuItem();
   menu.add(getMenuComponent(items[i].toString(), items[i]));
  }

  return new JComponent[] { menu };
 }
 
 /** Gets the JMenu for the current list.
  * @return Returns the JMenuItem for this list.
  */
 
 protected JMenu getMenuBarComponent() {
   return new JMenu(this);
 }
 
 /** Gets all of the JToolComponents for the current list.
  * @see com.jgraph.pad.actions.AbstractActionDefault#getToolBarComponents()
  * @return Returns an array of the JTool components.
  */
 public Component[] getToolComponents() {
  JComboBox combo = getToolBarComponent();
  combo.addActionListener(this);
  Object[] items = getItems();

  for (int i = 0; i < items.length; i++) {
   Object item = items[i];
   if (item != null)
    combo.addItem(item);
  }

  return new Container[] { combo };
 }
 
 /** Gets JComboBox for the current list.
  * @return Returns the JComboBox.
  */ 
 protected JComboBox getToolBarComponent() {
   JComboBox combo = new JComboBox();
   ListCellRenderer renderer = getItemListCellRenderer();
   if (renderer != null)
     combo.setRenderer(renderer);
   return combo;
 }
 
 /** Uses the specified action to either get the JComboBox or JMenuItem for the current list.
  * @param e This specifies the desired action that is taking place.
  * @return Returns an object of either type JComboBox or JMenuItem, depending on the action specified, and 
  * <tt>null</tt> is returned if parameter is of neither type.
  */ 
 protected Object getSelectedItem(ActionEvent e) {
   if (e.getSource() instanceof JComboBox) {
     return ((JComboBox) e.getSource()).getSelectedItem();
   } else if (e.getSource() instanceof JMenuItem) {
     return ((JMenuItem) e.getSource()).getClientProperty(ITEM_KEY);
   }
   return null;
 }
 
 /** Returns a JMenuItem with a link to this action.
  * @param actionCommand This String specifies the action that is taking place.
  * @param itemValue This Object holds the information for the item the action will take place on.
  * @return Returns a JMenuItem, which is set to have the properties set forth by the parameters.
  */
 protected Component getMenuComponent(String actionCommand, Object itemValue) {
   JMenuItem item = new JMenuItem(this);
   item.putClientProperty(ITEM_KEY, itemValue);
   
   GPBarFactory.fillAbstractButton(item,
                                   getName(),
                                   actionCommand);
   String presentationText = getPresentationText(actionCommand);
   if (presentationText != null)
     item.setText(presentationText);
   String presentation = getItemPresentationText(itemValue);
   if (presentation != null)
     item.setText(presentation);
   
   return item;
 }
 
 /** You should return a hashtable with the 
  *  possible items.
  */
 protected abstract Object[] getItems();
 
 
 /** Returns the List Cell Renderer for the
  *  Items (has an empty body for now). By default returns the 
  *  DefaultListCellRenderer.  
  * 
  * @see DefaultListCellRenderer
  * @return Simply returns <tt>null</tt> for now.
  */
 protected ListCellRenderer getItemListCellRenderer(){
   return null;
 };
 
 /** Returns the item presentation text 
  *  (buttonEdge.g. for the MenuItem)
  * 
  *  The default Implemenation returns
  *  <tt>item.toString()</tt>
  * 
  */
 protected String getItemPresentationText(Object item) {
   return item.toString();
 }
 
}
