
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;
import javax.swing.SwingUtilities;

/**
 * An abstract class that deals with a cell view manipulation.
 *
 * Referenced classes of package com.jgraph.graph are
 * GraphContext, CellHandle and GraphCelEditor.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see CellView
 * @see Serializable
 */

public abstract class AbstractCellView
implements CellView, Serializable {
    
    /**
     * An instance of GraphCellEditor.
     */
    protected static GraphCellEditor cellEditor = new DefaultGraphCellEditor();
    
    /**
     * An instance of JGraph.
     */
    protected JGraph graph;
    
    /**
     * An instance of CellMapper.
     */
    protected CellMapper mapper;
    
    /**
     * An instance of an Object.
     */
    protected Object cell;
    
    /**
     * An instance of CellView.
     */
    protected CellView parent;
    
    /**
     * An nstance of List.
     */
    protected java.util.List childViews;
    
    /**
     * An instance of Map.
     */
    protected Map attributes;
    
    /**
     * An instance of Map.
     */
    protected Map allAttributes;
    
    /**
     * Constructor.
     * @param obj A variable of Object data type
     * @param jgraph A variable of JGraph data type
     * @param cellmapper A variable of CellMapper data type
     * @see Object
     * @see JGraph
     * @see CellMapper
     */
    public AbstractCellView(Object obj, JGraph jgraph, CellMapper cellmapper) {
        graph = null;
        mapper = null;
        cell = null;
        parent = null;
        childViews = new ArrayList();
        attributes = GraphConstants.createMap();
        cell = obj;
        graph = jgraph;
        mapper = cellmapper;
        updateAllAttributes();
    }
    
    /**
     * Retrieves the current graph.
     * @return A JGraph data type.
     * @see JGraph
     */
    public JGraph getGraph() {
        return graph;
    }
    
    /**
     * Retrieves the current model.
     * @return A GraphModel data type.
     * @see GraphModel
     */
    public GraphModel getModel() {
        return graph.getModel();
    }
    
    /**
     * Retreives the current cell mapper.
     * @return A CellMapper data type.
     * @see CellMapper
     */
    public CellMapper getMapper() {
        return mapper;
    }
    
    /**
     * Retreives the current cell object.
     * @return An Object data type.
     * @see Object
     */
    public Object getCell() {
        return cell;
    }
    
    /**
     * Updates and refreshes the cell view.
     * @param flag A variable of boolean data type.
     * @see Object
     */
    public void refresh(boolean flag) {
        update();
        if(mapper != null && getModel() != null) {
            Object obj = getModel().getParent(cell);
            if(graph.getGraphLayoutCache().isVisible(obj)) {
                CellView cellview = mapper.getMapping(obj, flag);
                if(cellview != parent) {
                    removeFromParent();
                }
                parent = cellview;
            } else
                if(parent != null) {
                    removeFromParent();
                    parent = null;
                }
        }
        boolean flag1 = !graph.getGraphLayoutCache().isOrdered();
        if(flag1) {
            childViews.clear();
        }
        for(int i = 0; i < getModel().getChildCount(cell); i++) {
            Object obj1 = getModel().getChild(cell, i);
            if(graph.getGraphLayoutCache().isVisible(obj1)) {
                CellView cellview1 = mapper.getMapping(obj1, flag);
                if(cellview1 != null && !(obj1 instanceof Port) && (!childViews.contains(cellview1) || flag1)) {
                    childViews.add(cellview1);
                }
            }
        }
        
    }
    
    /**
     * Updates all the attributes including the child attributes.
     */
    public void update() {
        updateAllAttributes();
        childUpdated();
    }
    
    /**
     * Updates all the attributes of the graph.
     */
    protected void updateAllAttributes() {
        allAttributes = getModel().getAttributes(cell);
        if(allAttributes != null) {
            allAttributes = GraphConstants.cloneMap(allAttributes);
        } else {
            allAttributes = GraphConstants.createMap();
        }
        allAttributes.putAll(attributes);
    }
    
    /**
     * Updates the attributes of the child.
     */
    public void childUpdated() {
        if(parent != null) {
            parent.childUpdated();
        }
    }
    
    /**
     * Retrieves the parent view.
     * @return A CellView data type.
     * @see CellView
     */
    public CellView getParentView() {
        return parent;
    }
    
    /**
     * Retrieves all the child views.
     * @return An array of CellView data type.
     * @see CellView
     */
    public CellView[] getChildViews() {
        CellView acellview[] = new CellView[childViews.size()];
        childViews.toArray(acellview);
        return acellview;
    }
    
    /**
     * Retrieves all descenddant views.
     * @param acellview A variable of CellView data type.
     * @return An array of CellView data type.
     * @see CellView
     * @see Stack
     * @see ArrayList
     */
    public static CellView[] getDescendantViews(CellView acellview[]) {
        Stack stack = new Stack();
        for(int i = 0; i < acellview.length; i++) {
            stack.add(acellview[i]);
        }
        
        ArrayList arraylist = new ArrayList();
        CellView cellview;
        for(; !stack.isEmpty(); arraylist.add(cellview)) {
            cellview = (CellView)stack.pop();
            CellView acellview2[] = cellview.getChildViews();
            for(int j = 0; j < acellview2.length; j++) {
                stack.add(acellview2[j]);
            }
            
        }
        
        CellView acellview1[] = new CellView[arraylist.size()];
        arraylist.toArray(acellview1);
        return acellview1;
    }
    
    /**
     * Removes view from parent.
     */
    public void removeFromParent() {
        if(parent instanceof AbstractCellView) {
            java.util.List list = ((AbstractCellView)parent).childViews;
            list.remove(this);
        }
    }
    
    /**
     * Check if the given cellview is a leaf.
     * @return A boolean data type.
     */
    public boolean isLeaf() {
        return childViews.isEmpty();
    }
    
    /**
     * Retrives the attributes of a cell.
     * @return A Map data type
     * @see Map
     */
    public Map getAttributes() {
        return attributes;
    }
    
    /**
     * Retrieves all attributes of a cell.
     * @return A Map data type
     */
    public Map getAllAttributes() {
        return allAttributes;
    }
    
    /**
     * Set the attributes for a Map Object.
     * @param map A variable of Map data type.
     * @return A Map data type.
     * @see Map
     */
    public Map setAttributes(Map map) {
        Map map1 = GraphConstants.applyMap(map, attributes);
        update();
        return map1;
    }
    
    /**
     * Retrieves a Rectangle Object with its bounds.
     * @return A Rectangle data type.
     * @see Rectangle
     */
    public abstract Rectangle getBounds();
    
    /**
     * Retreives a set of bounds in the form of a Rectangle Object.
     * @param acellview A variable of array of CellView data type.
     * @return A Rectangle data type.
     * @see CellView
     * @see Rectangle
     */
    public static Rectangle getBounds(CellView acellview[]) {
        if(acellview != null && acellview.length > 0) {
            Rectangle rectangle = null;
            for(int i = 0; i < acellview.length; i++) {
                if(acellview[i] != null) {
                    Rectangle rectangle1 = acellview[i].getBounds();
                    if(rectangle1 != null) {
                        if(rectangle == null) {
                            rectangle = new Rectangle(rectangle1);
                        } else {
                            SwingUtilities.computeUnion(rectangle1.x, rectangle1.y, rectangle1.width, rectangle1.height, rectangle);
                        }
                    }
                }
            }
            
            return rectangle;
        } else {
            return null;
        }
    }
    
    /**
     * Sets the bounds of a cell view.
     * @param rectangle A variable of Rectangle data type.
     * @see Rectangle
     * @see Point
     * @see Dimension
     */
    public void setBounds(Rectangle rectangle) {
        Rectangle rectangle1 = getBounds();
        Point point = rectangle.getLocation();
        Point point1 = rectangle1.getLocation();
        Rectangle rectangle2 = new Rectangle(rectangle);
        boolean flag = GraphConstants.isMoveable(getAttributes());
        if(GraphConstants.isMoveable(getAttributes()) && !point.equals(point1)) {
            translate(point.x - point1.x, point.y - point1.y);
        } else {
            rectangle2.setSize((rectangle.width - point.x) + point1.x, (rectangle.height - point.y) + point1.y);
        }
        Dimension dimension = rectangle2.getSize();
        Dimension dimension1 = rectangle1.getSize();
        if(!dimension1.equals(dimension) && dimension1.width > 0 && dimension1.height > 0) {
            double d = (double)dimension.width / (double)dimension1.width;
            double d1 = (double)dimension.height / (double)dimension1.height;
            scale(d, d1, point);
        }
    }
    
    /**
     * Translate Object's attributes.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @see GraphConstants,
     * @see AbstractCellView
     */
    protected void translate(int i, int j) {
        if(isLeaf()) {
            GraphConstants.translate(getAttributes(), i, j);
        } else {
            for(Iterator iterator = childViews.iterator(); iterator.hasNext();) {
                Object obj = iterator.next();
                if(obj instanceof AbstractCellView) {
                    AbstractCellView abstractcellview = (AbstractCellView)obj;
                    abstractcellview.translate(i, j);
                }
            }
            
        }
    }
    
    /**
     * Scales Object to a certian size.
     * @param d A variable of double data type.
     * @param d1 A vairable of double data type.
     * @param point A variale of Point data type.
     * @see Point
     * @see GraphConstants
     * @see Iterator
     * @see AbstractCellView
     */
    protected void scale(double d, double d1, Point point) {
        if(isLeaf()) {
            GraphConstants.scale(getAttributes(), d, d1, point);
        } else {
            for(Iterator iterator = childViews.iterator(); iterator.hasNext();) {
                Object obj = iterator.next();
                if(obj instanceof AbstractCellView) {
                    AbstractCellView abstractcellview = (AbstractCellView)obj;
                    Map map = abstractcellview.getAttributes();
                    if(GraphConstants.isSizeable(map) || GraphConstants.isAutoSize(map)) {
                        abstractcellview.scale(d, d1, point);
                    }
                }
            }
            
        }
    }
    
    /**
     * Checks if the given graphics and rectangle intersect.
     * @param g A variable of Graphics data type.
     * @param rectangle A variable of Rectangle data type.
     * @return A boolean data type.
     * @see Graphics
     * @see Rectangle
     * @seeIterator
     * @see CellView
     */
    public boolean intersects(Graphics g, Rectangle rectangle) {
        if(isLeaf()) {
            Rectangle rectangle1 = getBounds();
            if(rectangle1 != null) {
                return rectangle1.intersects(rectangle);
            }
        } else {
            for(Iterator iterator = childViews.iterator(); iterator.hasNext();) {
                if(((CellView)iterator.next()).intersects(g, rectangle)) {
                    return true;
                }
            }
            
        }
        return false;
    }
    
    /**
     * Retrieves a RendererComponent Object.
     * @param jgraph A variable of JGraph data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @param flag2 A variable of boolean data type.
     * @return A Component data type.
     * @see JGraph
     * @see Component
     * @see CellViewRenderer
     */
    public Component getRendererComponent(JGraph jgraph, boolean flag, boolean flag1, boolean flag2) {
        CellViewRenderer cellviewrenderer = getRenderer();
        if(cellviewrenderer != null) {
            return cellviewrenderer.getRendererComponent(jgraph, this, flag, flag1, flag2);
        } else {
            return null;
        }
    }
    
    /**
     * Retrieves a CellViewRenderer Object.
     * @return A CellViewRenderer data type.
     * @see CellViewRenderer
     */
    protected abstract CellViewRenderer getRenderer();
    
    /**
     * Retrieves a CellHandle Object.
     * @param graphcontext A GraphContext data type.
     * @return A CellHandle data type.
     * @see GraphContext
     * @see CellHandle
     */
    public abstract CellHandle getHandle(GraphContext graphcontext);
    
    /**
     * Retrieves the GraphCellEditor Object.
     * @return A GraphCellEditor data type.
     * @see GraphCellEditor
     */
    public GraphCellEditor getEditor() {
        return cellEditor;
    }
    
}
