
import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import javax.swing.plaf.UIResource;

// Referenced classes of package com.jgraph.plaf.basic:
//            BasicGraphUI

/**
 * This is the BasicGraphDropTargetListener class. This class handles all the drags and drops that are done
 * on to the graphpad.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent.
 */

public class BasicGraphDropTargetListener
    implements DropTargetListener, UIResource, ActionListener
{

    /** This is the timer for the application. It stores information about when a drop was made to the graphpad.
     */
    private Timer timer;

    /** This stores the last point where a drop was made.
     */
    private Point lastPosition;

    /** The outer rectangle where the component is dropped. This is used to control the scrollable region
     *  of the component that visible on the application.
     */
    private Rectangle outer;

    /** 
     * The inner rectangle where the component is dropped.This is used to control the scrollable region
     *  of the component that visible on the application.
     */
    private Rectangle inner;

    /** 
     * The private variable hysteresis.
     * NOTE: The use of this variable is not very well defined. It appears that this variable is used to 
     * determine the minimum distance that a component has moved to consider the move as another drop.
     */
    private int hysteresis;

    /** 
     * Determines whether the component that is being dropped can be imported to the graphpad.
     */
    private boolean canImport;

    /** This stores that component that is being dropped.
     */
    private JComponent component;

    /** Creates a new BasicGraphDropTargetListener object. This object listens for event fired
     * when a component is dropped on the graph.
     */
    public BasicGraphDropTargetListener()
    {
        outer = new Rectangle();
        inner = new Rectangle();
        hysteresis = 10;
    }

    /**
     * Saves the state of the component that is specified.
     *
     * @param jcomponent This is the JComponent of which you want to save the state.
     */
    protected void saveComponentState(JComponent jcomponent)
    {
    }

    /**
     * Restores the state of the component specified.
     *
     * @param jcomponent This is the JComponent of which you want to restore the state.
     */
    protected void restoreComponentState(JComponent jcomponent)
    {
    }

    /**
     * Restores the state to drop this component.
     *
     * @param jcomponent This is the JComponent of which you want to restore the state for drop.
     */
    protected void restoreComponentStateForDrop(JComponent jcomponent)
    {
    }

    /**
     * Updates the location where a component in inserted on the graph.
     *
     * @param jcomponent The component that is inserted.
     * @param point The new location for the component.
     */
    protected void updateInsertionLocation(JComponent jcomponent, Point point)
    {
    }

    /**
     * Updates the region of the specified component that is scrollable automatically.
     *
     * @param jcomponent The component whose scrollable region you intend to modify.
     */
    void updateAutoscrollRegion(JComponent jcomponent)
    {
        Rectangle rectangle = jcomponent.getVisibleRect();
        outer.setBounds(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        Insets insets = new Insets(0, 0, 0, 0);
        if(jcomponent instanceof Scrollable)
        {
            Scrollable scrollable = (Scrollable)jcomponent;
            insets.left = scrollable.getScrollableUnitIncrement(rectangle, 0, 1);
            insets.top = scrollable.getScrollableUnitIncrement(rectangle, 1, 1);
            insets.right = scrollable.getScrollableUnitIncrement(rectangle, 0, -1);
            insets.bottom = scrollable.getScrollableUnitIncrement(rectangle, 1, -1);
        }
        inner.setBounds(rectangle.x + insets.left, rectangle.y + insets.top, rectangle.width - (insets.left + insets.right), rectangle.height - (insets.top + insets.bottom));
    }

    /**
     * Sets the auto-scrollable region of the component.
     *
     * @param jcomponent The component whose scrollable region you intend to modify.
     * @param point This is the second parameter to the BasicGraphUI.autoscroll method. It sets the current position of
     * the scroll bar.
     */
    void autoscroll(JComponent jcomponent, Point point)
    {
        if(jcomponent instanceof JGraph)
        {
            BasicGraphUI.autoscroll((JGraph)jcomponent, point);
        }
    }

    /**
     * Initializes the properties of the listener.
     */
    private void initPropertiesIfNecessary()
    {
        if(timer == null)
        {
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            Integer integer = new Integer(100);
            Integer integer1 = new Integer(100);
            try
            {
                integer = (Integer)toolkit.getDesktopProperty("DnD.Autoscroll.initialDelay");
            }
            catch(Exception exception) { }
            try
            {
                integer1 = (Integer)toolkit.getDesktopProperty("DnD.Autoscroll.interval");
            }
            catch(Exception exception1) { }
            timer = new Timer(integer1.intValue(), this);
            timer.setCoalesce(true);
            timer.setInitialDelay(integer.intValue());
            try
            {
                hysteresis = ((Integer)toolkit.getDesktopProperty("DnD.Autoscroll.cursorHysteresis")).intValue();
            }
            catch(Exception exception2) { }
        }
    }

    /**
     * Returns the component that raised the event specified in the parameter.
     *
     * @param droptargetevent The event that was raised by a component on the graph.
     * @return Returns the associated JComponent.
     */
    static JComponent getComponent(DropTargetEvent droptargetevent)
    {
        DropTargetContext droptargetcontext = droptargetevent.getDropTargetContext();
        return (JComponent)droptargetcontext.getComponent();
    }

    /**
     * This method is called after an action associated with actionevent is performed. It updates the auto-scroll region
     * for this component.
     *
     * @param actionevent The event raised when an action was performed.
     */
    public synchronized void actionPerformed(ActionEvent actionevent)
    {
        updateAutoscrollRegion(component);
        if(outer.contains(lastPosition) && !inner.contains(lastPosition))
        {
            autoscroll(component, lastPosition);
        }
    }

    /** This method is invoked when a component is dragged onto the graph pad.
     *
     * @param droptargetdragevent This is the object that is being dragged.
     */
    public void dragEnter(DropTargetDragEvent droptargetdragevent)
    {
        component = getComponent(droptargetdragevent);
        TransferHandler transferhandler = component.getTransferHandler();
        canImport = transferhandler.canImport(component, droptargetdragevent.getCurrentDataFlavors());
        if(canImport)
        {
            saveComponentState(component);
            lastPosition = droptargetdragevent.getLocation();
            updateAutoscrollRegion(component);
            initPropertiesIfNecessary();
        }
    }

    /** This method is invoked when a component is dragged over another.
     *
     * @param droptargetdragevent This is the object that is being dragged.
     * */
    public void dragOver(DropTargetDragEvent droptargetdragevent)
    {
        if(canImport)
        {
            Point point = droptargetdragevent.getLocation();
            updateInsertionLocation(component, point);
            synchronized(this)
            {
                if(Math.abs(point.x - lastPosition.x) > hysteresis || Math.abs(point.y - lastPosition.y) > hysteresis)
                {
                    if(timer.isRunning())
                    {
                        timer.stop();
                    }
                } else
                if(!timer.isRunning())
                {
                    timer.start();
                }
                lastPosition = point;
            }
        }
    }

    /**
     * This method is called after the target has been dropped. It cleans the graph and resets the state of the components.
     *
     * @param droptargetevent This is the event in which the target is dropped.
     */
    public void dragExit(DropTargetEvent droptargetevent)
    {
        if(canImport)
        {
            restoreComponentState(component);
        }
        cleanup();
    }

    /**
     * This method restores the state of the component for a drop. It also cleans the graph.
     *
     * @param droptargetdropevent This is the event in which the target is dropped.
     */
    public void drop(DropTargetDropEvent droptargetdropevent)
    {
        if(canImport)
        {
            restoreComponentStateForDrop(component);
        }
        cleanup();
    }

    /**
     * NOTE: THIS METHOD DOES NOT HAVE ANY IMPLEMENTATION.
     *
     * @param droptargetdragevent This is a DropTargetDragEvent.
     */
    public void dropActionChanged(DropTargetDragEvent droptargetdragevent)
    {
    }

    /** Performs all the cleanup after a drop has been completed. This includes setting the timer to zero,
     * the dropped component to null and the last position of the drop to null.
     */
    private void cleanup()
    {
        if(timer != null)
        {
            timer.stop();
        }
        component = null;
        lastPosition = null;
    }
}
