
import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

/**
 * This class implements a marquee handler.
 * 
 * Referenced classes of package com.jgraph.graph are
 * GraphLayoutCache and CellView.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */

public class BasicMarqueeHandler
{
  
  /**
   * An instance of Cursor.
   * Pointer to the previous cursor.
   */    
  protected transient Cursor previousCursor;
  
  /**
   * An instance of Rectangle.
   * To hold the bounds.
   */    
  protected Rectangle marqueeBounds;
  
  /**
   * An instance of Point.
   * Represents starting point.
   */    
  protected Point startPoint;
  
  /**
   * An instance of Point.
   * Represents current point.
   */    
  protected Point currentPoint;
  
  /** 
   * Default Constructor.
   */    
  public BasicMarqueeHandler()
  {
    previousCursor = null;
  }
  
  /**
   * Checks if the mouseevent is a marqueeevent.
   * @param mouseevent A variable of MouseEvent data type.
   * @return A boolean data type.
   * @see MouseEvent
   */    
  public boolean isForceMarqueeEvent(MouseEvent mouseevent)
  {
    return mouseevent.isAltDown();
  }
  
  /**
   * A specific action when mouse released is recognized.
   * @param mouseevent - The mouseevent in question.
   * @see MouseEvent
   * @see JGraph
   * @see Rectangle
   * @see CelView
   * @see ArrayList
   * @see Object
   */    
  public void mouseReleased(MouseEvent mouseevent) {
    
    try {
      if(mouseevent != null && !mouseevent.isConsumed() && marqueeBounds != null)
      {
        if(!(mouseevent.getSource() instanceof JGraph))
        {
          throw new IllegalArgumentException("MarqueeHandler cannot handle event from unknown source: " + mouseevent);
        }
        JGraph jgraph = (JGraph)mouseevent.getSource();
        Rectangle rectangle = jgraph.fromScreen(new Rectangle(marqueeBounds));
        CellView acellview[] = jgraph.getGraphLayoutCache().getRoots(rectangle);
        ArrayList arraylist = new ArrayList();
        for(int i = 0; i < acellview.length; i++)
        {
          if(rectangle.contains(acellview[i].getBounds()))
          {
            arraylist.add(acellview[i].getCell());
          }
        }
        
        Object aobj[] = arraylist.toArray();
        jgraph.getUI().selectCellsForEvent(jgraph, aobj, mouseevent);
        jgraph.setCursor(previousCursor);
        Rectangle rectangle1 = new Rectangle(marqueeBounds);
        rectangle1.width++;
        rectangle1.height++;
        jgraph.repaint(rectangle1);
        mouseevent.consume();
      }
    } finally {
      currentPoint = null;
      startPoint = null;
      marqueeBounds = null;
      previousCursor = null;
    }
  }
  
  /**
   * A specific action when mouse clicked and dragged is recognized.
   * @param mouseevent A variable of MouseEvent data type.
   * @see MouseEvent
   * @see JGraph
   * @see Graphics
   */    
  public void mouseDragged(MouseEvent mouseevent)
  {
    if(!mouseevent.isConsumed() && startPoint != null)
    {
      if(!(mouseevent.getSource() instanceof JGraph))
      {
        throw new IllegalArgumentException("MarqueeHandler cannot handle event from unknown source: " + mouseevent);
      }
      JGraph jgraph = (JGraph)mouseevent.getSource();
      Graphics g = jgraph.getGraphics();
      java.awt.Color color = jgraph.getBackground();
      java.awt.Color color1 = jgraph.getMarqueeColor();
      g.setColor(color1);
      g.setXORMode(color);
      overlay(g);
      currentPoint = mouseevent.getPoint();
      marqueeBounds = new Rectangle(startPoint);
      marqueeBounds.add(currentPoint);
      g.setColor(color);
      g.setXORMode(color1);
      overlay(g);
      mouseevent.consume();
    }
  }
  
  /**
   * Paints an Object.
   * @param g A variable of Graphics data type.
   * @see Graphics
   */    
  public void paint(Graphics g)
  {
    overlay(g);
  }
  
  /**
   * Draws a rectangle.
   * @param g A variable of Graphics data type.
   */    
  protected void overlay(Graphics g)
  {
    if(marqueeBounds != null)
    {
      g.drawRect(marqueeBounds.x, marqueeBounds.y, marqueeBounds.width, marqueeBounds.height);
    }
  }
  
  /** 
   * A specific action when mouse clicked is recognized.
   * @param mouseevent A variable of MouseEvent data type.
   * @see MouseEvent
   * @see JGraph
   */    
  public void mousePressed(MouseEvent mouseevent)
  {
    if(!mouseevent.isConsumed())
    {
      if(!(mouseevent.getSource() instanceof JGraph))
      {
        throw new IllegalArgumentException("MarqueeHandler cannot handle event from unknown source: " + mouseevent);
      }
      JGraph jgraph = (JGraph)mouseevent.getSource();
      startPoint = mouseevent.getPoint();
      marqueeBounds = new Rectangle(startPoint);
      previousCursor = jgraph.getCursor();
      jgraph.setCursor(new Cursor(1));
      mouseevent.consume();
    }
  }
  
  /** 
   * A specific action when mouse moved is recognized.
   * @param mouseevent A variable of MouseEvent data type.
   */    
  public void mouseMoved(MouseEvent mouseevent)
  {  
  }
  
  /**
   * Retrieves a current point.
   * @return A Point data type.
   * @see Point
   */    
  public Point getCurrentPoint()
  {
    return currentPoint;
  }
  
  /**
   * Retreives a Rectangle with the bounds.
   * @return A Rectangle data type.
   * @see Rectangle
   */    
  public Rectangle getMarqueeBounds()
  {
    return marqueeBounds;
  }
  
  /**
   * Retreives a previous cursor.
   * @return A Cursor data type.
   * @see Cursor
   */    
  public Cursor getPreviousCursor()
  {
    return previousCursor;
  }
  
  /**
   * Retrieves a starting point.
   * @return A Point data type.
   * @see Point
   */    
  public Point getStartPoint()
  {
    return startPoint;
  }
  
  /**
   * Sets a current point.
   * @param point A variable of Point data type.
   * @see Point
   */    
  public void setCurrentPoint(Point point)
  {
    currentPoint = point;
  }
  
  /**
   * Sets marquee bounds.
   * @param rectangle A variable of Rectangle data type.
   * @see Rectangle
   */    
  public void setMarqueeBounds(Rectangle rectangle)
  {
    marqueeBounds = rectangle;
  }
  
  /**
   * Sets a previous cursor.
   * @param cursor A variable of Cursor data type.
   * @see Cursor
   */    
  public void setPreviousCursor(Cursor cursor)
  {
    previousCursor = cursor;
  }
  
  /**
   * Sets a Starting point.
   * @param point A variable of Point data type.
   * @see Point
   */    
  public void setStartPoint(Point point)
  {
    startPoint = point;
  }
}
