import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.EventObject;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.CellEditorListener;
import javax.swing.plaf.FontUIResource;


/** 
 * This class handles default graph cell editting.
 *
 * Referenced classes of package com.jgraph.graph are
 * GraphCellEditor, GraphLayoutCache, CellView, GraphConstants
 * and DefaultRealEditor.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see CellView
 * @see ActionListener 
 * @see GraphCellEditor
 * @see GraphSelectionListener
 * @ee Serializable
 */

public class DefaultGraphCellEditor
    implements ActionListener, GraphCellEditor, GraphSelectionListener, Serializable
{
  
 /**
  * Container responsible for placing the editingComponent.
  */
  public class EditorContainer extends Container
  {
      
    /**
     * Constructs an EditorContainer object.
     */
//    public EditorContainer()
//    {
//      setLayout(null);
//    }
    
       
    /**
     * Overrides Container.paint to paint the node's
     * icon and use the selection color for the background.
     * @param g A variable of Graphics data type.
     * @see Graphics
     */
    public void paint(Graphics g)
        {
      Dimension dimension = getSize();
      // Then the icon.
      if(editingIcon != null)
      {
        int i = 0;
        int j = 0;
        editingIcon.paintIcon(this, g, j, i);
      }
      Color color = getBorderSelectionColor();
      if(color != null)
      {
        g.setColor(color);
        g.drawRect(0, 0, dimension.width - 1, dimension.height - 1);
      }
      super.paint(g);
    }
    
    /**
     * Lays out Container.
     * The editor will be placed at offset in the x direction and 0 for y if editing.
     * see Dimension
     */
    public void doLayout()
    {
      if(editingComponent != null)
      {
        Dimension dimension = getSize();
        int i = (int)editingComponent.getPreferredSize().getHeight();
        editingComponent.setBounds(offsetX, offsetY, dimension.width - offsetX, i);
      }
    }
    
    /**
     * Retrieves preferred size for Container.  This will be
     * the preferred size of the editor offset by offset.
     * @return A Dimension data type.
     * @see Dimension
     * @see Object
     */
    public Dimension getPreferredSize()
    {
      if(editingComponent != null)
      {
        Dimension dimension = editingComponent.getPreferredSize();
        Object obj = null;
        dimension.width += offsetX + 2;
        dimension.height += offsetY + 2;
        int i = 50;
        if(editingIcon != null)
        {
          i = Math.max(editingIcon.getIconWidth(), i);
        }
        dimension.height = Math.max(dimension.height, 24);
        dimension.width = Math.max(dimension.width, i);
        return dimension;
      } else
      {
        return new Dimension(0, 0);
      }
    }

    /**
     * Constructs an EditorContainer object.
     */
    public EditorContainer()
    {
      setLayout(null);
    }
  }

  /**
   * TextField used when no editor is supplied. This textfield locks into
   * the border it is constructed with. It also prefers its parents
   * font over its font. And if the renderer is not null and no font
   * has been specified the preferred height is that of the renderer.
   * @see JTextField
   */
  public class DefaultTextField extends JTextField
  {
    /**
     * An instance of Border.
     */
    protected Border border;
    
    /**
     * Overrides JComponent.getBorder.
     * @return A Border data type.
     * @see Border
     */
    public Border getBorder()
    {
      return border;
    }
    
    /**
     * Retrieves a font.
     * @return A Font data type.
     * @see Font
     */
    public Font getFont()
    {
      Font font1 = super.getFont();
      if(font1 instanceof FontUIResource)
      {
        Container container = getParent();
        if(container != null && container.getFont() != null)
        {
          font1 = container.getFont();
        }
      }
      return font1;
    }


    /** 
     * Constructs a DefaultTreeCellEditor$DefaultTextField object.
     * @param border1 A Border data type.
     * @see Border
     */
    public DefaultTextField(Border border1)
    {
      border = border1;
    }
  }


    /**
     * An instance of GraphCellEditor.
     */    
    protected GraphCellEditor realEditor;
    
    /**
     * An instance of Container.
     */  
    protected Container editingContainer;
    
    /**
     * An instance of Component.
     */
    protected transient Component editingComponent;
    
    /**
     * An instance of boolean.
     */
    protected boolean canEdit;
    
    /**
     * An instance of integer.
     */
    protected transient int offsetX;
    
    /**
     * An instance of integer.
     */    
    protected transient int offsetY;
    
    /**
     * An instance of JGraph.
     */
    protected transient JGraph graph;
    
    /**
     * An instance of Object.
     */
    protected transient Object lastCell;
    
    /**
     * An instance of Color.
     */
    protected Color borderSelectionColor;
    
    /**
     * An instance of Icon.
     */
    protected transient Icon editingIcon;
    
    /**
     * An instance of Font.
     */
    protected Font font;

    /** 
     * Default constructor.
     * Initializing current object to null.
     */    
    public DefaultGraphCellEditor()
    {
        this(null);
    }
    
    /** 
     * Constructor with one parameter
     * Initializes with parameter GraphCellEditor object.
     * @param graphcelleditor GraphCellEditor type, sets the realEditor to the graphcelleditor
     */    
    public DefaultGraphCellEditor(GraphCellEditor graphcelleditor)
    {
      realEditor = graphcelleditor;
      if(realEditor == null)
      {
        realEditor = createGraphCellEditor();
      }
      editingContainer = createContainer();
      setBorderSelectionColor(UIManager.getColor("Tree.editorBorderSelectionColor"));
    }

    /**
     * Sets a border selection color to the input.
     * @param A variable of Color data type.
     * @see Color
     */
    public void setBorderSelectionColor(Color color)
    {
        borderSelectionColor = color;
    }

    /**
     * Gets a border selection color.
     * @return A Color data type.
     * @see Color
     */
    public Color getBorderSelectionColor()
    {
        return borderSelectionColor;
    }

    /**
     * Sets a font to an input parameter.
     * @param font1 A variable of Font data type.
     * @see Font
     */
    public void setFont(Font font1)
    {
        font = font1;
    }

    /**
     * Gets current font.
     * @return A Font data data type.
     * @see Font
     */
    public Font getFont()
    {
        return font;
    }

    /**
     * Gets a graph cell editor component based on input parameters.
     * @param jgraph A variable of JGraph data type.
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @return A Component data type.
     * @see JGraph
     * @see Object
     * @see Component
     */
    public Component getGraphCellEditorComponent(JGraph jgraph, Object obj, boolean flag)
    {
        String s = jgraph.convertValueToString(obj);
        setGraph(jgraph);
        editingComponent = realEditor.getGraphCellEditorComponent(jgraph, s, flag);
        determineOffset(jgraph, obj, flag);
        canEdit = lastCell != null && obj != null && lastCell.equals(obj);
        CellView cellview = jgraph.getGraphLayoutCache().getMapping(obj, false);
        if(cellview != null)
        {
            setFont(GraphConstants.getFont(cellview.getAllAttributes()));
        }
        editingContainer.setFont(font);
        return editingContainer;
    }

    /**
     * Retrieves a cell editor value.
     * @return An Object data type.
     * @see Object
     */
    public Object getCellEditorValue()
    {
        return realEditor.getCellEditorValue();
    }

    /**
     * Checks whether or not a cell is editable.
     * @param eventobject A variable of EventObject data type.
     * @return A boolean data type.
     * @see EventObject
     */
    public boolean isCellEditable(EventObject eventobject)
    {
      boolean flag = false;
      if(!realEditor.isCellEditable(eventobject))
      {
        return false;
      }
      if(canEditImmediately(eventobject))
      {
        flag = true;
      }
      if(flag)
      {
        prepareForEditing();
      }
      return flag;
    }

    /**
     * Checks whether or not a cell should be selected.
     * @param eventobject A variable of EventObject data type.
     * @return A boolean data type.
     * @see EventObject
     */
    public boolean shouldSelectCell(EventObject eventobject)
    {
      return realEditor.shouldSelectCell(eventobject);
    }

    /**
     * Checks if a cell editing needs to be stopped.
     * @return A boolean data type.
     */    
    public boolean stopCellEditing()
    {
        if(realEditor.stopCellEditing())
        {
            if(editingComponent != null)
            {
                editingContainer.remove(editingComponent);
            }
            editingComponent = null;
            return true;
        } else
        {
            return false;
        }
    }

    /**
     * Cancels a cell editing.
     */
    public void cancelCellEditing()
    {
      realEditor.cancelCellEditing();
      if(editingComponent != null)
      {
        editingContainer.remove(editingComponent);
      }
      editingComponent = null;
    }

    /**
     * Adds a cell editor listener.
     * @param celleditorlistener A variable of CellEditorListener data type.
     * @see CellEditorListener
     */
    public void addCellEditorListener(CellEditorListener celleditorlistener)
    {
        realEditor.addCellEditorListener(celleditorlistener);
    }

    /**
     * Removes a cell editor listener.
     * @param celleditorlistener A variable of CellEditorListener data type.
     * @see CellEditorListener
     */
    public void removeCellEditorListener(CellEditorListener celleditorlistener)
    {
        realEditor.removeCellEditorListener(celleditorlistener);
    }

    /**
     * To check if graph selection count is 1 and sets the lastCell variable.
     * @param graphselectionevent A variable of GraphSelectionEvent data type.
     * @see GraphSelectionEvent
     */
    public void valueChanged(GraphSelectionEvent graphselectionevent)
    {
        if(graph != null)
        {
            if(graph.getSelectionCount() == 1)
            {
                lastCell = graph.getSelectionCell();
            } else
            {
                lastCell = null;
            }
        }
    }

    /**
     * To check if an action has been performed.
     * @param actionevent A variable of ActionEvent data type.
     * @see ActionEvent
     */
    public void actionPerformed(ActionEvent actionevent)
    {
      if(graph != null)
      {
        graph.startEditingAtCell(lastCell);
      }
    }

    /**
     * Sets current graph to an input jgraph.
     * @param jgraph A variable of JGraph data type.
     * @see JGraph
     */
    protected void setGraph(JGraph jgraph)
    {
      if(graph != jgraph)
      {
        if(graph != null)
        {
          graph.removeGraphSelectionListener(this);
        }
        graph = jgraph;
        if(graph != null)
        {
          graph.addGraphSelectionListener(this);
        }
      }
    }
    
    /**
     * Indicates whether or not to start editing.
     * @param eventobject A variable of EventObject data type.
     * @return A boolean data type.
     * @see EventObject
     */
    protected boolean shouldStartEditingTimer(EventObject eventobject)
    {
      if((eventobject instanceof MouseEvent) && SwingUtilities.isLeftMouseButton((MouseEvent)eventobject))
      {
        MouseEvent mouseevent = (MouseEvent)eventobject;
        return mouseevent.getClickCount() == 1 && inHitRegion(mouseevent.getX(), mouseevent.getY());
      } else
      {
        return false;
      }
    }
    
    /**
     * Indicates if edits can be made.
     * @param eventobject A variable of EventObject data type.
     * @return A boolean data type.
     * @see EventObject
     */
    protected boolean canEditImmediately(EventObject eventobject)
    {
      if((eventobject instanceof MouseEvent) && SwingUtilities.isLeftMouseButton((MouseEvent)eventobject))
      {
        MouseEvent mouseevent = (MouseEvent)eventobject;
        return mouseevent.getClickCount() == 1 && inHitRegion(mouseevent.getX(), mouseevent.getY());
      } else
      {
        return eventobject == null;
      }
    }
    
    /**
     * To check if two integers are in a hit region.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @return A boolean data type.
     */
    protected boolean inHitRegion(int i, int j)
    {
      if(lastCell != null && graph != null)
      {
        Rectangle rectangle = graph.getCellBounds(lastCell);
        if(rectangle != null && i <= rectangle.x + offsetX && j <= rectangle.y + offsetY && offsetX < rectangle.width - 5)
        {
          return false;
        }
      }
      return true;
    }
    
    /**
     * Determines an offset based on handle size.
     * @param jgraph A variable of JGraph data type.
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @see JGraph
     * @see Object
     */
    protected void determineOffset(JGraph jgraph, Object obj, boolean flag)
    {
      editingIcon = null;
      offsetX = jgraph.getHandleSize();
      offsetY = jgraph.getHandleSize();
    }
    
    /**
     * Prepares a graph for editing.
     */
    protected void prepareForEditing()
    {
      editingContainer.add(editingComponent);
    }
    
    /**
     * Creates a container.
     * @return A Container data type.
     * @see Container
     */
    protected Container createContainer()
    {
      return new EditorContainer();
    }
    
    /**
     * Creates a graph cell editor.
     * @return A GraphCellEditor data type.
     * @see GraphCellEditor
     */
    protected GraphCellEditor createGraphCellEditor()
    {
      Border border = UIManager.getBorder("Tree.editorBorder");
      DefaultRealEditor defaultrealeditor = new DefaultRealEditor(new DefaultTextField(border)) {
        
        public boolean shouldSelectCell(EventObject eventobject)
        {
          boolean flag = super.shouldSelectCell(eventobject);
          getComponent().requestFocus();
          return flag;
        }
        
      };
      defaultrealeditor.setClickCountToStart(1);
      return defaultrealeditor;
    }
    
    /**
     * Writes an object to an output stream.
     * @param objectoutputstream A variable of ObjectOutputStream data type.
     * @throws IOException if invalid.
     * @see ObjectOutputStream
     * @see IOException
     * @see Vector
     */
    private void writeObject(ObjectOutputStream objectoutputstream)
      throws IOException
    {
      Vector vector = new Vector();
      objectoutputstream.defaultWriteObject();
      if(realEditor instanceof Serializable)
      {
        vector.addElement("realEditor");
        vector.addElement(realEditor);
      }
      objectoutputstream.writeObject(vector);
    }
    
    /**
     * Reads an object from an input stream.
     * @param objectinputstream A variable of ObjectInputStream data type.
     * @throws IOException if invalid.
     * @throws ClassNotFoundException if class is not foud.
     * @see ObjectInputStream
     * @see IOException
     * @see ClassNotFoundException
     */
    private void readObject(ObjectInputStream objectinputstream)
      throws IOException, ClassNotFoundException
    {
      objectinputstream.defaultReadObject();
      Vector vector = (Vector)objectinputstream.readObject();
      int i = 0;
      int j = vector.size();
      if(i < j && vector.elementAt(i).equals("realEditor"))
      {
        realEditor = (GraphCellEditor)vector.elementAt(++i);
        i++;
      }
    }
}
