
import java.io.*;
import java.util.*;
import javax.swing.event.EventListenerList;
import javax.swing.tree.*;
import javax.swing.undo.*;

/**
 * This class extends UndoableEditSupport to implement a graph model.
 *
 * Referenced class of package com.jgraph.graph are
 * GraphCell, Edge, Port, ParentMap, ConnectionSet, GraphModel,
 * DefaultGraphCell, GraphLayoutCache and CellView.
 * 
 * @author Gaudenz Alder
 * @version 1. 0 1/1/02
 */

public class DefaultGraphModel extends UndoableEditSupport
  implements Serializable, GraphModel
{
  /** 
   * Implementing an empty iterator.
   */    
  public static class EmptyIterator
    implements Iterator, Serializable
  {
    
    /**
     * Checks if an iterator has next.
     * @return A boolean data type.
     */        
    public boolean hasNext()
    {
      return false;
    }
    
    /**
     * Retrieves next.
     * @return A boolean data type.
     */        
    public Object next()
    {
      return null;
    }
    
    /**
     * Removes an Object.
     */        
    public void remove()
    {
    }
    
    /** 
     * Default Constructor.
     */        
    public EmptyIterator()
    {
    }
  }
  
  /** 
   * An implementation of GraphViewChange.
   */
  public class GraphModelLayerEdit extends GraphLayoutCache.GraphViewLayerEdit
    implements GraphModelEvent.GraphModelChange
  {
    
    /** 
     * An instance of Object.
     * The cells that change are the parents, because they need to
     * reload their childs for reordering.
     */        
    protected Object parents[];
    
    /**
     * Returns a source of this change. This can either be a
     * view or a model, if this change is a GraphModelChange.
     * @return An Object data type.
     * @see Object
     */
    public Object getSource()
    {
      return DefaultGraphModel.this;
    }
    
    /**
     * Retrieve Objects that were changed.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getChanged()
    {
      return parents;
    }
    
    /** 
     * Retrieve Objects that were inserted.
     * @return An Object[] data type.
     * @see Object
     */        
    public Object[] getInserted()
    {
      return null;
    }
    
    /**
     * Retrieve Objects that were removed.
     * @return An Object[] data type.
     * @see Object
     */           
    public Object[] getRemoved()
    {
      return null;
    }
    
    /** 
     * Retrieves attributes of a previous Object.
     * @return A Map data type.
     * @see Map
     */           
    public Map getPreviousAttributes()
    {
      return null;
    }
    
    /**
     * Adds implicitEdit to an Object.
     * @param A variable of UndoableEdit data type.
     * @see UndoableEdit
     */
    public void addImplicitEdit(UndoableEdit undoableedit)
    {
    }
    
    /** 
     * Retrieve views from an Object.
     * @param g A variable of GraphLayoutCache data type.
     * @return A CellView[] data type.
     * @see GraphLayoutCache
     * @see CellView
     */   
    public CellView[] getViews(GraphLayoutCache g)
    {
      return null;
    }
    
    /** 
     * Sets views to an Object.
     * @param g A variable of GraphLayoutCache data type.
     * @param a A variable of CellView data type.
     * @see GraphLayoutCache
     * @see CellView
     */   
    public void putViews(GraphLayoutCache g, CellView a[])
    {
    }
    
    /**
     * Updates listeners using current object in scope.
     */
    protected void updateListeners()
    {
      fireGraphChanged(DefaultGraphModel.this, this);
    }
    
    /**
     * Returns a List that exclusively contains view.
     * @param obj A variable of Object data type.
     * @return A List data type.
     * @see Object
     */
    protected List getParentList(Object obj)
    {
      List list = null;
      if(obj instanceof DefaultMutableTreeNode)
      {
        TreeNode treenode = ((DefaultMutableTreeNode)obj).getParent();
        if(treenode instanceof DefaultGraphCell)
        {
          list = ((DefaultGraphCell)treenode).getChildren();
        } else
        {
          list = roots;
        }
      }
      return list;
    }
    
    /**
     * Constructs a GraphModelEdit. This modifies the order of the cells in the model.
     * @param aobj A variable of Object data type.
     * @param i A variable of integer data type.
     * @see Object
     */
    public GraphModelLayerEdit(Object aobj[], int i)
    {
      super(DefaultGraphModel.this, aobj, i);
      HashSet hashset = new HashSet();
      for(int j = 0; j < aobj.length; j++)
      {
        if(aobj[j] instanceof TreeNode)
        {
          hashset.add(((TreeNode)aobj[j]).getParent());
        }
      }
      
      parents = hashset.toArray();
    }
  }
  
  /** 
   * An implementation of GraphModelChange that can be added to the model event.
   */
  public class GraphModelEdit extends CompoundEdit
    implements GraphModelEvent.GraphModelChange, GraphModelEvent.ExecutableGraphChange
  {
    
    /** 
     * An instance of Object.
     * Cells that were inserted during the last execution. 
     */
    protected Object insert[];
    
    /** 
     * An instance of Object.
     * Cells that were changed during the last execution. 
     */
    protected Object changed[];
    
    /** 
     * An instance of Object.
     * Cells that were removed during the last execution. 
     */
    protected Object remove[];
    
    /** 
     * An instance of Object.
     * Cells that were inserted/removed/changed during the last execution. 
     */
    protected Object context[];
    
    /** 
     * An instance of Object.
     * Cells that were inserted during the last execution. 
     */
    protected Object inserted[];
    
    /** 
     * An instance of Object.
     * Cells that were removed during the last execution. 
     */
    protected Object removed[];
    
    /** 
     * An instance of Map.
     * Property map for the next execution. Attribute Map is
     * passed to the views on inserts. 
     */
    protected Map attributes;
    
    /** 
     * An instance of Map.
     * Property map for the next execution. Attribute Map is
     * passed to the views on inserts. 
     */
    protected Map previousAttributes;
    
    /** 
     * An instance of parentMap.
     * Parent map for the next execution. 
     */
    protected ParentMap parentMap;
    
    /** 
     * An instance of ConnectionSet.
     * ConnectionSet for the next execution. 
     */
    protected ConnectionSet connectionSet;
    
    /** 
     * An instance of Map.
     */
    protected Map cellViews;
    
    /**
     * Adds the groups that become empty to the cells that
     * will be removed. (Auto remove empty cells.) Removed
     * cells will be re-inserted on undo, and the parent child 
     * relations will be restored.
     * @param aobj A variable of Object data type.
     * @see Object
     */
    protected void handleEmptyGroups(Object aobj[])
    {
      if(aobj != null && aobj.length > 0)
      {
        if(remove == null)
        {
          remove = new Object[0];
        }
        Object aobj1[] = new Object[remove.length + aobj.length];
        System.arraycopy(((Object) (remove)), 0, ((Object) (aobj1)), 0, remove.length);
        System.arraycopy(((Object) (aobj)), 0, ((Object) (aobj1)), remove.length, aobj.length);
        remove = aobj1;
      }
    }
    
    /** 
     * Checks if an Object is significant.
     * @return A boolean data type.
     */        
    public boolean isSignificant()
    {
      return true;
    }
    
    /**
     * Returns a source of this change. This can either be a
     * view or a model, if this change is a GraphModelChange.
     * @return An Object data type.
     * @see Object
     */
    public Object getSource()
    {
      return DefaultGraphModel.this;
    }
    
    /**
     * Retrieve cells that have changed. This includes the cells
     * that have been changed through a call to getAttributes and the
     * edges that have been changed with the ConnectionSet.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getChanged()
    {
      return changed;
    }
    
    /**
     * Retrieve  objects that have not changed explicitly, but
     * implicitly because one of their dependent cells has changed.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getContext()
    {
      return context;
    }
    
    /**
     * Retrive cells that were inserted.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getInserted()
    {
      return inserted;
    }
    
    /**
     * Retrieve the cells that were removed.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getRemoved()
    {
      return removed;
    }
    
    /**
     * Retrieve a map that contains (object, map) pairs
     * of the attributes that have been stored in the model.
     * @return A map data type.
     * @see Map
     */
    public Map getPreviousAttributes()
    {
      return previousAttributes;
    }
    
    /**
     * Retrieves a map of (object, view attributes). The objects
     * are model objects which need to be mapped to views.
     * @return A Map data type.
     * @see Object
     */
    public Map getAttributes()
    {
      return attributes;
    }
    
    /**
     * Redoes a change.
     * @throws CannotRedoException if the change cannot be redone.
     * @see CannotRedoException
     */
    public void redo()
      throws CannotRedoException
    {
      super.redo();
      execute();
    }
    
    /**
     * Undoes a change.
     * @throws CannotUndoException if the change cannot be undone.
     * @see CannotUndoException
     */
    public void undo()
      throws CannotUndoException
    {
      super.undo();
      execute();
    }
    
    /**
     * Execute this edit such that the next invocation to this
     * method will invert the last execution.
     */
    public void execute()
    {
      HashSet hashset = new HashSet();
      if(attributes != null)
      {
        hashset.addAll(attributes.keySet());
      }
      if(parentMap != null)
      {
        hashset.addAll(parentMap.getChangedNodes());
      }
      if(connectionSet != null)
      {
        hashset.addAll(connectionSet.getChangedEdges());
      }
      if(remove != null)
      {
        for(int i = 0; i < remove.length; i++)
        {
          hashset.remove(remove[i]);
        }
        
      }
      changed = hashset.toArray();
      Set set = DefaultGraphModel.getEdges(DefaultGraphModel.this, changed);
      context = set.toArray();
      inserted = insert;
      removed = remove;
      remove = handleInsert(inserted);
      parentMap = handleParentMap(parentMap);
      connectionSet = handleConnectionSet(connectionSet);
      insert = handleRemove(removed);
      previousAttributes = attributes;
      attributes = handleAttributes(attributes);
      fireGraphChanged(DefaultGraphModel.this, this);
    }
    
    /** 
     * Places views into a graph layout cache.
     * @param graphlayoutcache A variable of GraphLayoutCache data type.
     * @param acellview A variable of CellView data type.
     * @see GraphLayoutCache
     * @see CellView
     */
    public void putViews(GraphLayoutCache graphlayoutcache, CellView acellview[])
    {
      if(graphlayoutcache != null && acellview != null)
      {
        cellViews.put(graphlayoutcache, acellview);
      }
    }
    
    /** 
     * Retrieve cell views.
     * @param graphlayoutcache A variable of GraphLayoutCache data type.
     * @return A CellView data type.
     * @see GraphLayoutCache
     * @see CellView
     */
    public CellView[] getViews(GraphLayoutCache graphlayoutcache)
    {
      return (CellView[])cellViews.get(graphlayoutcache);
    }
    
    /** 
     * Converts to string.
     * @return A String data type.
     */
    public String toString()
    {
      String s = new String();
      if(inserted != null)
      {
        s = s + "Inserted:\n";
        for(int i = 0; i < inserted.length; i++)
        {
          s = s + "  " + inserted[i] + "\n";
        }
        
      } else
      {
        s = s + "None inserted\n";
      }
      if(removed != null)
      {
        s = s + "Removed:\n";
        for(int j = 0; j < removed.length; j++)
        {
          s = s + "  " + removed[j] + "\n";
        }
        
      } else
      {
        s = s + "None removed\n";
      }
      if(changed != null && changed.length > 0)
      {
        s = s + "Changed:\n";
        for(int k = 0; k < changed.length; k++)
        {
          s = s + "  " + changed[k] + "\n";
        }
        
      } else
      {
        s = s + "None changed\n";
      }
      if(parentMap != null)
      {
        s = s + parentMap.toString();
      } else
      {
        s = s + "No parent map\n";
      }
      return s;
    }
    
    /**
     * Constructs an edit record.
     * @param map A variable of Map data type.
     * @param connectionset A variable of ConnectionSet data type.
     * @param parentmap A variable of ParentMap data type.
     * @param aobj1 A variable of Object data type.
     * @param aobj A variable of Object data type.
     * @ee Map
     * @see ConnectionSet
     * @see ParentMap
     * @see Object
     */
    public GraphModelEdit(Object aobj[], Object aobj1[], Map map, ConnectionSet connectionset, ParentMap parentmap)
    {
      cellViews = new Hashtable();
      insert = aobj;
      remove = aobj1;
      connectionSet = connectionset;
      attributes = map;
      parentMap = parentmap;
      previousAttributes = map;
      if(parentmap != null)
      {
        handleEmptyGroups(parentmap.emptyParents());
      }
    }
  }
  
  /** 
   * An instance of EventListenerList.
   */
  protected transient EventListenerList listenerList;
  
  /** 
   * An instance of Iterator.
   */
  protected transient Iterator emptyIterator;
  
  /** 
   * An instance of List.
   */
  protected List roots;
  
  /**
   * An instance of boolean.
   */
  protected boolean asksAllowsChildren;
  
  /**
   * Constructs a model that is not an attribute store.
   */
  public DefaultGraphModel()
  {
    listenerList = new EventListenerList();
    emptyIterator = new EmptyIterator();
    roots = new ArrayList();
    asksAllowsChildren = false;
  }
  
  /**
   * Returns a number of roots in the model.  Returns 0 if the
   * model is empty.
   * @return An integer data type.
   */
  public int getRootCount()
  {
    return roots.size();
  }
  
  /**
   * Returns a root at index index in the model.
   * This should not return null if index is a valid
   * index for the model (that is index >= 0 &&
   * index < getRootCount()).
   * @param i A variable of integer data type.
   * @return An Object data type.
   * @see Object
   */
  public Object getRootAt(int i)
  {
    return roots.get(i);
  }
  
  /**
   * Returns an index of root in the model.
   * If root is null, returns -1.
   * @param obj A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  public int getIndexOfRoot(Object obj)
  {
    return roots.indexOf(obj);
  }
  
  /**
   * Returns true if node or one of its
   * ancestors is in the model.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public boolean contains(Object obj)
  {
    if(obj instanceof DefaultMutableTreeNode)
    {
      return roots.contains(((DefaultMutableTreeNode)obj).getRoot());
    } else
    {
      return roots.contains(obj);
    }
  }
  
  /** 
   * Retrieve current attributes.
   * @param obj A variable of Object data type.
   * @return A Map data type.
   * @see Object
   * @see Map
   */
  public Map getAttributes(Object obj)
  {
    if(obj instanceof GraphCell)
    {
      return ((GraphCell)obj).getAttributes();
    } else
    {
      return null;
    }
  }
  
  /**
   * Retrieves a source of an edge. Edge must be an object
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public Object getSource(Object obj)
  {
    if(obj instanceof Edge)
    {
      return ((Edge)obj).getSource();
    } else
    {
      return null;
    }
  }
  
  /**
   * Retrieves a target of an edge. Edge must be an object.
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public Object getTarget(Object obj)
  {
    if(obj instanceof Edge)
    {
      return ((Edge)obj).getTarget();
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns true if port is a valid source
   * for edge. Edge and port must be
   * objects previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public boolean acceptsSource(Object obj, Object obj1)
  {
    return true;
  }
  
  /**
   * Returns true if port is a valid target
   * for edge. Edge and port must be
   * objects previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public boolean acceptsTarget(Object obj, Object obj1)
  {
    return true;
  }
  
  /**
   * Returns an iterator of the edges connected to <code>port</code>.
   * Port must be a object previously obtained from
   * this data source. This method never returns null.
   * @param obj A variable of Object data type.
   * @return An iterator data type.
   * @see Object
   * @see Iterator
   */
  public Iterator edges(Object obj)
  {
    if(obj instanceof Port)
    {
      return ((Port)obj).edges();
    } else
    {
      return emptyIterator;
    }
  }
  
  /**
   * Retrieves the parent of child in the model.
   * child must be a node previously obtained from
   * this data source. This returns null if child is
   * a root in the model.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public Object getParent(Object obj)
  {
    if(obj != null && (obj instanceof TreeNode))
    {
      return ((TreeNode)obj).getParent();
    } else
    {
      return null;
    }
  }
  
  /**
   * Retrieves an index of a child in parent.
   * If either the parent or child is null, returns -1.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  public int getIndexOfChild(Object obj, Object obj1)
  {
    if(obj == null || obj1 == null)
    {
      return -1;
    } else
    {
      return ((TreeNode)obj).getIndex((TreeNode)obj1);
    }
  }
  
  /**
   * Retrieves a child of parent at index index in the parent's
   * child array. Parent must be a node previously obtained from
   * this data source. This should not return null if index
   * is a valid index for parent (that is index >= 0 &&
   * index < getChildCount(parent)).
   * @param obj A variable of Object data type.
   * @param i A variable of integer data type.
   * @return An Object data type.
   * @see Object
   */
  public Object getChild(Object obj, int i)
  {
    if(obj instanceof TreeNode)
    {
      return ((TreeNode)obj).getChildAt(i);
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns a number of children of parent.  Returns 0 if the node
   * is a leaf or if it has no children. Parent must be a node
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  public int getChildCount(Object obj)
  {
    if(obj instanceof TreeNode)
    {
      return ((TreeNode)obj).getChildCount();
    } else
    {
      return 0;
    }
  }
  
  /**
   * Returns whether the specified node is a leaf node.
   * The way the test is performed depends on the.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   */
  public boolean isLeaf(Object obj)
  {
    if(asksAllowsChildren && (obj instanceof TreeNode))
    {
      return !((TreeNode)obj).getAllowsChildren();
    } else
    {
      return ((TreeNode)obj).isLeaf();
    }
  }
  /**
   * Inserts the roots and connections into the model.
   * Notifies the model- and undo listeners of the change. The passed-in
   * edits are executed if they implement the
   * GraphModelEvent.ExecutableGraphChange interface
   * in ascending array-order, after execution of the model change.
   * (Note: The external order is important in a
   * special case: After insertion on a partial view, ie. one that does not
   * display all cells of the model, the cell is made visible after
   * it is inserted into the model. This requires the inserting view
   * to be able to add the cell to the visible set before it is
   * inserted into the model.)
   * Note: The passed-in propertyMap may contains PortViews
   * which must be turned into Points when stored in the model.
   * @param aobj A variable of Object data type.
   * @param map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of UndoableEdit data type.
   * @see Object
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   */
  public void insert(Object aobj[], Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[])
  {
  	//System.out.println ("Called on Ctrl+V") ;
  	//(new Exception()).printStackTrace() ;
  	
    GraphModelEdit graphmodeledit = createInsertEdit(aobj, map, connectionset, parentmap, aundoableedit);
    if(graphmodeledit != null)
    {
      graphmodeledit.execute();
      if(aundoableedit != null)
      {
        for(int i = 0; i < aundoableedit.length; i++)
        {
          if(aundoableedit[i] instanceof GraphModelEvent.ExecutableGraphChange)
          {
            ((GraphModelEvent.ExecutableGraphChange)aundoableedit[i]).execute();
          }
        }
        
      }
      postEdit(graphmodeledit);
    }
    //EditPaste.Paste(false) ;
  }
  
  /**
   * Removes cells from the model.
   * Notifies the model and undo listeners of the change.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void remove(Object aobj[])
  {
  	//System.out.println ( "Called on Ctrl+X") ;
  	//(new Exception()).printStackTrace () ;
  
  	EditCut.CopyToClipBoard(false) ;
  	
    GraphModelEdit graphmodeledit = createRemoveEdit(aobj);
    if(graphmodeledit != null)
    {
      graphmodeledit.execute();
      postEdit(graphmodeledit);
    }
 
  }
  
  /**
   * Applies attributes and the connection changes to
   * the model. The initial edits that triggered the call
   * are considered to be part of this transaction.  The passed-in
   * edits are executed if they implement the
   * GraphModelEvent.ExecutableGraphChange interface
   * in ascending array-order, after execution of the model change.
   * Notifies the model- and undo listeners of the change.
   * Note: If only edits is non-null, the
   * edits are directly passed to the UndoableEditListeners.
   * Note: The passed-in propertyMap may contains PortViews
   * which must be turned into Points when stored in the model.
   * @param map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of UndoableEdit data type.
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   */
  public void edit(Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[])
  {
    if((map == null || map.isEmpty()) && (connectionset == null || connectionset.isEmpty()) && parentmap == null && aundoableedit != null && aundoableedit.length == 1)
    {
      if(aundoableedit[0] instanceof GraphModelEvent.ExecutableGraphChange)
      {
        ((GraphModelEvent.ExecutableGraphChange)aundoableedit[0]).execute();
      }
      postEdit(aundoableedit[0]);
    } else
    {
      GraphModelEdit graphmodeledit = createCellEdit(map, connectionset, parentmap, aundoableedit);
      if(graphmodeledit != null)
      {
        graphmodeledit.execute();
        if(aundoableedit != null)
        {
          for(int i = 0; i < aundoableedit.length; i++)
          {
            if(aundoableedit[i] instanceof GraphModelEvent.ExecutableGraphChange)
            {
              ((GraphModelEvent.ExecutableGraphChange)aundoableedit[i]).execute();
            }
          }
          
        }
        postEdit(graphmodeledit);
      }
    }
  }
  
  /**
   * Sends cells to back.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void toBack(Object aobj[])
  {
    GraphModelLayerEdit graphmodellayeredit = createLayerEdit(aobj, -2);
    if(graphmodellayeredit != null)
    {
      graphmodellayeredit.execute();
      postEdit(graphmodellayeredit);
    }
  }
  
  /**
   * Brings cells to front.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void toFront(Object aobj[])
  {
    GraphModelLayerEdit graphmodellayeredit = createLayerEdit(aobj, -1);
    if(graphmodellayeredit != null)
    {
      graphmodellayeredit.execute();
      postEdit(graphmodellayeredit);
    }
  }
  
  /**
   * Creates a layer to edit.
   * @param aobj A variable of Object data type.
   * @param i A variable of integer data type.
   * @return A GraphModelLayerEdit data type.
   * @see Object
   * @see GraphModelLayerEdit
   */
  protected GraphModelLayerEdit createLayerEdit(Object aobj[], int i)
  {
    return new GraphModelLayerEdit(aobj, i);
  }
  
  /**
   * Returns an edit that represents an insert.
   * @param aobj A variable of Object data type.
   * @param map Map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of UndoableEdit data type.
   * @return GraphModelEdit A variable of GraphModelEdit data type.
   * @see Object
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   * @see GraphModelEdit
   */
  protected GraphModelEdit createInsertEdit(Object aobj[], Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[])
  {
    GraphModelEdit graphmodeledit = createEdit(aobj, null, map, connectionset, parentmap);
    if(graphmodeledit != null)
    {
      if(aundoableedit != null)
      {
        for(int i = 0; i < aundoableedit.length; i++)
        {
          graphmodeledit.addEdit(aundoableedit[i]);
        }
        
      }
      graphmodeledit.end();
    }
    return graphmodeledit;
  }
  
  /**
   * Returns an edit that represents a remove.
   * @param aobj A variable of Object data type.
   * @return A GraphModelEdit data type.
   * @see Object
   * @see GraphModelEdit
   */
  protected GraphModelEdit createRemoveEdit(Object aobj[])
  {
    ConnectionSet connectionset = ConnectionSet.create(this, aobj, true);
    ParentMap parentmap = ParentMap.create(this, aobj, true);
    GraphModelEdit graphmodeledit = createEdit(null, aobj, null, connectionset, parentmap);
    if(graphmodeledit != null)
    {
      graphmodeledit.end();
    }
    return graphmodeledit;
  }
  
  /** 
   * Returns an edit that represents a change.
   * @param map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of UndoableEdit data type.
   * @return A GraphModelEdit data type.
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   * @see GraphModelEdit
   */    
  protected GraphModelEdit createCellEdit(Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[])
  {
    GraphModelEdit graphmodeledit = createEdit(null, null, map, connectionset, parentmap);
    if(graphmodeledit != null)
    {
      if(aundoableedit != null)
      {
        for(int i = 0; i < aundoableedit.length; i++)
        {
          graphmodeledit.addEdit(aundoableedit[i]);
        }
        
      }
      graphmodeledit.end();
    }
    return graphmodeledit;
  }
  
  /** 
   * Returns an edit that represents a change.
   * @param aobj A variable of Object data type.
   * @param aobj1 A variable of Object data type.
   * @param map Map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @return A GraphModelEdit data type.
   * @see Object
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see GraphModelEdit
   */    
  protected GraphModelEdit createEdit(Object aobj[], Object aobj1[], Map map, ConnectionSet connectionset, ParentMap parentmap)
  {
    return new GraphModelEdit(aobj, aobj1, map, connectionset, parentmap);
  }
  
  /**
   * Inserts cells into the model. Returns
   * the cells that were inserted (including descendants).
   * @param aobj A variable of Object data type.
   * @return An Object[] data type.
   * @see Object
   */
  protected Object[] handleInsert(Object aobj[])
  {
    Object aobj1[] = null;
    if(aobj != null)
    {
      for(int i = 0; i < aobj.length; i++)
      {
        if(getParent(aobj[i]) == null)
        {
          roots.add(aobj[i]);
        }
      }
      
      aobj1 = getDescendants(this, aobj).toArray();
    }
    return aobj1;
  }
  
  /**
   * Removes cells from the model. Returns
   * the cells that were removed as roots.
   * @param aobj A variable of Object data type.
   * @return An Object[] data type.
   * @see Object
   */
  protected Object[] handleRemove(Object aobj[])
  {
    ArrayList arraylist = new ArrayList();
    if(aobj != null)
    {
      for(int i = 0; i < aobj.length; i++)
      {
        if(getParent(aobj[i]) == null && roots.remove(aobj[i]))
        {
          arraylist.add(aobj[i]);
        }
      }
      
    }
    return arraylist.toArray();
  }
  
  /**
   * Applies cells to the model. Returns
   * a parent map that may be used to undo this change.
   * @param parentmap A variable of ParentMap data type.
   * @return A ParentMap data type.
   * @see ParentMap
   */
  protected ParentMap handleParentMap(ParentMap parentmap)
  {
    if(parentmap != null)
    {
      ParentMap parentmap1 = new ParentMap();
      for(Iterator iterator = parentmap.entries(); iterator.hasNext();)
      {
        ParentMap.Entry entry = (ParentMap.Entry)iterator.next();
        MutableTreeNode mutabletreenode = (MutableTreeNode)entry.getChild();
        DefaultMutableTreeNode defaultmutabletreenode = (DefaultMutableTreeNode)entry.getParent();
        parentmap1.addEntry(mutabletreenode, getParent(mutabletreenode));
        if(defaultmutabletreenode == null)
        {
          mutabletreenode.removeFromParent();
        } else
        {
          defaultmutabletreenode.add(mutabletreenode);
        }
        boolean flag = roots.contains(mutabletreenode);
        if(defaultmutabletreenode == null && !flag)
        {
          roots.add(mutabletreenode);
        } else
          if(defaultmutabletreenode != null && flag)
        {
          roots.remove(mutabletreenode);
        }
      }
      
      return parentmap1;
    } else
    {
      return null;
    }
  }
  
  /**
   * Applies attributes to the cells specified as keys.
   * Returns the attributes to undo the change.
   * @param map A variable of Map data type.
   * @return A Map data type.
   * @see Map
   */
  protected Map handleAttributes(Map map)
  {
    if(map != null)
    {
      Hashtable hashtable = new Hashtable();
      GraphCell graphcell;
      Map map2;
      for(Iterator iterator = map.entrySet().iterator(); iterator.hasNext(); hashtable.put(graphcell, map2))
      {
        java.util.Map.Entry entry = (java.util.Map.Entry)iterator.next();
        graphcell = (GraphCell)entry.getKey();
        Map map1 = (Map)entry.getValue();
        map2 = graphcell.changeAttributes(map1);
      }
      
      return hashtable;
    } else
    {
      return null;
    }
  }
  
  /**
   * Applies connectionSet to the model. Returns
   * a connection set that may be used to undo this change.
   * @param connectionset A variable of ConnectionSet data type.
   * @return A ConnectionSet data type.
   * @see ConnectionSet
   */
  protected ConnectionSet handleConnectionSet(ConnectionSet connectionset)
  {
    if(connectionset != null)
    {
      ConnectionSet connectionset1 = new ConnectionSet();
      ConnectionSet.Connection connection;
      for(Iterator iterator = connectionset.connections(); iterator.hasNext(); handleConnection(connection))
      {
        connection = (ConnectionSet.Connection)iterator.next();
        Object obj = connection.getEdge();
        if(connection.isSource())
        {
          connectionset1.connect(obj, getSource(obj), true);
        } else
        {
          connectionset1.connect(obj, getTarget(obj), false);
        }
      }
      
      return connectionset1;
    } else
    {
      return null;
    }
  }
  /**
   * Inserts the specified connection into the model.
   * @param connection A variable of Connection data type.
   * @see ConnectionSet.Connection
   */
  protected void handleConnection(ConnectionSet.Connection connection)
  {
    Object obj = connection.getEdge();
    Object obj1 = connection.isSource() ? getSource(obj) : getTarget(obj);
    Object obj2 = connection.getPort();
    if(obj2 != obj1)
    {
      connect(obj, obj1, connection.isSource(), true);
      if(contains(obj2) && contains(obj))
      {
        connect(obj, obj2, connection.isSource(), false);
      }
    }
  }
  
  /**
   * Connects or disconnects the edge and port in this model
   * based on <code>remove</code>. Subclassers should override
   * this to update connectivity datastructures.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @param flag A variable of boolean data type.
   * @param flag1 A variable of boolean data type.
   * @see Object
   */
  protected void connect(Object obj, Object obj1, boolean flag, boolean flag1)
  {
    if(obj1 instanceof Port)
    {
      if(flag1)
      {
        ((Port)obj1).removeEdge(obj);
      } else
      {
        ((Port)obj1).addEdge(obj);
      }
    }
    if(flag1)
    {
      obj1 = null;
    }
    if(obj instanceof Edge)
    {
      if(flag)
      {
        ((Edge)obj).setSource(obj1);
      } else
      {
        ((Edge)obj).setTarget(obj1);
      }
    }
  }
  
  /**
   * Adds a listener for the GraphModelEvent posted after the graph changes.
   * @param graphmodellistener A variable of GraphModelListener data type.
   * @see GraphModelListener
   */
  public void addGraphModelListener(GraphModelListener graphmodellistener)
  {
    listenerList.add(GraphModelListener.class, graphmodellistener);
  }
  
  /**
   * Removes a listener previously added with addGraphModelListener().
   * @param graphmodellistener A variable of GraphModelListener.
   * See GraphModelListener
   */
  public void removeGraphModelListener(GraphModelListener graphmodellistener)
  {
    listenerList.remove(GraphModelListener.class, graphmodellistener);
  }
  
  /** 
   * Notify all listeners that have registered interest for
   * notification on this event type.  The event instance
   * is lazily created using the parameters passed into
   * the fire method.
   * @param obj A variable of Object data type.
   * @param graphmodelchange A variable of GraphModelChange data type.
   * @see Object
   * @see GraphModelChange
   */
  protected void fireGraphChanged(Object obj, GraphModelEvent.GraphModelChange graphmodelchange)
  {
    Object aobj[] = listenerList.getListenerList();
    GraphModelEvent graphmodelevent = null;
    for(int i = aobj.length - 2; i >= 0; i -= 2)
    {
      if(aobj[i] == (GraphModelListener.class))
      {
        if(graphmodelevent == null)
        {
          graphmodelevent = new GraphModelEvent(obj, graphmodelchange);
        }
        ((GraphModelListener)aobj[i + 1]).graphChanged(graphmodelevent);
      }
    }
    
  }
  
  /**
   * Return an array of all GraphModelListeners that were added to this model.
   * @return A GraphModelList[] data type.
   * @see GraphModelList
   */
  public GraphModelListener[] getGraphModelListeners()
  {
    return (GraphModelListener[])listenerList.getListeners(GraphModelListener.class);
  }
  
  /**
   * Returns the source vertex of the edge by calling getParent on
   * getSource on the specified model.
   * @param graphmodel A variable of GraphModel data type.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see GraphModel
   * @see Object
   */
  public static Object getSourceVertex(GraphModel graphmodel, Object obj)
  {
    if(graphmodel != null)
    {
      return graphmodel.getParent(graphmodel.getSource(obj));
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns the target vertex of the edge by calling getParent on
   * getTarget on the specified model.
   * @param graphmodel A variable of GraphModel data type.
   * @param aobj A variable of Object data type.
   * @return An Object data type.
   * @see GraphModel
   * @see Object
   * @see Object
   */
  public static Object getTargetVertex(GraphModel graphmodel, Object obj)
  {
    if(graphmodel != null)
    {
      return graphmodel.getParent(graphmodel.getTarget(obj));
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns the roots of the specified model as an array.
   * This implementation only uses the GraphModel interface.
   * @param graphmodel A variable of GraphModel data type.
   * @return An Object[] data type.
   * @see GraphModel
   * @see Object
   */
  public static Object[] getRoots(GraphModel graphmodel)
  {
    Object aobj[] = null;
    if(graphmodel != null)
    {
      aobj = new Object[graphmodel.getRootCount()];
      for(int i = 0; i < aobj.length; i++)
      {
        aobj[i] = graphmodel.getRootAt(i);
      }
      
    }
    return aobj;
  }
  
  /**
   * Return the set of edges that are connected to the
   * specified cells. The array is flattened and then
   * all attached edges that are not part of the cells
   * array are returned.
   * @param graphmodel A variable of GraphMode data type.
   * @param aobj A variable of Object data type.
   * @return A Set data type.
   * @see GraphModel
   * @see Object
   * @see Set
   */
  public static Set getEdges(GraphModel graphmodel, Object aobj[])
  {
    HashSet hashset = new HashSet();
    Set set = getDescendants(graphmodel, aobj);
    if(set != null)
    {
      for(Iterator iterator = set.iterator(); iterator.hasNext();)
      {
        for(Iterator iterator1 = graphmodel.edges(iterator.next()); iterator1.hasNext(); hashset.add(iterator1.next())) { }
      }
      
      hashset.removeAll(set);
    }
    return hashset;
  }
  
  /**
   * Flattens the given array of root cells by adding the roots
   * and their descandants. The resulting set contains all cells,
   * which means it contains branches <strong>and</strong> leafs.
   * Note: This is an iterative implementation. No recursion used.
   * @param graphmodel A variable of GraphMode data type.
   * @param aobj A variable of Object data type.
   * @return A Set data type.
   * @see GraphModel
   * @see Object
   * @see Set
   */
  public static Set getDescendants(GraphModel graphmodel, Object aobj[])
  {
    if(aobj != null)
    {
      Stack stack = new Stack();
      for(int i = 0; i < aobj.length; i++)
      {
        stack.add(aobj[i]);
      }
      
      HashSet hashset = new HashSet();
      while(!stack.isEmpty()) 
      {
        Object obj = stack.pop();
        for(int j = 0; j < graphmodel.getChildCount(obj); j++)
        {
          stack.add(graphmodel.getChild(obj, j));
        }
        
        if(obj != null)
        {
          hashset.add(obj);
        }
      }
      return hashset;
    } else
    {
      return null;
    }
  }
  
  /**
   * This method supports serialization.
   * @param objectinputstream A variable of ObjectInputStream data type.
   * @throws IOException if invalid.
   * @throws ClassNotFoundException if no class exists.
   * @see ObjectInputStream
   * @see IOException
   * @see ClassNotFoundException
   */
  private void readObject(ObjectInputStream objectinputstream)
    throws IOException, ClassNotFoundException
  {
    objectinputstream.defaultReadObject();
    listenerList = new EventListenerList();
    emptyIterator = new EmptyIterator();
  }
}
