
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.*;
import javax.swing.event.EventListenerList;
import javax.swing.event.SwingPropertyChangeSupport;

/**
 * Default implementation of GraphSelectionModel.  Listeners are notified
 *
 * Referenced classes of package com.jgraph.graph are
 * Port, GraphSelectionModel, GraphLayoutCache, GraphModel
 * and DefaultGraphModel.
 *
 * @author Gaudenz Alder 
 * @version 1.0 1/1/02
 */

public class DefaultGraphSelectionModel
  implements GraphSelectionModel, Cloneable, Serializable
{
  
  /**
   * Holds a path and whether or not it is new.
   */
  protected class CellPlaceHolder
  {
    /** 
     * An instance of boolean. 
     */
    protected boolean isNew;
    
    /**
     * An instance of Object. 
     */
    protected Object cell;
    
    /**
     * Returns the cell.
     * @return An Object data type.
     * @see Object
     */
    public Object getCell()
    {
      return cell;
    }
    
    /**
     * Returns the isNew.
     * @return A boolean data type.
     */
    public boolean isNew()
    {
      return isNew;
    }
    
    /**
     * Sets the cell.
     * @param obj A variable of Object data type.
     * @see Object
     */
    public void setCell(Object obj)
    {
      cell = obj;
    }
    
    /**
     * Sets the flag.
     * @param flag A variable of boolean data type.
     */
    public void setNew(boolean flag)
    {
      isNew = flag;
    }
    
    /**
     * Holds the place of a cell
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     */
    protected CellPlaceHolder(Object obj, boolean flag)
    {
      cell = obj;
      isNew = flag;
    }
  }
  
  /** 
   * An instance of String.
   * Property name for selectionMode. 
   */
  public static final String SELECTION_MODE_PROPERTY = "selectionMode";
  
  /** 
   * An instant of integer.
   * Value that represents selected state in cellStates. 
   */
  public static final int SELECTED = -1;
  
  /** 
   * An instance of JGraph.
   * Reference to the parent graph. Used to find parents and childs. 
   */
  protected JGraph graph;
  
  /** 
   * An instance of SwingPropertyChangeSupport
   * Used to message registered listeners. 
   */
  protected SwingPropertyChangeSupport changeSupport;
  
  /** 
   * An instance of EventListenerList.
   * Event listener list. 
   */
  protected EventListenerList listenerList;
  
  /**
   * An instance of integer.
   * Mode for the selection, will be either SINGLE_TREE_SELECTION,
   * CONTIGUOUS_TREE_SELECTION or DISCONTIGUOUS_TREE_SELECTION.
   */
  protected int selectionMode;
  
  /**
   * An instance of boolean.
   * Boolean that indicates if the model allows stepping-into groups. 
   */
  protected boolean childrenSelectable;
  
  /**
   * An instance of Map.
   * Maps the cells to their selection state. 
   */
  protected Map cellStates;
  
  /** 
   * An instance of List.
   * List that contains the selected items. 
   */
  protected List selection;
  
  /** 
   * Constructs a DefaultGraphSelectionModel for the specified graph. 
   * @param jgraph A variable of JGraph.
   * @see JGraph
   */
  public DefaultGraphSelectionModel(JGraph jgraph)
  {
    listenerList = new EventListenerList();
    childrenSelectable = true;
    cellStates = new Hashtable();
    selection = new ArrayList();
    graph = jgraph;
  }
  
  /**
   * Sets the selection mode, which must be one of SINGLE_TREE_SELECTION
   * @param i A variable of integer data type.
   */
  public void setSelectionMode(int i)
  {
    int j = selectionMode;
    selectionMode = i;
    if(selectionMode != 4 && selectionMode != 1)
    {
      selectionMode = 4;
    }
    if(j != selectionMode && changeSupport != null)
    {
      changeSupport.firePropertyChange("selectionMode", new Integer(j), new Integer(selectionMode));
    }
  }
  
  /**
   * Returns the selection mode, one of SINGLE_TREE_SELECTION,
   * DISCONTIGUOUS_TREE_SELECTION or CONTIGUOUS_TREE_SELECTION.
   * @return An integer data type.
   */
  public int getSelectionMode()
  {
    return selectionMode;
  }
  
  /**
   * Sets if the selection model allows the selection
   * of children.
   * @param flag A variable of boolean data type.
   */
  public void setChildrenSelectable(boolean flag)
  {
    childrenSelectable = flag;
  }
  
  /**
   * Returns true if the selection model allows the selection
   * of children.
   * @return A boolean data type.
   */
  public boolean isChildrenSelectable()
  {
    return childrenSelectable;
  }
  
  /**
   * Hook for subclassers for fine-grained control over stepping-into cells.
   * This implementation returns <code>childrenSelectable</code>.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  protected boolean isChildrenSelectable(Object obj)
  {
    return childrenSelectable;
  }
  
  /**
   * Sets the selection to path. If this represents a change, then
   * the TreeSelectionListeners are notified. If path is
   * null, this has the same effect as invoking clearSelection.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public void setSelectionCell(Object obj)
  {
    if(obj == null)
    {
      setSelectionCells(null);
    } else
    {
      setSelectionCells(new Object[] {
        obj
      });
    }
  }
  
  /**
   * Sets the selection to cells. If this represents a
   * change the GraphSelectionListeners are notified.  Potentially
   * paths will be held by this object; in other words don't change
   * any of the objects in the array once passed in.
   * If paths is null, this has the same effect as invoking clearSelection.
   * The lead path is set to the last path in Paths.
   * If the selection mode is CONTIGUOUS_TREE_SELECTION,
   * and adding the new paths would make the selection discontiguous,
   * the selection is reset to the first TreePath in paths.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void setSelectionCells(Object aobj[])
  {
    if(aobj != null)
    {
      if(selectionMode == 1 && aobj.length > 0)
      {
        aobj = (new Object[] {
          aobj[aobj.length - 1]
        });
      }
      Vector vector = new Vector();
      cellStates.clear();
      ArrayList arraylist = new ArrayList();
      for(int i = 0; i < aobj.length; i++)
      {
        if(aobj[i] != null)
        {
          vector.addElement(new CellPlaceHolder(aobj[i], !selection.remove(aobj[i])));
          select(arraylist, aobj[i]);
        }
      }
      
      for(Iterator iterator = selection.iterator(); iterator.hasNext(); vector.addElement(new CellPlaceHolder(iterator.next(), false))) { }
      selection = arraylist;
      if(vector.size() > 0)
      {
        notifyCellChange(vector);
      }
    }
  }
  
  /**
   * Adds path to the current selection. If path is not currently
   * in the selection the TreeSelectionListeners are notified. This has
   * no effect if path is null.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public void addSelectionCell(Object obj)
  {
    if(obj != null)
    {
      addSelectionCells(new Object[] {
        obj
      });
    }
  }
  
  /**
   * Adds cells to the current selection. If any of the paths in
   * paths are not currently in the selection the TreeSelectionListeners
   * are notified. This has no effect if paths is null.
   * The lead path is set to the last element in paths.
   * If the selection mode is CONTIGUOUS_TREE_SELECTION,
   * and adding the new paths would make the selection discontiguous.
   * Then two things can result: if the TreePaths in paths
   * are contiguous, then the selection becomes these TreePaths,
   * otherwise the TreePaths aren't contiguous and the selection becomes
   * the first TreePath in paths.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void addSelectionCells(Object aobj[])
  {
    if(aobj != null)
    {
      if(selectionMode == 1)
      {
        setSelectionCells(aobj);
      } else
      {
        Vector vector = new Vector();
        for(int i = 0; i < aobj.length; i++)
        {
          if(aobj[i] != null)
          {
            boolean flag = select(selection, aobj[i]);
            if(flag)
            {
              vector.addElement(new CellPlaceHolder(aobj[i], true));
            }
          }
        }
        
        if(vector.size() > 0)
        {
          notifyCellChange(vector);
        }
      }
    }
  }
  
  /**
   * Removes path from the selection. If path is in the selection
   * The TreeSelectionListeners are notified. This has no effect if
   * path is null.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public void removeSelectionCell(Object obj)
  {
    if(obj != null)
    {
      removeSelectionCells(new Object[] {
        obj
      });
    }
  }
  
  /**
   * Removes paths from the selection.  If any of the paths in paths
   * are in the selection the TreeSelectionListeners are notified.
   * This has no effect if paths is null.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public void removeSelectionCells(Object aobj[])
  {
    if(aobj != null)
    {
      Vector vector = new Vector();
      for(int i = 0; i < aobj.length; i++)
      {
        if(aobj[i] != null)
        {
          boolean flag = deselect(aobj[i]);
          if(flag)
          {
            vector.addElement(new CellPlaceHolder(aobj[i], false));
          }
        }
      }
      
      if(vector.size() > 0)
      {
        notifyCellChange(vector);
      }
    }
  }
  
  /**
   * Returns the cells that are currently selectable.
   * @return An Object[] data type.
   * @see Object
   */
  public Object[] getSelectables()
  {
    if(isChildrenSelectable())
    {
      ArrayList arraylist = new ArrayList();
      Stack stack = new Stack();
      Object aobj[] = graph.getGraphLayoutCache().getCells(graph.getGraphLayoutCache().getRoots());
      for(int i = 0; i < aobj.length; i++)
      {
        stack.add(aobj[i]);
      }
      
      while(!stack.isEmpty()) 
      {
        Object obj = stack.pop();
        if(!(obj instanceof Port))
        {
          arraylist.add(obj);
        }
        if(isChildrenSelectable(obj) && getSelectedChildCount(obj) != 0)
        {
          for(int j = 0; j < graph.getModel().getChildCount(obj); j++)
          {
            stack.add(graph.getModel().getChild(obj, j));
          }
          
        }
      }
      return arraylist.toArray();
    } else
    {
      return graph.getRoots();
    }
  }
  
  /**
   * Returns the first cell in the selection. This is useful if there
   * if only one item currently selected.
   * @return An Object data type.
   * @see Object
   */
  public Object getSelectionCell()
  {
    if(selection != null && selection.size() > 0)
    {
      return selection.toArray()[0];
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns the cells in the selection. This will return null (or an
   * empty array) if nothing is currently selected.
   * @return An Object[] data type.
   * @see Object
   */
  public Object[] getSelectionCells()
  {
    if(selection != null)
    {
      return selection.toArray();
    } else
    {
      return null;
    }
  }
  
  /**
   * Returns the number of paths that are selected.
   * @return An integer data type.
   */
  public int getSelectionCount()
  {
    return selection != null ? selection.size() : 0;
  }
  
  /**
   * Returns true if the cell
   * is in the current selection.
   * @param obj A variable of Object data type. 
   * @return A boolean data type.
   * @see Object
   */
  public boolean isCellSelected(Object obj)
  {
    int i = getSelectedChildCount(obj);
    return i == -1;
  }
  
  /**
   * Returns true if the cell
   * has selected children.
   * @param obj A variable of Object data type. 
   * @return A boolean data type.
   * @see Object
   */
  public boolean isChildrenSelected(Object obj)
  {
    int i = getSelectedChildCount(obj);
    return i > 0;
  }
  
  /**
   * Returns true if the selection is currently empty.
   * @return A boolean data type.
   */
  public boolean isSelectionEmpty()
  {
    return selection.isEmpty();
  }
  
  /**
   * Empties the current selection.  If this represents a change in the
   * current selection, the selection listeners are notified.
   */
  public void clearSelection()
  {
    if(selection != null)
    {
      Object aobj[] = getSelectionCells();
      Vector vector = new Vector();
      for(int i = 0; i < aobj.length; i++)
      {
        if(aobj[i] != null)
        {
          vector.addElement(new CellPlaceHolder(aobj[i], false));
        }
      }
      
      selection.clear();
      cellStates.clear();
      if(vector.size() > 0)
      {
        notifyCellChange(vector);
      }
    }
  }
  
  /**
   * Returns the number of selected childs for cell.
   * @param obj A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  protected int getSelectedChildCount(Object obj)
  {
    if(obj != null)
    {
      Integer integer = (Integer)cellStates.get(obj);
      if(integer == null)
      {
        integer = new Integer(0);
        cellStates.put(obj, integer);
      }
      return integer.intValue();
    } else
    {
      return 0;
    }
  }
  
  /**
   * Sets the number of selected childs for cell
   * to count.
   * @param obj A variable of Object data type.
   * @param i A variable of boolean data type.
   * @see Object
   */
  protected void setSelectedChildCount(Object obj, int i)
  {
    Integer integer = new Integer(i);
    cellStates.put(obj, integer);
  }
  
  /**
   * Selects a single cell and updates all datastructures.
   * No listeners are notified. Override this method to control
   * individual cell selection.
   * @param list A variable of List data type.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see List
   * @see Object
   */
  protected boolean select(List list, Object obj)
  {
    if(!isCellSelected(obj) && graph.getGraphLayoutCache().isVisible(obj))
    {
      for(Object obj1 = graph.getModel().getParent(obj); obj1 != null; obj1 = graph.getModel().getParent(obj1))
      {
        int i = getSelectedChildCount(obj1);
        if(i == -1)
        {
          i = 0;
        }
        i++;
        setSelectedChildCount(obj1, i);
        selection.remove(obj1);
      }
      
      Object aobj[] = {
        obj
      };
      Set set = DefaultGraphModel.getDescendants(graph.getModel(), aobj);
      set.remove(obj);
      for(Iterator iterator = set.iterator(); iterator.hasNext();)
      {
        Object obj2 = iterator.next();
        if(obj2 != null && !(obj2 instanceof Port))
        {
          selection.remove(obj2);
          cellStates.remove(obj2);
        }
      }
      
      setSelectedChildCount(obj, -1);
      return list.add(obj);
    } else
    {
      return false;
    }
  }
  
  /**
   * Deselects a single cell and updates all datastructures.
   * No listeners are notified.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  protected boolean deselect(Object obj)
  {
    if(isCellSelected(obj))
    {
      Object obj1 = graph.getModel().getParent(obj);
      boolean flag = true;
      for(byte byte0 = -1; obj1 != null && byte0 != 0;)
      {
        int i = getSelectedChildCount(obj1);
        i += byte0;
        if(i == 0 && flag)
        {
          byte0 = 0;
          i = -1;
          selection.add(obj1);
        }
        setSelectedChildCount(obj1, i);
        obj1 = graph.getModel().getParent(obj1);
        flag = false;
      }
      
      cellStates.remove(obj);
      return selection.remove(obj);
    } else
    {
      return false;
    }
  }
  
  /**
   * Adds graphselectionlistener to the list of listeners that are notified each time the
   * set of selected TreePaths changes.
   * @param graphselectionlistener A variable of GraphSelectionListener data type.
   * @see GraphSelectionListener
   */
  
  public void addGraphSelectionListener(GraphSelectionListener graphselectionlistener)
  {
    listenerList.add(GraphSelectionListener.class, graphselectionlistener);
  }
  
  /**
   * Removes graphselectionlistener from the list of listeners that are notified each time
   * the set of selected TreePaths changes.
   * @param graphselectionlistener A variable of GraphSelectionListener data type.
   * @see GraphSelectionListener
   */
  public void removeGraphSelectionListener(GraphSelectionListener graphselectionlistener)
  {
    listenerList.remove(GraphSelectionListener.class, graphselectionlistener);
  }
  
  /**
   * Notifies all listeners that are registered for
   * tree selection events on this object.
   * @param graphselectionevent A variable of GraphSelectionEvent data type.
   * @see GraphSelectionEvent
   */
  protected void fireValueChanged(GraphSelectionEvent graphselectionevent)
  {
    Object aobj[] = listenerList.getListenerList();
    for(int i = aobj.length - 2; i >= 0; i -= 2)
    {
      if(aobj[i] == (GraphSelectionListener.class))
      {
        ((GraphSelectionListener)aobj[i + 1]).valueChanged(graphselectionevent);
      }
    }
    
  }
  
  /**
   * Returns an array of all the listeners of the given type that
   * were added to this model.
   * @param class1 A variable of Class data type.
   * @return An EventListener[] data type.
   * @see EventListener
   */
  public EventListener[] getListeners(Class class1)
  {
    return listenerList.getListeners(class1);
  }
  
  /**
   * Adds a PropertyChangeListener to the listener list.
   * The listener is registered for all properties.
   * A PropertyChangeEvent will get fired when the selection mode
   * changes.
   * @param propertychangelistener A variable of PropertyChangeListener data type.
   * @see PropertyChangeListener
   */
  public synchronized void addPropertyChangeListener(PropertyChangeListener propertychangelistener)
  {
    if(changeSupport == null)
    {
      changeSupport = new SwingPropertyChangeSupport(this);
    }
    changeSupport.addPropertyChangeListener(propertychangelistener);
  }
  
  /**
   * Removes a PropertyChangeListener from the listener list.
   * This removes a PropertyChangeListener that was registered
   * for all properties.
   * @param propertychangelistener A variable of PropertyChangeListener data type.
   * @see PropertyChangeListener
   */
  public synchronized void removePropertyChangeListener(PropertyChangeListener propertychangelistener)
  {
    if(changeSupport == null)
    {
      return;
    } else
    {
      changeSupport.removePropertyChangeListener(propertychangelistener);
      return;
    }
  }
  
  /**
   * Notifies listeners of a change in path. changePaths should contain
   * instances of PathPlaceHolder.
   * @param vector A variable of Vector data type.
   * @see Vector
   */
  protected void notifyCellChange(Vector vector)
  {
    int i = vector.size();
    boolean aflag[] = new boolean[i];
    Object aobj[] = new Object[i];
    for(int j = 0; j < i; j++)
    {
      CellPlaceHolder cellplaceholder = (CellPlaceHolder)vector.elementAt(j);
      aflag[j] = cellplaceholder.isNew;
      aobj[j] = cellplaceholder.cell;
    }
    
    GraphSelectionEvent graphselectionevent = new GraphSelectionEvent(this, aobj, aflag);
    fireValueChanged(graphselectionevent);
  }
  
  /**
   * Returns a clone of this object with the same selection.
   * This method does not duplicate
   * selection listeners and property listeners.
   * @return An Object data type.
   * @throws CloneNotSupportedException never thrown by instances of this class.
   * @see Object
   * @see CloneNotSupportedException
   */
  public Object clone()
    throws CloneNotSupportedException
  {
    DefaultGraphSelectionModel defaultgraphselectionmodel = (DefaultGraphSelectionModel)super.clone();
    defaultgraphselectionmodel.changeSupport = null;
    if(selection != null)
    {
      defaultgraphselectionmodel.selection = new ArrayList(selection);
    }
    defaultgraphselectionmodel.listenerList = new EventListenerList();
    return defaultgraphselectionmodel;
  }
}
