
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;
//import java.awt.geom.Line2D;
import java.io.Serializable;
//import java.util.Collection;
import java.util.Map;
import javax.swing.SwingUtilities;

/**
 * This class takes the abstracted Edge data and provides additional tools for viewing the edges.
 *
 * Referenced classes of package com.jgraph.graph are
 * AbstractCellView, PortView, EdgeRenderer, GraphModel, 
 * CellMapper, GraphConstants, Edge, CellViewRenderer, 
 * GraphContext, CellHandle, CellView, ConnectionSet, 
 * Port and GraphLayoutCache.
 *
 * @author TerpPresent
 * @version 2.0
 *
 * @see AbstractCellView
 */

public class EdgeView extends AbstractCellView
{
    /** 
     * The Edge Handling class.
     */    
    public class EdgeHandle
        implements CellHandle, Serializable
    {

        /**
         * An instance of JGraph.
         */        
        protected JGraph graph;
        
        /**
         * An instance of EdgeView.
         * The current edge.
         */        
        protected EdgeView edge;
        
        /**
         * An instance of EdgeView.
         * The original edge.
         */        
        protected EdgeView orig;
        
        /**
         * An instance of boolean
         * Whether a label exists.
         */        
        protected boolean label;
        
        /**
         * An instance of boolean.
         * Whether the source exists.
         */        
        protected boolean source;
        
        /**
         * An instance of boolean.
         * Whether the target exists.
         */        
        protected boolean target;
        
        /**
         * An instance of Point.
         * The current point.
         */        
        protected Point currentPoint;
        
        /**
         * An instance of Rectangle.
         * The current rectangle.
         */        
        protected transient Rectangle r[];
        
        /**
         * An instance of Rectangle.
         * The rectangle location.
         */        
        protected transient Rectangle loc;
        
        /** 
         * An instance of boolean
         * The first overlayed call.
         */        
        protected boolean firstOverlayCall;
        
        /**
         * An instance of boolean.
         * Whether the edge is connectable.
         */        
        protected boolean isEdgeConnectable;
        
        /**
         * An instance of EdgeView.
         * the associated/adjoining edge object.
         */        
        protected EdgeView relevantEdge;

        /** 
         * Reloads the set of points.
         * @param edgeview A variable of EdgeView data type.
         * @see EdgeView
         */        
        protected void reloadPoints(EdgeView edgeview)
        {
            relevantEdge = edgeview;
            r = new Rectangle[edgeview.getPointCount()];
            for(int i = 0; i < r.length; i++)
            {
                r[i] = new Rectangle();
            }

            invalidate();
        }

        /**
         * Repaints the graphics area.
         * @param g A variable of Graphics data type.
         * @see Graphics
         */        
        public void paint(Graphics g)
        {
            invalidate();
            for(int i = 0; i < r.length; i++)
            {
                if(isEdgeConnectable)
                {
                    g.setColor(graph.getHandleColor());
                } else
                {
                    g.setColor(graph.getLockedHandleColor());
                }
                g.fill3DRect(r[i].x, r[i].y, r[i].width, r[i].height, true);
                PortView portview = null;
                if(i == 0 && edge.getSource() != null)
                {
                    portview = edge.getSource();
                } else
                if(i == r.length - 1 && edge.getTarget() != null)
                {
                    portview = edge.getTarget();
                }
                if(portview != null)
                {
                    g.setColor(graph.getLockedHandleColor());
                    Point point = GraphConstants.getOffset(portview.getAllAttributes());
                    if(point != null)
                    {
                        g.drawLine(r[i].x + 1, r[i].y + 1, (r[i].x + r[i].width) - 3, (r[i].y + r[i].height) - 3);
                        g.drawLine(r[i].x + 1, (r[i].y + r[i].height) - 3, (r[i].x + r[i].width) - 3, r[i].y + 1);
                    } else
                    {
                        g.drawRect(r[i].x + 2, r[i].y + 2, r[i].width - 5, r[i].height - 5);
                    }
                }
            }

        }

        /**
         * Overlays the current graphics area.
         * @param g A variable of Graphics data type.
         * @see Graphics
         */        
        public void overlay(Graphics g)
        {
            if(edge != null && !firstOverlayCall)
            {
                g.setColor(graph.getForeground());
                g.setXORMode(graph.getBackground());
                Graphics2D graphics2d = (Graphics2D)g;
                java.awt.geom.AffineTransform affinetransform = graphics2d.getTransform();
                graphics2d.scale(graph.getScale(), graph.getScale());
                graph.getUI().paintCell(g, edge, edge.getBounds(), true);
                graphics2d.setTransform(affinetransform);
                if(isSourceEditing() && edge.getSource() != null)
                {
                    paintPort(g, edge.getSource());
                } else
                if(isTargetEditing() && edge.getTarget() != null)
                {
                    paintPort(g, edge.getTarget());
                }
            }
            firstOverlayCall = false;
        }

        /**
         * Paints the port area.
         * @param g A variable of Graphics data type.
         * @param portview A variable of PortView data type.
         * @see Graphics
         * @see PortView
         */        
        protected void paintPort(Graphics g, PortView portview)
        {
            boolean flag = GraphConstants.getOffset(portview.getAllAttributes()) != null;
            Rectangle rectangle = flag ? portview.getBounds() : portview.getParentView().getBounds();
            rectangle = graph.toScreen(new Rectangle(rectangle));
            byte byte0 = 3;
            rectangle.translate(-byte0, -byte0);
            rectangle.setSize(rectangle.width + 2 * byte0, rectangle.height + 2 * byte0);
            graph.getUI().paintCell(g, portview, rectangle, true);
        }

        /**
         * Snaps in the current graphics area.
         * @param flag A variable of boolean data type.
         * @param point A variable of Point data type.
         * @return A boolean data type.
         * @see Point
         */        
        protected boolean snap(boolean flag, Point point)
        {
            boolean flag1 = graph.isConnectable() && isEdgeConnectable;
            Port port = graph.getPortForLocation(point.x, point.y);
            if(port != null && flag1)
            {
                PortView portview = (PortView)graph.getGraphLayoutCache().getMapping(port, false);
                if(GraphConstants.isConnectable(portview.getParentView().getAllAttributes()))
                {
                    Edge edge1 = (Edge)edge.getCell();
                    if(flag && edge.getSource() != portview && getModel().acceptsSource(edge1, port))
                    {
                        overlay(graph.getGraphics());
                        edge.setSource(portview);
                        edge.update();
                        overlay(graph.getGraphics());
                    } else
                    if(!flag && edge.getTarget() != portview && getModel().acceptsTarget(edge1, port))
                    {
                        overlay(graph.getGraphics());
                        edge.setTarget(portview);
                        edge.update();
                        overlay(graph.getGraphics());
                    }
                    return portview != null;
                }
            }
            return false;
        }

        /**
         * Checks for source editing.
         * @return A boolean data type.
         */        
        protected boolean isSourceEditing()
        {
            return source;
        }

        /**
         * Checks for target editing.
         * @return A boolean data type.
         */        
        protected boolean isTargetEditing()
        {
            return target;
        }

        /**
         * Checks for editing.
         * @return A boolean data type.
         */        
        protected boolean isEditing()
        {
            return source || target || label || currentPoint != null;
        }

        /**
         * Handles mouse movement.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */        
        public void mouseMoved(MouseEvent mouseevent)
        {
            for(int i = 0; i < r.length; i++)
            {
                if(r[i].contains(mouseevent.getPoint()))
                {
                    graph.setCursor(new Cursor(1));
                    mouseevent.consume();
                    return;
                }
            }

            if(loc.contains(mouseevent.getPoint()))
            {
                graph.setCursor(new Cursor(12));
                mouseevent.consume();
            }
        }

        /**
         * Handles mouse presses.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */        
        public void mousePressed(MouseEvent mouseevent)
        {
            boolean flag = graph.isBendable() && GraphConstants.isBendable(edge.getAllAttributes());
            boolean flag1 = graph.isDisconnectable() && GraphConstants.isDisconnectable(orig.getAllAttributes());
            int i = mouseevent.getX();
            int j = mouseevent.getY();
            int k = 0;
            for(k = 0; k < r.length; k++)
            {
                if(!r[k].contains(i, j))
                {
                    continue;
                }
                currentPoint = edge.getPoint(k);
                source = k == 0 && (edge.getSource() == null || flag1 && GraphConstants.isDisconnectable(edge.getSource().getParentView().getAllAttributes()));
                target = k == r.length - 1 && (edge.getTarget() == null || flag1 && GraphConstants.isDisconnectable(edge.getTarget().getParentView().getAllAttributes()));
                break;
            }

            if(!isEditing() && loc != null && loc.contains(i, j) && !isAddPointEvent(mouseevent) && !isRemovePointEvent(mouseevent))
            {
                if(mouseevent.getClickCount() == graph.getEditClickCount())
                {
                    graph.startEditingAtCell(edge);
                } else
                {
                    label = true;
                }
            }
            if(isRemovePointEvent(mouseevent) && currentPoint != null && flag)
            {
                edge.removePoint(k);
                mouseReleased(mouseevent);
            } else
            if(isAddPointEvent(mouseevent) && !isEditing() && flag)
            {
                int l = graph.getHandleSize();
                Rectangle rectangle = graph.fromScreen(new Rectangle(i - l, j - l, 2 * l, 2 * l));
                if(edge.intersects(graph.getGraphics(), rectangle))
                {
                    Point point = graph.fromScreen(graph.snap(new Point(mouseevent.getPoint())));
                    double d = 1.7976931348623157E+308D;
                    double d1 = 0.0D;
                    for(int i1 = 0; i1 < edge.getPointCount() - 1; i1++)
                    {
                        Point point1 = edge.getPoint(i1);
                        Point point2 = edge.getPoint(i1 + 1);
                        double d2 = (new java.awt.geom.Line2D.Double(point1, point2)).ptLineDistSq(point);
                        if(d2 < d)
                        {
                            d = d2;
                            k = i1 + 1;
                        }
                    }

                    edge.addPoint(k, point);
                    currentPoint = point;
                    reloadPoints(edge);
                    paint(graph.getGraphics());
                }
            }
            if(isEditing())
            {
                mouseevent.consume();
            }
        }

        /**
         * Handles mouse drags.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */              
        public void mouseDragged(MouseEvent mouseevent)
        {
            Point point = graph.fromScreen(new Point(mouseevent.getPoint()));
            if(label)
            {
                Rectangle rectangle = edge.getBounds();
                if(rectangle != null)
                {
                    int i = point.x - rectangle.x;
                    int k = point.y - rectangle.y;
                    int l = 1;
                    if(rectangle.width != 0)
                    {
                        l = 1000 / rectangle.width;
                    }
                    int j1 = 1;
                    if(rectangle.height != 0)
                    {
                        j1 = 1000 / rectangle.height;
                    }
                    point = new Point(i * l, k * j1);
                    overlay(graph.getGraphics());
                    edge.setLabelPosition(point);
                    edge.update();
                    overlay(graph.getGraphics());
                }
            } else
            if(isEditing() && (!source || !snap(true, point)) && (!target || !snap(false, point)) && (source && getModel().acceptsSource(cell, null) || target && getModel().acceptsTarget(cell, null) || !source && !target))
            {
                overlay(graph.getGraphics());
                Point point1 = graph.fromScreen(graph.snap(new Point(mouseevent.getPoint())));
                if(isConstrainedMoveEvent(mouseevent))
                {
                    EdgeView edgeview = (EdgeView)graph.getGraphLayoutCache().getMapping(edge.getCell(), false);
                    int j = 0;
                    if(target)
                    {
                        j = edgeview.getPointCount() - 1;
                    } else
                    {
                        edge.getPoints().indexOf(currentPoint);
                    }
                    Point point2 = edgeview.getPoint(j);
                    int i1 = point1.x - point2.x;
                    int k1 = point1.y - point2.y;
                    if(Math.abs(i1) < Math.abs(k1))
                    {
                        point1.x = point2.x;
                    } else
                    {
                        point1.y = point2.y;
                    }
                }
                currentPoint.setLocation(point1);
                if(source)
                {
                    edge.setPoint(0, point1);
                    edge.setSource(null);
                } else
                if(target)
                {
                    edge.setPoint(edge.getPointCount() - 1, point1);
                    edge.setTarget(null);
                }
                edge.update();
                overlay(graph.getGraphics());
            }
        }
        
        /**
         * Handles mouse Releases.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */           
        public void mouseReleased(MouseEvent mouseevent)
        {
            boolean flag = mouseevent.isControlDown() && graph.isCloneable();
            ConnectionSet connectionset = createConnectionSet(edge, edge.getCell(), flag);
            Map map = GraphConstants.createAttributes(new CellView[] {
                edge
            }, null);
            if(flag)
            {
                Map map1 = graph.cloneCells(new Object[] {
                    edge.getCell()
                });
                map = GraphConstants.replaceKeys(map1, map);
                connectionset = connectionset.clone(map1);
                graph.getGraphLayoutCache().insert(map1.values().toArray(), map, connectionset, null, null);
            } else
            {
                graph.getGraphLayoutCache().edit(map, connectionset, null, null);
            }
            mouseevent.consume();
        }

        /**
         * Creates a connection set.
         * @param edgeview A variable of EdgeView data type.
         * @param obj A variable of Object data type.
         * @param flag A variable of boolean data type.
         * @return A ConnectionSet data type.
         * @see EdgeView
         * @see Object
         * @see Connectionset
         */        
        protected ConnectionSet createConnectionSet(EdgeView edgeview, Object obj, boolean flag)
        {
            ConnectionSet connectionset = new ConnectionSet();
            Port port = null;
            Port port1 = null;
            if(edgeview.getSource() != null)
            {
                port = (Port)edgeview.getSource().getCell();
            }
            if(edgeview.getTarget() != null)
            {
                port1 = (Port)edgeview.getTarget().getCell();
            }
            if(flag || port != getModel().getSource(obj))
            {
                connectionset.connect(obj, port, true);
            }
            if(flag || port1 != getModel().getTarget(obj))
            {
                connectionset.connect(obj, port1, false);
            }
            return connectionset;
        }

        /**
         * Checks the validation scheme.
         */        
        protected void invalidate()
        {
            EdgeView edgeview = relevantEdge;
            int i = graph.getHandleSize();
            EdgeRenderer edgerenderer = EdgeView.renderer;
            for(int j = 0; j < r.length; j++)
            {
                Point point = graph.toScreen(new Point(edgeview.getPoint(j)));
                r[j].setBounds(point.x - i, point.y - i, 2 * i, 2 * i);
                point = graph.toScreen(edgerenderer.getLabelPosition(edgeview));
                Dimension dimension = edgerenderer.getLabelSize(edgeview);
                if(point != null && dimension != null)
                {
                    Point point1 = graph.toScreen(new Point(dimension.width, dimension.height));
                    loc.setBounds(point.x - point1.x / 2, point.y - point1.y / 2, point1.x, point1.y);
                }
            }

        }

        /**
         * The main constructor.
         * @param edgeview1 A variable of EdgeView data type.
         * @param graphcontext A variable of GraphContext data type.
         * @see EdgeView
         * @see GraphContext
         */        
        public EdgeHandle(EdgeView edgeview1, GraphContext graphcontext)
        {
            label = false;
            source = false;
            target = false;
            firstOverlayCall = true;
            isEdgeConnectable = true;
            relevantEdge = null;
            graph = graphcontext.getGraph();
            edge = edgeview1;
            loc = new Rectangle();
            orig = (EdgeView)graph.getGraphLayoutCache().getMapping(edgeview1.getCell(), false);
            reloadPoints(orig);
            isEdgeConnectable = GraphConstants.isConnectable(edgeview1.getAllAttributes());
        }
    }


    /**
     * An instance of EdgeRenderer.
     * The main edgeRenderer.
     */    
    public static EdgeRenderer renderer = new EdgeRenderer();
    
    /**
     * An instance of Point.
     * The list of points.
     */    
    protected java.util.List points;
    
    /**
     * An instance of PortView.
     * The source PortView data.
     */    
    protected PortView source;
    
    /**
     * An instance of PortView.
     * The target PortView data.
     */    
    protected PortView target;
    
    /**
     * An instance of Point.
     * The coordinates of the label.
     */    
    protected Point labelPosition;
    
    /**
     * An instance of Shape.
     * The initial shape.
     */    
    protected Shape beginShape;
    
    /**
     * An instance of Shape.
     * The final shape.
     */    
    protected Shape endShape;
    
    /** 
     * An instance of Shape.
     * The ending line shape.
     */    
    protected Shape lineShape;
    
    /**
     * An instance of GeneralPath.
     * Indicator for sharing of like paths.
     */    
    protected transient GeneralPath sharedPath;

    /**
     * The main constructor.
     * @param obj A variable of Object data type.
     * @param jgraph A variable of JGraph data type.
     * @param cellmapper A variable of CellMapper data type.
     * @see Object
     * @see JGraph
     * @see CellMapper
     */    
    public EdgeView(Object obj, JGraph jgraph, CellMapper cellmapper)
    {
        super(obj, jgraph, cellmapper);
        sharedPath = null;
    }

    /**
     * Refreshes the graphics area.
     * @param flag A variable of boolean data type.
     */    
    public void refresh(boolean flag)
    {
        super.refresh(flag);
        if(points != null)
        {
            Object obj = getModel().getSource(cell);
            Object obj1 = getModel().getTarget(cell);
            setSource((PortView)mapper.getMapping(obj, flag));
            setTarget((PortView)mapper.getMapping(obj1, flag));
            Edge.Routing routing = GraphConstants.getRouting(allAttributes);
            if(routing != null)
            {
                routing.route(this, points);
            }
        }
    }

    /**
     * Updates the graphics area.
     */    
    public void update()
    {
        super.update();
        points = GraphConstants.getPoints(allAttributes);
        labelPosition = GraphConstants.getLabelPosition(allAttributes);
        Edge.Routing routing = GraphConstants.getRouting(allAttributes);
        if(routing != null)
        {
            routing.route(this, points);
        }
        if(getModel().getSource(cell) != null)
        {
            setSource(getSource());
        }
        if(getModel().getTarget(cell) != null)
        {
            setTarget(getTarget());
        }
        beginShape = null;
        endShape = null;
        lineShape = null;
        sharedPath = null;
    }

    /**
     * Determines the intersection state.
     * @param g A variable of Graphics data type.
     * @param rectangle A variable of Rectangle data type.
     * @return A Boolean data type.
     * @see Graphics
     * @see Rectangle
     */    
    public boolean intersects(Graphics g, Rectangle rectangle)
    {
        return getEdgeRenderer().intersects(g, this, rectangle);
    }

    /**
     * Finds the outer bounds.
     * @return A Rectangle data type.
     * @see Rectangle
     */    
    public Rectangle getBounds()
    {
        return getEdgeRenderer().getBounds(this);
    }

    /**
     * Obtains the egdeRenderer.
     * @return A EdgeRenderer data type.
     * @see EdgeRenderer
     */    
    public EdgeRenderer getEdgeRenderer()
    {
        return (EdgeRenderer)getRenderer();
    }

    /**
     * Obtains the CellViewRenderer.
     * @return A CellViewRenderer data type.
     * @see CellViewRenderer
     */    
    public CellViewRenderer getRenderer()
    {
        return renderer;
    }

    /**
     * Finds the Handle.
     * @param graphcontext A variable of GraphContext data type.
     * @return A CellHandle data type.
     * @see GraphContext
     * @see CellHandle
     */    
    public CellHandle getHandle(GraphContext graphcontext)
    {
        return new EdgeHandle(this, graphcontext);
    }

    /**
     * Find the Source.
     * @return A PortView data type.
     * @see PortView
     */    
    public PortView getSource()
    {
        return source;
    }

    /**
     * Sets the source.
     * @param portview A variable of PortView data type.
     * @see PortView
     */    
    public void setSource(PortView portview)
    {
        source = portview;
        if(source != null)
        {
            points.set(0, source);
        } else
        {
            points.set(0, getPoint(0));
        }
    }

    /**
     * Finds the Target.
     * @return A PortView data type.
     */    
    public PortView getTarget()
    {
        return target;
    }

    /**
     * Sets the Target.
     * @param portview A variable of PortView data type.
     * @see PortView
     */    
    public void setTarget(PortView portview)
    {
        target = portview;
        int i = points.size() - 1;
        if(target != null)
        {
            points.set(i, target);
        } else
        {
            points.set(i, getPoint(i));
        }
    }

    /**
     * Finds the Label's position.
     * @return A Point data type.
     * @see Point
     */    
    public Point getLabelPosition()
    {
        return labelPosition;
    }

    /**
     * Sets the label's position.
     * @param point A variable of Point data type.
     * @see Point
     */    
    public void setLabelPosition(Point point)
    {
        labelPosition.setLocation(point);
    }

    /**
     * Finds the set of all points.
     * @return A List data type.
     * @see List
     */    
    public java.util.List getPoints()
    {
        return points;
    }

    /**
     * Counts the total number of points.
     * @return An integer data type.
     */    
    public int getPointCount()
    {
        return points.size();
    }

    /**
     * Finds the point at a location.
     * @param i A variable of integer data type.
     * @return A Point data type.
     * @see Point
     */    
    public Point getPoint(int i)
    {
        Object obj = points.get(i);
        if(obj instanceof PortView)
        {
            return ((PortView)obj).getLocation(this);
        }
        if(obj instanceof Point)
        {
            return (Point)obj;
        } else
        {
            return null;
        }
    }

    /**
     * Set the point.
     * @param i A variable of integer data type.
     * @param point A variable of Point data type.
     * @see Point
     */    
    public void setPoint(int i, Point point)
    {
        points.set(i, point);
    }

    /**
     * Add the point.
     * @param i A variable of integer data type.
     * @param point A variable of Point data type.
     * @see Point
     */     
    public void addPoint(int i, Point point)
    {
        points.add(i, point);
    }

    /**
     * Removes the given point at a location.
     * @param i A variable of integer data type.
     */    
    public void removePoint(int i)
    {
        points.remove(i);
    }

    /**
     * Checks for Add point event.
     * @param mouseevent A variable of MouseEvent data type.
     * @return a boolean data type.
     * @see MouseEvent
     */    
    public boolean isAddPointEvent(MouseEvent mouseevent)
    {
        return SwingUtilities.isRightMouseButton(mouseevent);
    }

    /**
     * Checks for Remove point event.
     * @param mouseevent A variable of MouseEvent data type.
     * @return a boolean data type.
     */    
    public boolean isRemovePointEvent(MouseEvent mouseevent)
    {
        return SwingUtilities.isRightMouseButton(mouseevent);
    }

    /**
     * Finds the length.
     * @param cellview A variable of CellView data type.
     * @return A double data type.
     * @see CellView
     */    
    public static double getLength(CellView cellview)
    {
        double d = 1.0D;
        if(cellview instanceof EdgeView)
        {
            EdgeView edgeview = (EdgeView)cellview;
            Point point = null;
            Object obj = null;
            for(int i = 0; i < edgeview.getPointCount(); i++)
            {
                Point point1 = edgeview.getPoint(i);
                if(point != null)
                {
                    d += point.distance(point1);
                }
                point = point1;
            }

        }
        return d;
    }

    /**
     * Checks for Constrained Move Event.
     * @param mouseevent A variable of MouseEvent data type.
     * @return A boolean data type.
     * @see MouseEvent
     */    
    public boolean isConstrainedMoveEvent(MouseEvent mouseevent)
    {
        GraphUI graphui = graph.getUI();
        if(graphui instanceof BasicGraphUI)
        {
            return ((BasicGraphUI)graphui).isConstrainedMoveEvent(mouseevent);
        } else
        {
            return false;
        }
    }

}
