/*
 * @(#)FileExportGXL.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.io.File;
//import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/** Imports a GXL file to the current file.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public class FileImportGXL extends AbstractActionDefault {
  
  /**
   * Constructor for FileImportGXL.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   */
  public FileImportGXL(GPGraphpad graphpad) {
    super(graphpad);
  }
  
  /** Performs an action.
   * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
   * @param e The action to be performed.
   */
  public void actionPerformed(ActionEvent e) {
    FileDialog f = new FileDialog(graphpad.getFrame(), "GXL File", FileDialog.LOAD);
    f.show();
    if (f.getFile() == null)
      return;
    
    try {
      String file = f.getDirectory() + f.getFile();
      GPGraph graph = getCurrentGraph();
      parseGXLFileInto(file, graph);
    } catch (Exception ex) {
      graphpad.error(ex.toString());
    }
  }
  
  //
  //
  // IMPORT GXL
  //
  //
  
  /** Imports the GXL file. Parsing done here.
   * @param name The name of the file to be imported.
   * @param graph A reference to the graph for importing this file.
   */
  public static void parseGXLFileInto(
                                      String name,
                                      GPGraph graph) throws Exception {
    GraphModel model = graph.getModel();
    File f = new File(name);
    // Create a DocumentBuilderFactory
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    // Create a DocumentBuilder
    DocumentBuilder db = dbf.newDocumentBuilder();
    // Parse the input file to get a Document object
    Document doc = db.parse(f);
    // Get the first child (the graph-element)
    Node node = doc.getDocumentElement().getChildNodes().item(0);
    // Get Graph's Child Nodes (the cells)
    NodeList list = node.getChildNodes();
    // List for the new Cells
    List newCells = new ArrayList();
    // ConnectionSet for the Insert method
    ConnectionSet cs = new ConnectionSet();
    // Hashtable for the ID lookup (ID to Vertex)
    Map ids = new Hashtable();
    // Hashtable for Attributes (Vertex to Map)
    Hashtable attributes = new Hashtable();
    // Loop Children
    for (int i = 0; i < list.getLength(); i++) {
      node = list.item(i);
      // Fetch Label
      String label = getLabel(node);
      // If Valid Node
      if (node.getAttributes() != null && node.getNodeName() != null) {
        // Fetch Type
        String type = node.getNodeName().toString().toLowerCase();
        
        // Create Vertex
        if (type.equals("node")) {
          // Fetch ID Node
          String id = null;
          Node tmp = node.getAttributes().getNamedItem("id");
          // Fetch ID Value
          if (tmp != null)
            id = tmp.getNodeValue();
          // Need unique valid ID
          if (id != null && !ids.keySet().contains(id)) {
            // Create Vertex with label
            DefaultGraphCell vertex = new DefaultGraphCell(label);
            // Add One Floating Port
            vertex.add(new DefaultPort());
            // Add ID, Vertex pair to Hashtable
            ids.put(id, vertex);
            // Add Default Attributes
            attributes.put(vertex, createDefaultAttributes());
            // Add Vertex to new Cells
            newCells.add(vertex);
          }
          
          // Create Edge
        } else if (type.equals("edge")) {
          // Fetch Source ID Node
          Node tmp = node.getAttributes().getNamedItem("from");
          // Fetch Source ID Value
          String source = null;
          if (tmp != null)
            source = tmp.getNodeValue();
          // Fetch Target ID Node
          tmp = node.getAttributes().getNamedItem("to");
          // Fetch Target ID Value
          String target = null;
          if (tmp != null)
            target = tmp.getNodeValue();
          // Create Edge with label
          DefaultEdge edge = new DefaultEdge(label);
          // Find Source Port
          if (source != null) {
            // Fetch Vertex for Source ID
            DefaultGraphCell vertex =
              (DefaultGraphCell) ids.get(source);
            if (vertex != null)
              // Connect to Source Port
              cs.connect(edge, vertex.getChildAt(0), true);
          }
          // Find Target Port
          if (target != null) {
            // Fetch Vertex for Target ID
            DefaultGraphCell vertex =
              (DefaultGraphCell) ids.get(target);
            if (vertex != null)
              // Connect to Target Port
              cs.connect(edge, vertex.getChildAt(0), false);
          }
          // Add Edge to new Cells
          newCells.add(edge);
        }
      }
    }
    // Insert the cells (View stores attributes)
    model.insert(newCells.toArray(), attributes, cs, null, null);
  }
  
  
  /**
   * Creates an attributeMap for the specified position and color.
   * @return Returns a Map Object that holds the attributes for the import.
   */
  public static Map createDefaultAttributes() {
    // Create an AttributeMap
    Map map = GraphConstants.createMap();
    // Set a Black Line Border (the Border-Attribute must be Null!)
    GraphConstants.setBorderColor(map, Color.black);
    // Return the Map
    return map;
  }
  
  
  /** Fetch Cell Label from Node.
   * @param node The desired node to get the label from.
   * @return Returns the label for the node desired.
   */ 
  protected static String getLabel(Node node) {
    String lab = null;
    NodeList children = node.getChildNodes();
    for (int j = 0; j < children.getLength(); j++) {
      Node attr = children.item(j);
      if (attr.getNodeName().equals("attr")
            && attr
            .getAttributes()
            .getNamedItem("name")
            .getNodeValue()
            .equals(
                    "Label")) {
        NodeList values = attr.getChildNodes();
        for (int k = 0; k < values.getLength(); k++) {
          if (values.item(k).getNodeName().equals("string")) {
            Node labelNode = values.item(k).getFirstChild();
            if (labelNode != null)
              lab = labelNode.getNodeValue();
          }
        }
      }
    }
    return (lab != null) ? lab : new String("");
  }
  
}
