import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;


/** Opens a saved file.
* @author sven.luzar
* @version 2.0
* To change this generated comment edit the template variable "typecomment":
* Window>Preferences>Java>Templates.
* To enable and disable the creation of type comments go to
* Window>Preferences>Java>Code Generation.
*/
public class FileOpen extends AbstractActionFile {
    
    /** Constructs a new GPGraphpad from the current graphpad.
    * @param graphpad the graphpad to be edited
    */
    public FileOpen(GPGraphpad graphpad) {
        super(graphpad);
    }
    
    /** Shows a file chooser with the
    *  file filters from the file formats
    *  to select a file.
    *
    *  Furthermore, the method uses the selected
    *  file format for the read process.
    *
    * 
    * @see com.jgraph.pad.GraphModelProviderRegistry#getGraphModelProvider() getGraphModeProvider
    * @param e the event trigger
    */
    public void actionPerformed(ActionEvent e) {
        // show the file chooser
        GPFileChooser chooser = new GPFileChooser(null);
        chooser.setDialogTitle(Translator.getString("openLabel"));
        
        // return if cancel
        int result = chooser.showOpenDialog(graphpad);
        if (result == JFileChooser.CANCEL_OPTION)
            return;
        
        // check if already open
        String name = chooser.getSelectedFile().getAbsolutePath();
        if (name != null) {
            // get all open files to test against
            GPDocument[] docs = graphpad.getAllDocuments();
            
            if (docs != null) {
                for (int i = 0; i < docs.length; i++) {
                    String docname = docs[i].getFilename();
                    // check if names are the same
                    if (docname != null && name.equals(docname)) {
                        int r = JOptionPane.showConfirmDialog(
                                                              graphpad,
                                                              Translator.getString("FileAlreadyOpenWarning"),
                                                              Translator.getString("Title"),
                                                              JOptionPane.YES_NO_OPTION);
                        // if YES then user wants to revert to previously saved version
                        if (r == JOptionPane.YES_OPTION) {
                            // close existing internal frame without saving
                            graphpad.removeGPInternalFrame(docs[i].getInternalFrame());
                            
                            // open old version to revert to
                            addDocument(chooser);
                            return;
                        }
                        // doesn't want to revert and already open so cancel open
                        else
                            return;
                    }
                }
            }
        }
        else return;
        
        // test confirms file not already open so open
        addDocument(chooser);
    }
    
    
    /** Adds a new document.
    *  Document must have valid graph mode and file format.
    *  @see com.jgraph.GPGraphpad#addDocument(String,GraphModelProvider,GPGraph,GraphModel,GraphUndoManager) addDocument
    *  @param chooser the file to add
    *  @return true if successful in adding the document, false otherwise
    */        
    protected boolean addDocument (GPFileChooser chooser){
        // get the file format, provider
        File file = chooser.getSelectedFile();
        GraphModelProvider graphModelProvider =
            GraphModelProviderRegistry.getGraphModelProvider(file);
        GraphModelFileFormat graphModelFileFormat =
            GraphModelProviderRegistry.getGraphModelFileFormat(file);
        
        // error message if no file format found
        if (graphModelFileFormat == null) {
            JOptionPane.showMessageDialog(
                                          graphpad,
                                          Translator.getString("Error.No_GraphModelFileFormat_available"),
                                          Translator.getString("Error"),
                                          JOptionPane.ERROR_MESSAGE);
            return false;
        }
        
        // extract the read properties
        Hashtable props =
            graphModelFileFormat.getReadProperties(chooser.getAccessory());
        
        // create a new and clean graph
        GPGraph gpGraph =
            graphModelProvider.createCleanGraph(
                                                graphModelProvider.createCleanGraphModel());
        
        List gpGraphList = new LinkedList();
        
        // try to read the graph model
        GraphModel model = null;
        String filename = file.getAbsolutePath();
        try {
            //			model = graphModelFileFormat.read(filename, props, gpGraph);
            if( graphModelFileFormat instanceof TPGraphModelFileFormatSerial ) {
                model = ((TPGraphModelFileFormatSerial)graphModelFileFormat).read( filename, props, gpGraphList );
            } else if( graphModelFileFormat instanceof TPGraphModelFileFormatXML ) {
                model = ((TPGraphModelFileFormatXML)graphModelFileFormat).read( filename, props, gpGraphList );
            } else {
                System.err.println( "This format is currently not supported." );
                throw new Exception( "Invalid GraphModelFileFormat used." );
            }
            
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(
                                          graphpad,
                                          ex.getLocalizedMessage(),
                                          Translator.getString("Error"),
                                          JOptionPane.ERROR_MESSAGE);
            return false;
        }
        
        // add the new document with the new graph and the new model
        graphpad.addDocument(
                             filename,
                             graphModelProvider,
                             gpGraph,
                             model,
                             null, gpGraphList );
        
        //*****begin update recent files list added 04/05/2003*****
        File recentFiles;
        
        try
        {
            String[] oldRecentList = new String[4];
            String[] newRecentList = new String[4];
            
            String temp = null;
            
            recentFiles = new File("RecentFiles.txt");
            
            BufferedReader recentFilesReader = new BufferedReader(new FileReader(recentFiles));
            
            //read from the old recent files list
            int oldCount = 0;
            
            temp = recentFilesReader.readLine();
            
            while (temp != null && oldCount < 4)
            {
                oldRecentList[oldCount] = temp.trim();
                temp = recentFilesReader.readLine();
                oldCount++;
            }
            
            for (int x = 0; x < oldCount; x++)
                System.out.println(oldRecentList[x]);
            
            recentFilesReader.close();
            
            //update new recent files list
            newRecentList[0] = filename;
            int counter = 1;
            
            for (int x = 0; x < oldCount && counter < 4; x++)
            {
                if (!oldRecentList[x].equals(filename))
                {
                    newRecentList[counter] = oldRecentList[x];
                    counter++;
                }
            }
            
            //clear the file and rewrite
            recentFiles = new File("RecentFiles.txt");
            
            BufferedWriter recentFilesWriter = new BufferedWriter(new FileWriter(recentFiles));
            //recentFilesWriter.flush();
            
            for (int x = 0; x < counter; x++)
            {
                recentFilesWriter.write(newRecentList[x], 0, newRecentList[x].length());
                recentFilesWriter.newLine();
            }
            
            recentFilesWriter.close();
        }
        catch(Exception myE)
        {
            System.out.println("Error");
        }
        //*****end update recent files list added 04/05/2003*****
        
        graphpad.update();
        
        return true;
    }
    
    /** Empty implementation.
    *  This Action should be available
    *  each time.
    */
    public void update() {
    };
    
}
