/*
 * @(#)GPConverter.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.Date;
import java.text.DateFormat;
import java.util.Hashtable;
import java.util.Iterator;


/** An image converter.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GPConverter {

	//
	// Image Converter
	//

    /** Convert a graph object into an image that can be displayed in a frame (rectangle).
     * @param graph An object that contains an image and other specifications
     * @return BufferedImage that can be used to display graphics in a frame
     */    
	public static BufferedImage toImage(GPGraph graph) {
		Object[] cells = graph.getRoots(); System.out.println("cell length="+cells.length);

		if (cells.length > 0) {// Modified by Xun if (cells.length >= 0) {
			Rectangle bounds = graph.getCellBounds(cells); 
			
			bounds = graph.toScreen(bounds);

			// Create a Buffered Image
			Dimension d = bounds.getSize();
			
      BufferedImage img = new BufferedImage(bounds.x + d.width, bounds.y + d.height, BufferedImage.TYPE_INT_RGB);            
			
			Graphics2D graphics = img.createGraphics();
			
			graph.paint(graphics);

			return img;
		}
		return null;
	}

	/** Convert a graph object into an image that can be displayed in a frame (rectangle).
		* The image returned is just the cells that are passed in as a parameter
	 	* @param graph An object that contains an image and other specifications
		* @param cells The cells that need to be copied
	 	* @return BufferedImage that can be used to display graphics in a frame
	 	*/    
	public static BufferedImage toImage(GPGraph graph, Object [] cells) 
	{
		// This method works by first coverting the entire Graphpad into a JPG
		// and the portion that is represented by the cells passed in as a parameter is
		// extracted later from the complete image.
		// There is no other way to just convert the selected portion of the graph into a JPG

		Object [] allCells = graph.getRoots() ; // all the cells in the application

		if (cells.length > 0 )
		{	
			Rectangle bounds = graph.getCellBounds(allCells ) ; // bounds of the entire graph
			//Rectangle bounds = new Rectangle (0,0,800,600) ;	
			Rectangle actualBounds = graph.getCellBounds (cells) ; // bounds of the portion that we want to copy
			graph.toScreen(bounds);

			// Create a Buffered Image of the entire graph
			Dimension d = bounds.getSize();
			BufferedImage img = new BufferedImage(bounds.x + d.width + 1, bounds.y + d.height+1, BufferedImage.TYPE_INT_RGB);            
			Graphics2D graphics = img.createGraphics();
			graphics.setColor(graph.getBackground());
			graphics.fillRect(0, 0, img.getWidth(), img.getHeight());

			// Prevent the selection from being visible to the newly created image.
			Object[] selection = graph.getSelectionCells();
			boolean gridVisible = graph.isGridVisible();
			graph.setGridVisible(false);
			graph.clearSelection();
			
			graph.paint(graphics);
			
			graph.setSelectionCells(selection);
			graph.setGridVisible(gridVisible);

			// Extract the portion of the image that we actually want.
			img = img.getSubimage (actualBounds.x, actualBounds.y, actualBounds.width, actualBounds.height) ;
			return img;
		}
		return null;
	}
	
	//
	// GXL Converter
	//

        /** A hashtable.
         */        
	static transient Hashtable hash;

        /** Create a GXL-representation for the specified cells.
         * @param graph An object that contains an image and other specifications
         * @param cells objects storing hashing information of a graph
         * @return A string containing information on nodes and edges
         */        
	public static String toGXL(GPGraph graph, Object[] cells) {
		hash = new Hashtable();
		String gxl = "<gxl><graph>";

		// Create external keys for nodes
		for (int i = 0; i < cells.length; i++)
			if (graph.isVertex(cells[i]))
				hash.put(cells[i], new Integer(hash.size()));

		// Convert Nodes
		Iterator it = hash.keySet().iterator();
		while (it.hasNext()) {
			Object node = it.next();
			gxl += vertexGXL(graph, hash.get(node), node);
		}

		// Convert Edges
		int edges = 0;
		for (int i = 0; i < cells.length; i++)
			if (graph.isEdge(cells[i]))
				gxl += edgeGXL(graph, new Integer(edges++), cells[i]);

		// Close main tags
		gxl += "\n</graph></gxl>";
		return gxl;
	}

        /** Convert an vertex's value into a string.
         * @param graph An object that contains an image and other specifications
         * @param id an indentification object
         * @param vertex an object storing information of a vertex
         * @return A string containing the id and string representation of a vertex
         */        
	public static String vertexGXL(GPGraph graph, Object id, Object vertex) {
		String label = graph.convertValueToString(vertex);
		return "\n\t<node id=\"node"
			+ id.toString()
			+ "\">"
			+ "\n\t\t<attr name=\"Label\">"
			+ "\n\t\t\t<string>"
			+ label
			+ "</string>"
			+ "\n\t\t</attr>"
			+ "\n\t</node>";
	}

        /** Convert an edge's value into a string.
         * @param graph graph An object that contains an image and other specifications
         * @param id an identification object
         * @param edge an object storing information of an edge
         * @return a string representation of an edge.
         */        
	public static String edgeGXL(GPGraph graph, Object id, Object edge) {
		GraphModel model = graph.getModel();
		String from = "";
		if (model.getSource(edge) != null) {
			Object source = hash.get(model.getParent(model.getSource(edge)));
			if (source != null)
				from = "node" + source.toString();
		}
		String to = "";
		if (model.getTarget(edge) != null) {
			Object target = hash.get(model.getParent(model.getTarget(edge)));
			if (target != null)
				to = "node" + target.toString();
		}
		if (from != null && to != null) {
			String label = graph.convertValueToString(edge);
			return "\n\t<edge id=\"edge"
				+ id.toString()
				+ "\""
				+ " from=\""
				+ from
				+ "\""
				+ " to=\""
				+ to
				+ "\">"
				+ "\n\t\t<attr name=\"Label\">"
				+ "\n\t\t\t<string>"
				+ label
				+ "</string>"
				+ "\n\t\t</attr>"
				+ "\n\t</edge>";
		} else
                    return "";
	}



	// Graphviz converter ===============================================
	/* sample syntax:
			digraph G {
				subgraph cluster0 {
					node [style=filled,color=white];
					style=filled;
					color=lightgrey;
					a0 -> a1 -> a2 -> a3;
					label = "process #1";
				}
				subgraph cluster1 {
					node [style=filled];
					b0 -> b1 -> b2 -> b3;
					label = "process #2";
					color=blue
				}
				start -> a0;
				start -> b0;
				a1 -> b3;
				b2 -> a3;
				a3 -> a0;
				a3 -> end;
				b3 -> end;
				start [shape=Mdiamond];
				end [shape=Msquare];
			}
			*/
        /** Create an graphviz formated string with information on nodes and edges.
         * @param graph An object that contains an image and other specifications
         * @param cells objects storing hashing information of a graph
         * @return a graphviz string representation of nodes and edges.
         */        
	public static String toGraphviz(GPGraph graph, Object[] cells) {
		hash = new Hashtable();
		DateFormat dateformat = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG);
		String date = dateformat.format(new Date());
		StringBuffer gv = new StringBuffer("/* Graphviz file generated by " +
			"JGraph - " + date + " */" +
			"\n\ndigraph G {");

		// Create external keys for nodes
		for (int i = 0; i < cells.length; i++)
			if (graph.isVertex(cells[i]))
				hash.put(cells[i], new Integer(hash.size()));

		// Process Nodes
		Iterator it = hash.keySet().iterator();
		while (it.hasNext()) {
			Object node = it.next();
			gv.append(vertexGraphviz(graph, hash.get(node), node));
		}

		// Process Edges
		int edges = 0;
		for (int i = 0; i < cells.length; i++)
			if (graph.isEdge(cells[i]))
				gv.append(edgeGraphviz(graph, new Integer(edges++), cells[i]));

		// Close main tags
		gv.append("\n}");
		return gv.toString();
	}

        /** Convert an vertex's value into a graphviz string.
         * @param graph An object that contains an image and other specifications
         * @param id an indentification object
         * @param vertex an object storing information of a vertex
         * @return A graphviz string containing the id and string representation of a vertex
         */      
	private static String vertexGraphviz(GPGraph graph, Object id, Object vertex) {
		if (id==null) return "";
		String label = graph.convertValueToString(vertex);
		if (label==null) label = "";
		return "\n\t" +
		       id.toString() +
			   " [label=\"" + label + "\", " +
			   "shape=\"box\"];";
	}

	/** Convert an edge's value into a graphviz string.
         * @param graph graph An object that contains an image and other specifications
         * @param id an identification object
         * @param edge an object storing information of an edge
         * @return a graphviz string representation of an edge.
         */
        private static String edgeGraphviz(GPGraph graph, Object id, Object edge) {
		GraphModel model = graph.getModel();
		String from = null;

		Object es = model.getSource(edge);
		if (es != null) {
			Object ps = hash.get(model.getParent(es));
			if (ps != null)
				from = ps.toString();
//			if (es != null)
// debug		from = graph.convertValueToString(model.getParent(es));

			String to = null;
			Object et = model.getTarget(edge);
			if (et != null) {
				Object tp = hash.get(model.getParent(et));
				if (tp != null)
					to = tp.toString();
// debug		if (et != null)
//					to = graph.convertValueToString(model.getParent(et));
			}

			if (from != null && to != null)
				return "\n\t" + from + " -> " + to + ";";
		}
		return "";
	}
}
