import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.*;

/**
 * A Document is a panel with a splitpane.
 * The splitpane shows a library resource.
 *
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class GPDocument
	extends JPanel
	implements
		GraphSelectionListener,
		ComponentListener,
		Printable,
		GraphModelListener,
		PropertyChangeListener,
		Observer {

	/** A boolean value for tooltips.
         */
	protected boolean enableTooltips;

	/** Filename for the current document.
	 *  Null if never saved or opened. *
	 *
	 */
	protected String filename;

	/** A reference to the top level
	 *  component.
	 */
	protected GPGraphpad graphpad;

	/** Splitpane between the
	 *  libraries and the graph.
	 */
	protected JSplitPane splitPane;

	/** The left side of this document that
	 *  shows the libraries.
	 *
	 */
	protected GPLibraryPanel libraryPanel;

	/** Container for the graph so that you
	 *  can scroll over the graph.
	 */
	protected JScrollPane scrollPane;

	/** The joint graph for this document.
	 */
	protected GPGraph graph;

	/** The overview Dialog for this document.
	 *  Can be <tt>null</tt>.
	 *
	 */
	protected JDialog overviewDialog;

	/** The column rule for the graph.
	 */
	protected Rule columnRule;

	/** The row rule for the graph.
	 */
	protected Rule rowRule;

	/** The graphUndoManager manager for the joint graph.
	 *
	 *  @see #graph
	 */
	protected GraphUndoManager graphUndoManager;

	/** The graphUndoManager handler for the current document.
	 *  Each document has its own handler
	 *  so you can make an graphUndoManager separate for each document.
	 *
	 */
	protected UndoHandler undoHandler;

	/** On the fly layout.
	 *
	 */
	protected Touch touch;

	/** True if this documents graph model
	 *  was modified since last save.
         */
	protected boolean modified = false;

	/** True if the current graph is Metric.
	 *  Default is true.
	 */
	protected static boolean isMetric = true;

	/** True if the library expand is expanded.
	 *  Default is true.
	 */
	protected static boolean libraryExpanded = true;

	/** True if the ruler show is activated.
	 */
	protected static boolean showRuler = true;

	/** Action used for fitting the size.
	 */
	protected Action fitAction;

	/** Contains the find pattern for this document.
	 */
	protected String findPattern;

	/** Contains the last found object.
	 */
	protected Object lastFound;

	/** A reference to the internal Frame.
	 */
	protected GPInternalFrame internalFrame;

	/** A reference to the graph model provider for
	 *  this document.
	 */
	protected GraphModelProvider graphModelProvider;

	/** Static initializer.
	 *  Initializes static variables.
	 */
	static {
		libraryExpanded =
			new Boolean(Translator.getString("LibraryExpanded")).booleanValue();
		isMetric = new Boolean(Translator.getString("IsMetric")).booleanValue();
		showRuler =
			new Boolean(Translator.getString("ShowRuler")).booleanValue();
	}

	/** Constructor for GPDocument.
         * @param graphpad a GPGraphpad object
         * @param filename a string with path to a file
         * @param graphModelProvider an GraphModelProvider object
         * @param gpGraph graph An object that contains an image and other specifications
         * @param model an graph model
         * @param undo the undo manager for the graph
         */
	public GPDocument(
		GPGraphpad graphpad,
		String filename,
		GraphModelProvider graphModelProvider,
		GPGraph gpGraph,
		GraphModel model,
		GraphUndoManager undo) {
		super(true);

		this.filename = filename;
		this.graphpad = graphpad;
		this.graphModelProvider = graphModelProvider;

		undoHandler = new UndoHandler(this, true);
		this.graphUndoManager = undo == null ? createGraphUndoManager() : undo;

		setBorder(BorderFactory.createEtchedBorder());
		setLayout(new BorderLayout());

		// create the graph
		graph = gpGraph;
		touch = new Touch(graph);
		registerListeners(graph);

		// Add this as a listener for undoable edits.
		graph.getModel().addUndoableEditListener(undoHandler);

		Component comp = this.createCenterComponent();
		this.add(comp);

		addComponentListener(this);

		setEnableTooltips(false);
		update();
	}

	/**
	 * Returns the filename.
	 * @return String type file name
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * Sets the filename.
	 * @param filename The filename to set
	 */
	public void setFilename(String filename) {
		this.filename = filename;
		updateFrameTitle();
	}

	/** Create the center component of this panel.
         * This creates a scroll-
         * pane for the current graph variable and stores the scrollpane
         * in the scrollPane variable.
         * @return a Component that contains the center component info
         */
	protected Component createCenterComponent() {
		libraryPanel = createLibrary();
		//libraryPanel.setVisible(false);
		splitPane =
			new JSplitPane(
				JSplitPane.HORIZONTAL_SPLIT,
				libraryPanel,
				createScrollPane());
		splitPane.setOneTouchExpandable(true);
		if (!libraryExpanded)
			splitPane.setDividerLocation(0.0d);
		return splitPane;
	}

        /** Create a scroll pane.
         * @return a component with a scroll pane
         */        
	protected Component createScrollPane() {
		scrollPane = new JScrollPane(graph);
		JViewport port = scrollPane.getViewport();
		try {
			String vpFlag = Translator.getString("ViewportBackingStore");
			Boolean bs = new Boolean(vpFlag);
			if (bs.booleanValue()) {
				port.setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
			} else {
				port.setScrollMode(JViewport.BLIT_SCROLL_MODE);
			}
			// deprecated
			// port.setBackingStoreEnabled(bs.booleanValue());
		} catch (MissingResourceException mre) {
			// just use the viewport default
		}
		columnRule = new Rule(Rule.HORIZONTAL, isMetric, graph);
		rowRule = new Rule(Rule.VERTICAL, isMetric, graph);
		if (showRuler) {
			scrollPane.setColumnHeaderView(columnRule);
			scrollPane.setRowHeaderView(rowRule);
		}
		return scrollPane;
	}

        /** Create a new GPLibraryPanel.
         * @return a newly created GPLibraryPanel object
         */        
	protected GPLibraryPanel createLibrary() {
		return new GPLibraryPanel(
			Translator.getString("LibraryName"),
			Utilities.tokenize(Translator.getString("LoadLibraries")),
			new Integer(Translator.getString("EntrySize")).intValue());
	}

        /** Create an undo manager.
         * @return an GraphUndoManager object
         */        
	protected GraphUndoManager createGraphUndoManager() {
		return new GraphUndoManager();
	}

	/** Fetch the editor contained in this panel.
         * @return a GPGraph object of this panel
         */
	public GPGraph getGraph() {
		return graph;
	}

	/** Returns the model of the graph.
         * @return a GraphModel object of the graph
         */
	public GraphModel getModel() {
		return graph.getModel();
	}

	/** Returns the GPGraph UI.
         * @return a GPGraphUI object
         */
	protected GPGraphUI getGraphUI() {
		return (GPGraphUI) graph.getUI();
	}

	/** Returns the view from the current graph.
         * @return a GraphLayoutCache object for the current graph
         */
	public GraphLayoutCache getGraphLayoutCache() {
		return graph.getGraphLayoutCache();
	}

	
        /** Add this documents listeners to the specified graph.
         * @param graph An object that contains an image and other specifications
         */        
	protected void registerListeners(JGraph graph) {
		graph.getSelectionModel().addGraphSelectionListener(this);
		graph.getModel().addGraphModelListener(this);
		graph.getGraphLayoutCache().addObserver(this);
		graph.addPropertyChangeListener(this);
	}

        /** Remove this documents listeners from the specified graph.
         * @param graph An object that contains an image and other specifications
         */        
	protected void unregisterListeners(JGraph graph) {
		graph.getSelectionModel().removeGraphSelectionListener(this);
		graph.getModel().removeGraphModelListener(this);
		graph.removePropertyChangeListener(this);
	}

        /** Set if current object has been modified.
         * @param modified a boolean value
         */        
	public void setModified(boolean modified) {
		this.modified = modified;
		updateFrameTitle();
		graphpad.update();
	}

	
        /** Return the title of this document as a string.
         * @return a string with the document title
         */        
	protected String getDocumentTitle() {
		String s =
			(filename != null) ? filename : Translator.getString("NewGraph");
		if (modified)
			s = s + " *";
		return Translator.getString("Title") + " - " + s;
	}

	
        /** Return the status of this document as a string.
         * @return a string representation of document status
         */        
	protected String getDocumentStatus() {
		String s = null;
		int n = graph.getSelectionCount();
		if (n > 0)
			s = n + " " + Translator.getString("Selected");
		else {
			int c = graph.getModel().getRootCount();
			if (c == 0) {
				s = Translator.getString("Empty");
			} else {
				s = c + " ";
				if (c > 1)
					s += Translator.getString("Cells");
				else
					s += Translator.getString("Cell");
				c = GPGraphpad.getGraphTools().getComponentCount(graph);
				s = s + " / " + c + " ";
				if (c > 1)
					s += Translator.getString("Component");
				else
					s += Translator.getString("Components");
			}
		}
		return s;
	}

	
        /** Return the scale of this document as a string.
         * @return a string with document scale
         */        
	protected String getDocumentScale() {
		return Integer.toString((int) (graph.getScale() * 100)) + "%";
	}

	
        /** Sets the attributes of the selected cells.
         * @param map a Map object
         */        
	public void setSelectionAttributes(Map map) 
	{
		Object[] cells =
			DefaultGraphModel.getDescendants(graph.getModel(), graph.getSelectionCells()).toArray();
		
		//Filter ports out
		java.util.List list = new ArrayList();
		for (int i = 0; i < cells.length; i++)
			if (!graph.isPort(cells[i]))
				list.add(cells[i]);
		cells = list.toArray();
		map = GraphConstants.cloneMap(map);
		map.remove(GraphConstants.BOUNDS);
		map.remove(GraphConstants.POINTS);

		Map nested = new Hashtable();
		for (int i = 0; i < cells.length; i++)
			nested.put(cells[i], GraphConstants.cloneMap(map));
		graph.getGraphLayoutCache().edit(nested, null, null, null);
	}

	/** Sets the font style on the selected cells.
         *  Bold and Italic are toggled.
         * @param style an int value representing style
         */
	public void setFontStyle(int style) {
		Object[] cells = graph.getSelectionCells();
		CellView[] views = getGraphLayoutCache().getMapping(cells);
		Map viewMap = new Hashtable();
		for (int i = 0; i < views.length; i++) {
			Map map = views[i].getAttributes();
			GraphConstants.setFontStyle(map, style);
			viewMap.put(cells[i], map);
			// FIX: Toggle Bold Italic
		}
		getGraphLayoutCache().edit(viewMap, null, null, null);
	}

	//-----------------------------------------------------------------
	// Component Listener
	//-----------------------------------------------------------------
        /** Set the fitAction with a new resize action.
         * @param e an AbstractAction object
         */        
	public void setResizeAction(AbstractAction e) {
		fitAction = e;
	}

        /** Listen for component hidden action.
         * @param e an component hidden event
         */        
	public void componentHidden(ComponentEvent e) {
	}
        
        /** Listen for component moved action.
         * @param e an component moved event
         */     
	public void componentMoved(ComponentEvent e) {
	}
        
        /** Listen for component resized action.
         * @param e an component resized event
         */     
	public void componentResized(ComponentEvent e) {
		if (fitAction != null)
			fitAction.actionPerformed(null);
	}

        /** Listen for component shown action.
         * @param e an component shown event
         */     
	public void componentShown(ComponentEvent e) {
		componentResized(e);
	}
        
        /** Set the scale for component.
         * @param scale a scale in double
         */     
	public void setScale(double scale) {
		scale = Math.max(Math.min(scale, 16), .01);
		graph.setScale(scale);
		componentResized(null);
	}

	//----------------------------------------------------------------------
	// Printable
	//----------------------------------------------------------------------

	/** Update the page format.
         *  Not from Printable interface, but related.
	 */
	public void updatePageFormat() {
		PageFormat f = graph.getPageFormat();
		columnRule.setActiveOffset((int) (f.getImageableX()));
		rowRule.setActiveOffset((int) (f.getImageableY()));
		columnRule.setActiveLength((int) (f.getImageableWidth()));
		rowRule.setActiveLength((int) (f.getImageableHeight()));
		if (graph.isPageVisible()) {
			int w = (int) (f.getWidth());
			int h = (int) (f.getHeight());
			graph.setMinimumSize(new Dimension(w + 5, h + 5));
		} else
			graph.setMinimumSize(null);
		invalidate();
		// Execute fitAction...
		componentResized(null);
		graph.repaint();
	}

        /** Print the passed in graph.
         * @param g a Graphics object that contains the image
         * @param pF a PageFormat object
         * @param page counter for number of pages
         * @return result of page status
         */        
	public int print(Graphics g, PageFormat pF, int page) {
		if (page > 0)
			return NO_SUCH_PAGE;
		graph.paint(g);
		return PAGE_EXISTS;
	}

	//
	// Listeners
	//

	
        /** PropertyChangeListener, update properties.
         * @param evt an event that changed the property
         */        
	public void propertyChange(PropertyChangeEvent evt) {
		if (graphpad != null)
			update();
	}

	
        /** GraphSelectionListener, update the event.
         * @param e a graph seletion event
         */        
	public void valueChanged(GraphSelectionEvent e) {
		if (!graph.isSelectionEmpty())
			touch.setDamper(0);
		update();
	}

	
        /** View Observer.
         * @param obs an object being observed
         * @param arg an object
         */        
	public void update(Observable obs, Object arg) {
		modified = true;
		touch.resetDamper();
		update();
	}

	
        /** GraphModelListener.
         * @param e an GraphModelEvent object
         */        
	public void graphChanged(GraphModelEvent e) {
		modified = true;
		touch.resetDamper();
		update();
		//System.out.println("Change:\n"+buttonEdge.getChange().getStoredAttributeMap());
	}

        /** Update the graphpad.
         */        
	protected void update() {
		updateFrameTitle();
		graphpad.update();
	}

	/**
	 * Returns the graphUndoManager.
	 * @return GraphUndoManager
	 */
	public GraphUndoManager getGraphUndoManager() {
		return graphUndoManager;
	}

	/**
	 * Sets the graphUndoManager.
	 * @param graphUndoManager The graphUndoManager to set
	 */
	public void setGraphUndoManager(GraphUndoManager graphUndoManager) {
		this.graphUndoManager = graphUndoManager;
	}

	/** Resets the Graph undo manager.
	 */
	public void resetGraphUndoManager() {
		graphUndoManager.discardAllEdits();
	}

	/**
	 * Returns the graphpad.
	 * @return GPGraphpad
	 */
	public GPGraphpad getGraphpad() {
		return graphpad;
	}

	/**
	 * Sets the graphpad.
	 * @param graphpad The graphpad to set
	 */
	public void setGraphpad(GPGraphpad graphpad) {
		this.graphpad = graphpad;
	}

	/**
	 * Returns the touch.
	 * @return Touch
	 */
	public Touch getTouch() {
		return touch;
	}

	/**
	 * Sets the touch.
	 * @param touch The touch to set
	 */
	public void setTouch(Touch touch) {
		this.touch = touch;
	}

	/** Returns true if the user really wants to close.
         * Gives chance to save work.
         * @param showConfirmDialog boolean for showing dialog or not
         * @return status of the close operation
         */
	public boolean close(boolean showConfirmDialog) {
		// set default to save on close
		int r = JOptionPane.YES_OPTION;

		if (modified){
			if (showConfirmDialog)
				r = JOptionPane.showConfirmDialog(
					graphpad.getFrame(),
					Translator.getString("SaveChangesDialog"),
					Translator.getString("Title"),
					JOptionPane.YES_NO_CANCEL_OPTION);

      // if yes, then save and close
			if (r == JOptionPane.YES_OPTION) 
			{
				ActionEvent e = new ActionEvent (new Object (), 0, "") ;
				graphpad.getCurrentActionMap()
					.get(Utilities.getClassNameWithoutPackage(FileSave.class)).actionPerformed(e);
				
				// Just using the String object because only the source of an event can be modified. 
				// When the user chooses the cancel option in the FileChooser in the FileSave class, the source
				// gets set to a string object. So that here we can verify that it is a String, if so the user pressed
				// cancel, do nothing, otherwise the user has saved the file.
				if (e.getSource() instanceof String)
					return false;
				
				return true ;
			}
			// if no, then don't save and just close
			else if (r == JOptionPane.NO_OPTION) {
				return true;
			}
			// all other conditions (cancel and dialog's 'X' button)
			// don't save and don't close
			else
				return false;
			}
		else
			// if not modified just close
			return true;
	}

	/** Get the library panel.
	 * @return a GPLibraryPanel object
	 */        
	public GPLibraryPanel getLibraryPanel() {
		return libraryPanel;
	}

	/** This will change the source of the actionevent to graph.
	 *
         */
	protected class EventRedirector implements ActionListener {

            /** An Action object.
             */            
		protected Action action;

                /** An event redirector.
                 * @param a an Action object
                 */                
		public EventRedirector(Action a) {
			this.action = a;
		}

                /** Performs an action being passed in.
                 * @param e an action that will be performed
                 */                
		public void actionPerformed(ActionEvent e) {
			JComponent source = graph;
			if (libraryPanel.hasFocus())
				source = libraryPanel;
			e =
				new ActionEvent(
					source,
					e.getID(),
					e.getActionCommand(),
					e.getModifiers());
			action.actionPerformed(e);
		}

	}

	/**
	 * Returns the findPattern.
	 * @return String value
	 */
	public String getFindPattern() {
		return findPattern;
	}

	/**
	 * Sets the findPattern.
	 * @param findPattern The findPattern to set
	 */
	public void setFindPattern(String findPattern) {
		this.findPattern = findPattern;
	}

	/**
	 * Returns the lastFound.
	 * @return Object
	 */
	public Object getLastFound() {
		return lastFound;
	}

	/**
	 * Sets the lastFound.
	 * @param lastFound The lastFound to set
	 */
	public void setLastFound(Object lastFound) {
		this.lastFound = lastFound;
	}

	/**
	 * Returns the overviewDialog.
	 * @return JDialog
	 */
	public JDialog getOverviewDialog() {
		return overviewDialog;
	}

	/**
	 * Sets the overviewDialog.
	 * @param overviewDialog The overviewDialog to set
	 */
	public void setOverviewDialog(JDialog overviewDialog) {
		this.overviewDialog = overviewDialog;
	}

	/**
	 * Returns the splitPane.
	 * @return JSplitPane
	 */
	public JSplitPane getSplitPane() {
		return splitPane;
	}

	/**
	 * Sets the splitPane.
	 * @param splitPane The splitPane to set
	 */
	public void setSplitPane(JSplitPane splitPane) {
		this.splitPane = splitPane;
	}

	/**
	 * Returns the scrollPane.
	 * @return JScrollPane
	 */
	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	/**
	 * Sets the scrollPane.
	 * @param scrollPane The scrollPane to set
	 */
	public void setScrollPane(JScrollPane scrollPane) {
		this.scrollPane = scrollPane;
	}

	/**
	 * Returns the columnRule.
	 * @return Rule
	 */
	public Rule getColumnRule() {
		return columnRule;
	}

	/**
	 * Returns the rowRule.
	 * @return Rule
	 */
	public Rule getRowRule() {
		return rowRule;
	}

	/**
	 * Sets the columnRule.
	 * @param columnRule The columnRule to set
	 */
	public void setColumnRule(Rule columnRule) {
		this.columnRule = columnRule;
	}

	/**
	 * Sets the rowRule.
	 * @param rowRule The rowRule to set
	 */
	public void setRowRule(Rule rowRule) {
		this.rowRule = rowRule;
	}

	/**
	 * Returns the enableTooltips.
	 * @return boolean
	 */
	public boolean isEnableTooltips() {
		return enableTooltips;
	}

	/**
	 * Sets the enableTooltips.
	 * @param enableTooltips The enableTooltips to set
	 */
	public void setEnableTooltips(boolean enableTooltips) {
		this.enableTooltips = enableTooltips;

		if (this.enableTooltips)
			ToolTipManager.sharedInstance().registerComponent(graph);
		else
			ToolTipManager.sharedInstance().unregisterComponent(graph);
	}

	/**
	 * Returns the internalFrame.
	 * @return GPInternalFrame
	 */
	public GPInternalFrame getInternalFrame() {
		return internalFrame;
	}

	/**
	 * Sets the internalFrame.
	 * @param internalFrame The internalFrame to set
	 */
	protected void setInternalFrame(GPInternalFrame internalFrame) {
		this.internalFrame = internalFrame;
	}

        /** Update the frame title.
         */        
	protected void updateFrameTitle() {
		if (this.internalFrame != null) {
			this.internalFrame.setTitle(getFrameTitle());
		}

	}

        /** Get the string representation of the frame title.
         * @return a string with frame title information
         */        
	public String getFrameTitle() {
		return (
			this.filename == null
				? Translator.getString("NewGraph")
				: filename)
			+ (modified ? "*" : "");
	}
	/**
	 * Returns the graphModelProvider.
	 * @return GraphModelProvider
	 */
	public GraphModelProvider getGraphModelProvider() {
		return graphModelProvider;
	}

}
