/*
 * @(#)GraphContext.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.util.*;


/*
 * This object is used for interactive operations such as move and copy.
 * It offers two basic operations: 1. Create temporary views for a set
 * of cells, and 2. Create temporary views for a set of edges that are
 * connected to a set of cells.<p>
 * <strong>Note:</strong>The views are consistent subgraphs. Consequently,
 * if an edge points to a selected and an unselected port, the selected
 * port is replaced by its temporary view, but the unselected port is not.
 *
 * Referenced classes of package com.jgraph.graph are
 * Port, CellView, PortView, ConnectionSet, EdgeView, CellMapper, DefaultGraphModel,
 * GraphLayoutCache, CellViewFactory and GraphConstants.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 */

/**
 * A graph context.
 */
public class GraphContext
implements CellMapper {
    
    /**
     * An instance of JGraph.
     * Reference to the parent graph. 
     */
    protected JGraph graph;
    
    /** 
     * An instance of GraphLayoutCache.
     * Reference to the graphs GraphLayoutCache. 
     */
    protected transient GraphLayoutCache graphLayoutCache;
    
    /**
     * An instance of Object.
     * Reference to the cells. 
     */
    protected Object cells[];
    
    /**
     * An instance of Set. 
     * Set of all cells including all descendants. 
     */
    protected Set cellSet;
    
    /**
     * An instance of integer. 
     * Number of all descendants without ports. 
     */
    protected int cellCount;
    
    /**
     * An instance of Set. 
     * Map of (cell, view) pairs including ports. 
     */
    protected Map views;
    
    /** 
     * Constructs a graph context for <code>cells</code> with respect to
     * the connections defined in the model, and the views in the view
     * of <code>graph</code>.
     * @param jgraph A variable of JGraph data type.
     * @param aobj A varibale of Object data type.
     * @see JGraph
     * @see Object
     */
    public GraphContext(JGraph jgraph, Object aobj[]) {
        views = new Hashtable();
        cellSet = DefaultGraphModel.getDescendants(jgraph.getModel(), aobj);
        graphLayoutCache = jgraph.getGraphLayoutCache();
        graph = jgraph;
        cells = aobj;
        for(Iterator iterator = cellSet.iterator(); iterator.hasNext();) {
            if(!(iterator.next() instanceof Port)) {
                cellCount++;
            }
        }
        
    }
    
    /** 
     * Returns <code>true</code> if this object contains no cells.
     * @return A boolean data type.
     */
    public boolean isEmpty() {
        return cells == null || cells.length == 0;
    }
    
    /** 
     * Returns the number of all objects (cells and children) in this object.
     * @return An integer data type.
     */
    public int getDescendantCount() {
        return cellCount;
    }
    
    /** 
     * Returns the graph that was passed to the constructor.
     * @return A JGraph data type.
     * @see JGraph
     */
    public JGraph getGraph() {
        return graph;
    }
    
    /** 
     * Returns the array that was passed to the constructor.
     * @return An Object[] data type.
     * @see Object
     */
    
    public Object[] getCells() {
        return cells;
    }
    
    /** 
     * Returns <code>true</code> if <code>node</code> or one of
     * its ancestors is contained in this object.
     * @param obj A variable of Object data type.
     * @return A boolean data type.
     * @see Object
     */
    public boolean contains(Object obj) {
        return cellSet.contains(obj);
    }
    
    /** 
     * Returns an new consistent array of views for <code>cells</code>.
     * @return A CellView[] data type.
     * @see CellView
     */
    public CellView[] createTemporaryCellViews() {
        CellView acellview[] = new CellView[cells.length];
        for(int i = 0; i < cells.length; i++) {
            acellview[i] = getMapping(cells[i], true);
        }
        
        return acellview;
    }
    
    /** 
     * Returns an new consistent array of views for the ports.
     * @return A PortView[] data type.
     * @see PortView
     */
    public PortView[] createTemporaryPortViews() {
        ArrayList arraylist = new ArrayList();
        for(Iterator iterator = cellSet.iterator(); iterator.hasNext();) {
            Object obj = iterator.next();
            if((obj instanceof Port) && graph.getGraphLayoutCache().isVisible(obj)) {
                arraylist.add(getMapping(obj, true));
            }
        }
        
        PortView aportview[] = new PortView[arraylist.size()];
        arraylist.toArray(aportview);
        return aportview;
    }
    
    /** 
     * Returns an new consistent array of views for the edges
     * that are connected to and not contained in <code>cells</code>.
     * @return A CellView[] data type.
     * @see CellView
     */
    public CellView[] createTemporaryContextViews() {
        return createTemporaryContextViews(cellSet);
    }
    
    /** 
     * Returns an new consistent array of views for the edges that
     * are connected to and not contained in <code>cellSet</code>.
     * @param set A variable of Set data type.
     * @return A CellView[] data type.
     * @see Set
     * @see CellView
     */
    public CellView[] createTemporaryContextViews(Set set) {
        Object aobj[] = set.toArray();
        Set set1 = DefaultGraphModel.getEdges(graph.getModel(), aobj);
        ArrayList arraylist = new ArrayList();
        for(Iterator iterator = set1.iterator(); iterator.hasNext();) {
            Object obj = iterator.next();
            if(!set.contains(obj) && graphLayoutCache.isVisible(obj) && graphLayoutCache.getMapping(obj, false) != null) {
                arraylist.add(createMapping(obj));
            }
        }
        
        CellView acellview[] = new CellView[arraylist.size()];
        arraylist.toArray(acellview);
        return acellview;
    }
    
    /** 
     * Returns the <code>CellView</code> that is mapped to <code>cell</code>
     * in the graph context. New views are created based on whether cell
     * is containes in the context. The <code>create</code>-flag is ignored.
     * @param obj A variable of Object data type.
     * @param flag A variable boolean data type.
     * @return A CellView data type.
     * @see Object
     * @see CellView
     */
    public CellView getMapping(Object obj, boolean flag) {
        if(obj != null) {
            CellView cellview = (CellView)views.get(obj);
            if(cellview != null) {
                return cellview;
            }
            if(contains(obj)) {
                return createMapping(obj);
            } else {
                return graphLayoutCache.getMapping(obj, false);
            }
        } else {
            return null;
        }
    }
    
    /** 
     * Create a mapping
     * @param obj A variable of Object data type.
     * @return A CellView data type.
     * @see Object
     * @see CellView
     */
    public CellView createMapping(Object obj) {
        CellView cellview = graphLayoutCache.getMapping(obj, false);
        CellView cellview1 = graphLayoutCache.getFactory().createView(obj, this);
        if(cellview != null) {
            cellview1.setAttributes(GraphConstants.cloneMap(cellview.getAllAttributes()));
            cellview1.refresh(false);
        }
        return cellview1;
    }
    
    /** 
     * Disconnects the edges in <code>cells</code> from the sources
     * and targets that are not in this context and returns a
     * ConnectionSet that defines the disconnection.
     * @param acellview A variable of CellView data type.
     * @return A ConnectionSet data type.
     * @see CellView
     * @see ConnectionSet
     */
    public ConnectionSet disconnect(CellView acellview[]) {
        ConnectionSet connectionset = new ConnectionSet();
        for(int i = 0; i < acellview.length; i++) {
            if(acellview[i] instanceof EdgeView) {
                EdgeView edgeview = (EdgeView)acellview[i];
                PortView portview = edgeview.getSource();
                if(GraphConstants.isDisconnectable(edgeview.getAllAttributes())) {
                    if(portview != null && GraphConstants.isDisconnectable(portview.getParentView().getAllAttributes()) && !contains(portview.getCell())) {
                        edgeview.setSource(null);
                        connectionset.disconnect(edgeview.getCell(), true);
                    }
                    portview = edgeview.getTarget();
                    if(portview != null && GraphConstants.isDisconnectable(portview.getParentView().getAllAttributes()) && !contains(portview.getCell())) {
                        edgeview.setTarget(null);
                        connectionset.disconnect(edgeview.getCell(), false);
                    }
                }
            }
        }
        
        return connectionset;
    }
    
    /** 
     * Associates <code>cell</code> with <code>view</code>
     * in the graph context.
     * @param obj A variable of Object data type.
     * @param cellview A variable of Object data type.
     * @see Object
     * @see CellView
     */
    public void putMapping(Object obj, CellView cellview) {
        views.put(obj, cellview);
    }
}
