/*
 * @(#)GraphLayoutCache.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Set;
import java.util.Stack;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;

/**
 * An object that defines the view of a graphmodel. This object
 * maps between model cells and views and provides a set of methods
 * to change these views.
 * The view may also contain its own set of attributes and is therefore
 * an extension of an Observable, which may be observed by the GraphUI.
 * It uses the model to send its changes to the command history.
 *
 * Referenced classes of package com.jgraph.graph are
 * Port, CellView, PortView, ConnectionSet, EdgeView, CellMapper, DefaultGraphModel,
 * GraphModel, AbstractCellView, CellViewFactory, GraphConstants and ParentMap.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 */

public class GraphLayoutCache extends Observable
implements CellMapper, Serializable {
    /**
     * The graph view layer edit.
     */
    public static class GraphViewLayerEdit extends AbstractUndoableEdit
    implements GraphModelEvent.GraphViewChange, GraphModelEvent.ExecutableGraphChange {
        
        /**
         * An instance of integer.
         * Set to front.
         */
        public static final int FRONT = -1;
        
        /**
         * An instance of integer.
         * Set to back.
         */
        public static final int BACK = -2;
        
        /**
         * An instance of Object.
         * Change the source.
         */
        protected Object changeSource;
        
        /**
         * An instance of Cell.
         * An array of cells.
         */
        protected transient Object cells[];
        
        /**
         * An instance of integer.
         * Array of integer for next objects.
         */
        protected transient int next[];
        
        /**
         * An instance of integer.
         * array of integer for prev objects.
         */
        protected transient int prev[];
        
        /**
         * An instance of integer.
         * The layer number
         */
        protected int layer;
        
        /**
         * Update the next layer.
         */
        protected void updateNext() {
            for(int i = 0; i < next.length; i++) {
                next[i] = layer;
            }
            
        }
        
        /**
         * Get the source.
         * @return An Object data type.
         * @see Object
         */
        public Object getSource() {
            return changeSource;
        }
        
        /**
         * Get the changed cells.
         * @return An Object data type.
         */
        public Object[] getChanged() {
            return cells;
        }
        
        /**
         * Get the context (always null).
         * @return Object data type.
         * @see Object
         */
        public Object[] getContext() {
            return null;
        }
        
        /**
         * Get the attributes.
         * @return A Map data type.
         * @see Map
         */
        public Map getAttributes() {
            return null;
        }
        
        /** redo
         * @throws CannotRedoException exception thrown in super class
         */
        public void redo()
        throws CannotRedoException {
            super.redo();
            updateNext();
            execute();
        }
        
        /**
         * An undo method
         * @throws CannotUndoException exception thrown in super class.
         * @see CannotUndoException
         */
        public void undo()
        throws CannotUndoException {
            super.undo();
            execute();
        }
        
        /**
         * An Execute method.
         */
        public void execute() {
            for(int i = 0; i < cells.length; i++) {
                List list = getParentList(cells[i]);
                if(list != null) {
                    prev[i] = list.indexOf(cells[i]);
                    if(prev[i] >= 0) {
                        list.remove(prev[i]);
                        int j = next[i];
                        if(j == -1) {
                            j = list.size();
                        } else
                            if(j == -2) {
                                j = 0;
                            }
                        list.add(j, cells[i]);
                        next[i] = prev[i];
                    }
                }
            }
            
            updateListeners();
        }
        
        /** update listeners
         */
        protected void updateListeners() {
            ((GraphLayoutCache)changeSource).setChanged();
            ((GraphLayoutCache)changeSource).notifyObservers(this);
        }
        
        /**
         * Get the parent list.
         * @param obj A variable of Object data type.
         * @return A List data type.
         * @see Object
         * @see List
         */
        protected List getParentList(Object obj) {
            if(obj instanceof CellView) {
                CellView cellview = ((CellView)obj).getParentView();
                List list = null;
                if(cellview == null) {
                    list = ((GraphLayoutCache)changeSource).roots;
                } else
                    if(cellview instanceof AbstractCellView) {
                        list = ((AbstractCellView)cellview).childViews;
                    }
                return list;
            } else {
                return null;
            }
        }
        
        /** 
         * Edit the graph view layers.
         * @param obj A variable of Object data type.
         * @param aobj A variable of Object data type.
         * @param i A variable of integer data type.
         * @see Object
         */
        public GraphViewLayerEdit(Object obj, Object aobj[], int i) {
            changeSource = obj;
            cells = aobj;
            layer = i;
            next = new int[aobj.length];
            prev = new int[aobj.length];
            updateNext();
        }
    }
    
    /** 
     * Edit the graph view.
     */
    public class GraphViewEdit extends CompoundEdit
    implements GraphModelEvent.GraphViewChange, GraphModelEvent.ExecutableGraphChange {
        
        /** 
         * An instance of Array.
         * An array of cells.
         */
        protected Object cells[];
        
        /** 
         * An instance of CellView.
         * An array of cell views.
         */
        protected CellView context[];
        
        /** 
         * An instance of CellView.
         * Array of hiddent cell views.
         */
        protected CellView hidden[];
        
        /** 
         * An instance of Map.
         * A map of attributes.
         */
        protected Map attributes;
        
        /** 
         * An instance of Object.
         * An array of visible objects.
         */
        protected Object visible[];
        
        /** 
         * An instance of Object.
         * An array of invisible objects.
         */
        protected Object invisible[];
        
        /** 
         * Checks if is significant.
         * @return A boolean data type.
         */
        public boolean isSignificant() {
            return true;
        }
        
        /** 
         * Gets source.
         * @return An Object data type.
         * @see Object
         */
        public Object getSource() {
            return GraphLayoutCache.this;
        }
        
        /** 
         * Gets changed.
         * @return An Object[] data type.
         * @see Object
         */
        public Object[] getChanged() {
            if(attributes != null) {
                return attributes.keySet().toArray();
            } else {
                return null;
            }
        }
        
        /** 
         * Gets context.
         * @return An Object[] data type.
         * @see Object
         */
        public Object[] getContext() {
            return context;
        }
        
        /** 
         * Get the attributes.
         * @return A Map data type.
         * @see Map
         */
        public Map getAttributes() {
            return attributes;
        }
        
        /** 
         * Redo Method.
         * @throws CannotRedoException exception thrown in supper.
         * @see CannotRedoException
         */
        public void redo()
        throws CannotRedoException {
            super.redo();
            execute();
        }
        
        /** 
         * Undo method.
         * @throws CannotUndoException exception thrown in supper.
         * @see CannotUndoException
         */
        public void undo()
        throws CannotUndoException {
            super.undo();
            execute();
        }
        
        /** 
         * Execute method.
         */
        public void execute() {
            if(hidden != null) {
                for(int i = 0; i < hidden.length; i++) {
                    if(hidden[i] != null) {
                        mapping.put(hidden[i].getCell(), hidden[i]);
                    }
                }
                
            }
            if(!rememberCellViews) {
                hidden = getMapping(invisible);
            }
            boolean flag = setVisibleImpl(visible, true) | setVisibleImpl(invisible, false);
            Object aobj[] = visible;
            visible = invisible;
            invisible = aobj;
            if(attributes != null) {
                attributes = handleAttributes(attributes);
            }
            if(flag) {
                updatePorts();
            }
            if(context != null) {
                for(int j = 0; j < context.length; j++) {
                    if(context[j] != null) {
                        context[j].refresh(false);
                    }
                }
                
            }
            setChanged();
            notifyObservers(this);
        }
        
        /** 
         * Edit graph view on a map.
         * @param map A variable of Map data type.
         * @see Map
         */
        public GraphViewEdit(Map map) {
            this(map, null, null);
            attributes = map;
        }
        
        /** 
         * Edit graph view.
         * @param map A variable of Map data type.
         * @param aobj A variable of Object data type.
         * @param aobj1 A variable of Map data type.
         * @see Map
         * @see Object
         */
        public GraphViewEdit(Map map, Object aobj[], Object aobj1[]) {
            attributes = map;
            if(map != null) {
                cells = map.keySet().toArray();
                Set set = DefaultGraphModel.getEdges(getModel(), cells);
                context = getMapping(set.toArray());
            }
            visible = aobj;
            invisible = aobj1;
        }
    }
    
    /**
     * An instance of boolean. 
     * Boolean indicating whether new or changed edges should be made visible if
     * their source and target vertices are visible.
     * This setting has no effect in non-partial views.
     */
    public boolean showAllEdgesForVisibleVertices;
    
    /** 
     * An instance of boolean.
     * Boolean indicating whether edges should be made visible if
     * their connected vertices become visible. This does cause new edges to
     * be displayed automatically.
     * This setting has no effect in non-partial views.
     */
    public boolean showEdgesOnShow;
    
    /** 
     * An instance of boolean.
     * Boolean indicating whether attached edges should be made invisible
     * if their source or target port is hidden.
     * This setting has no effect in non-partial views.
     */
    public boolean hideEdgesOnHide;
    
    /** 
     * An instance of boolean.
     * Boolean indicating whether edges should be made invisible if
     * their connected vertices become invisible (for example when removed).
     * This setting has no effect in non-partial views.
     */
    public boolean hideEdgesOnBecomeInvisible;
    
    /**
     * An instance of boolean. 
     * Boolean indicating whether cellviews should be remembered once visible
     * in this GraphLayoutCache.
     */
    public boolean rememberCellViews;
    
    /** 
     * An instance of GraphModel.
     * Reference to the graphModel 
     */
    protected GraphModel graphModel;
    
    /** 
     * An instance of Map.
     * Maps cells to views. 
     */
    protected Map mapping;
    
    /** 
     * An instance of CellMapper.
     * Reference to the cell mapper, typically this. 
     */
    protected CellMapper mapper;
    
    /** 
     * An instance of CellViewFactory.
     * Factory to create the views. 
     */
    protected CellViewFactory factory;
    
    /**
     * An instance of Set. 
     * The set of visible cells. 
     */
    protected Set visibleSet;
    
    /** 
     * An instance of List.
     * Ordered list of roots for the view. 
     */
    protected List roots;
    
    /** 
     * An instance of PortView.
     * Cached array of all ports for the view. 
     */
    protected PortView ports[];
    
    /**
     * An instance of Map.
     * Remebered cell views.
     */
    protected transient Map hiddenSet;
    
    /** 
     * An instance of boolean.
     * View overrides model layering. 
     */
    protected boolean ordered;
    
    /** 
     * An instance of boolean.
     * Only portions of the model are visible. 
     */
    protected boolean partial;
    
    /** 
     * Constructs a view for the specified model that uses
     * <code>factory</code> to create its views.
     * @param graphmodel A variable of GraphModel data type.
     * @param cellviewfactory A variable of CellViewFactory data type.
     * @see GraphModel
     * @see CellViewFactory
     */
    public GraphLayoutCache(GraphModel graphmodel, CellViewFactory cellviewfactory) {
        this(graphmodel, cellviewfactory, false, false);
    }
    
    /** 
     * Constructs a view for the specified model that uses
     * <code>factory</code> to create its views.
     * @param graphmodel A variable of GraphModel data type.
     * @param cellviewfactory  A variable of CellViewFactory data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @see GraphModel
     * @see CellViewFactory
     */
    public GraphLayoutCache(GraphModel graphmodel, CellViewFactory cellviewfactory, boolean flag, boolean flag1) {
        this(graphmodel, cellviewfactory, flag, flag1, true, true, true, true, true);
    }
    
    /** 
     * Constructs a view for the specified model that uses
     * <code>factory</code> to create its views.
     * @param graphmodel A variable of GraphModel data type.
     * @param cellviewfactory A variable of CellViewFactory data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @param flag2 A variable of boolean data type.
     * @param flag3 A variable of boolean data type.
     * @param flag4 A variable of boolean data type.
     * @param flag5 A variable of boolean data type.
     * @param flag6 A variable of boolean data type.
     * @see GraphModel
     * @see CellViewFactory
     */
    public GraphLayoutCache(GraphModel graphmodel, CellViewFactory cellviewfactory, boolean flag, boolean flag1, boolean flag2, boolean flag3, boolean flag4,
    boolean flag5, boolean flag6) {
        showAllEdgesForVisibleVertices = true;
        showEdgesOnShow = true;
        hideEdgesOnHide = true;
        hideEdgesOnBecomeInvisible = true;
        rememberCellViews = true;
        mapping = new Hashtable();
        factory = null;
        visibleSet = new HashSet();
        roots = new ArrayList();
        hiddenSet = new Hashtable();
        ordered = false;
        partial = false;
        factory = cellviewfactory;
        ordered = flag;
        partial = flag1;
        rememberCellViews = flag2;
        showAllEdgesForVisibleVertices = flag3;
        showEdgesOnShow = flag4;
        hideEdgesOnHide = flag5;
        hideEdgesOnBecomeInvisible = flag6;
        setModel(graphmodel);
    }
    
    /** 
     * Sets the factory that creates the cell views.
     * @param cellviewfactory A variable of CellViewFactory data type.
     * @see CellViewFactory
     */
    public void setFactory(CellViewFactory cellviewfactory) {
        factory = cellviewfactory;
    }
    
    /** 
     * Returns the factory that was passed to the constructor.
     * @return A CellViewFactory data type.
     * @see CellViewFactory
     */
    public CellViewFactory getFactory() {
        return factory;
    }
    
    /** 
     * Sets the current model.
     * @param graphmodel A variable of GraphModel data type.
     * @see GraphModel
     */
    public void setModel(GraphModel graphmodel) {
        roots.clear();
        mapping.clear();
        hiddenSet.clear();
        visibleSet.clear();
        graphModel = graphmodel;
        Object aobj[] = DefaultGraphModel.getRoots(graphmodel);
        if(!isPartial()) {
            insertRoots(getMapping(aobj, true));
        }
        if(aobj != null) {
            for(int i = 0; i < aobj.length; i++) {
                factory.updateAutoSize(getMapping(aobj[i], false));
            }
            
        }
        updatePorts();
    }
    
    
    /** 
     * Remaps all existing views using the CellViewFactory and replaces the respective root views.
     */
    public synchronized void reload() {
        ArrayList arraylist = new ArrayList();
        Hashtable hashtable = new Hashtable(mapping);
        mapping.clear();
        for(Iterator iterator = hashtable.keySet().iterator(); iterator.hasNext();) {
            Object obj = iterator.next();
            CellView cellview = (CellView)hashtable.get(obj);
            CellView cellview1 = getMapping(obj, true);
            cellview1.setAttributes(cellview.getAttributes());
            if(roots.contains(cellview)) {
                arraylist.add(cellview1);
            }
        }
        
        hiddenSet.clear();
        roots = arraylist;
    }
    
    /**
     * Returns the current model.
     * @return A GraphModel data type.
     * @see GraphModel
     */
    public GraphModel getModel() {
        return graphModel;
    }
    
    /** 
     * Returns the roots of the view.
     * @return A CellView[] data type.
     * @see CellView
     */
    public CellView[] getRoots() {
        CellView acellview[] = new CellView[roots.size()];
        roots.toArray(acellview);
        return acellview;
    }
    
    /** 
     * Return all cells that intersect the given rectangle.
     * @param rectangle A variable of Rectangle data type.
     * @return A CellView[] data type.
     * @see Rectangle
     * @see CellView
     */
    public CellView[] getRoots(java.awt.Rectangle rectangle) {
        ArrayList arraylist = new ArrayList();
        CellView acellview[] = getRoots();
        for(int i = 0; i < acellview.length; i++) {
            if(acellview[i].getBounds().intersects(rectangle)) {
                arraylist.add(acellview[i]);
            }
        }
        
        acellview = new CellView[arraylist.size()];
        arraylist.toArray(acellview);
        return acellview;
    }
    
    /** 
     * Returns the ports of the view.
     * @return A PortView[] data type.
     * @see PortView
     */
    public PortView[] getPorts() {
        return ports;
    }
    
    /**
     * Updates the cached array of ports.
     */
    protected void updatePorts() {
        Object aobj[] = DefaultGraphModel.getRoots(graphModel);
        Set set = DefaultGraphModel.getDescendants(graphModel, aobj);
        if(set != null) {
            Object aobj1[] = set.toArray();
            ArrayList arraylist = new ArrayList();
            for(int i = 0; i < aobj1.length; i++) {
                if(aobj1[i] instanceof Port) {
                    CellView cellview = getMapping(aobj1[i], false);
                    if(cellview != null) {
                        arraylist.add(cellview);
                        cellview.refresh(false);
                    }
                }
            }
            
            ports = new PortView[arraylist.size()];
            arraylist.toArray(ports);
        }
    }
    
    /** 
     * Refresh the views.
     * @param acellview A variable of CellView data type.
     * @param flag A variable of boolean data type.
     */
    public void refresh(CellView acellview[], boolean flag) {
        if(acellview != null) {
            for(int i = 0; i < acellview.length; i++) {
                refresh(acellview[i], flag);
            }
            
        }
    }
    
    /** 
     * Refresh the view.
     * @param cellview  A variable of CellView data type.
     * @param flag  A variable of boolean data type.
     * @see CellView
     */
    public void refresh(CellView cellview, boolean flag) {
        if(cellview != null) {
            cellview.refresh(flag);
            CellView acellview[] = cellview.getChildViews();
            for(int i = 0; i < acellview.length; i++) {
                refresh(acellview[i], flag);
            }
            
        }
    }
    
    /** 
     * Update the cell views.
     * @param acellview  A variable of CellView data type.
     * @see CellView
     */
    public void update(CellView acellview[]) {
        if(acellview != null) {
            for(int i = 0; i < acellview.length; i++) {
                update(acellview[i]);
            }
            
        }
    }
    
    /** 
     * Update a cell view.
     * @param cellview A variable of CellView data type.
     * @see CellView
     */
    public void update(CellView cellview) {
        if(cellview != null) {
            cellview.update();
            CellView acellview[] = cellview.getChildViews();
            for(int i = 0; i < acellview.length; i++) {
                update(acellview[i]);
            }
            
        }
    }
    
    /** 
     * Called from BasicGraphUI.ModelHandler to update the view
     * based on the specified GraphModelEvent.
     * @param graphmodelchange  A variable of GraphModelChange data type.
     * @see GraphModelChange
     */
    public void graphChanged(GraphModelEvent.GraphModelChange graphmodelchange) {
        CellView acellview[] = graphmodelchange.getViews(this);
        if(acellview != null) {
            for(int i = 0; i < acellview.length; i++) {
                if(acellview[i] != null) {
                    mapping.put(acellview[i].getCell(), acellview[i]);
                }
            }
            
            setVisibleImpl(getCells(acellview), true);
        }
        Object aobj[] = order(graphmodelchange.getChanged());
        CellView acellview1[] = getMapping(graphmodelchange.getInserted(), true);
        acellview = removeRoots(graphmodelchange.getRemoved());
        graphmodelchange.putViews(this, acellview);
        insertRoots(acellview1);
        if(isPartial()) {
            showCellsForChange(graphmodelchange);
            hideCellsForChange(graphmodelchange);
        }
        if(aobj != null && aobj.length > 0) {
            if(!isOrdered()) {
                roots.clear();
                Object aobj1[] = DefaultGraphModel.getRoots(graphModel);
                CellView acellview2[] = getMapping(aobj1, false);
                for(int l = 0; l < acellview2.length; l++) {
                    if(acellview2[l] != null) {
                        roots.add(acellview2[l]);
                        acellview2[l].refresh(true);
                        factory.updateAutoSize(acellview2[l]);
                    }
                }
                
            }
            for(int j = 0; j < aobj.length; j++) {
                CellView cellview = getMapping(aobj[j], false);
                if(cellview != null) {
                    cellview.refresh(true);
                    update(cellview);
                    factory.updateAutoSize(cellview);
                    if(isOrdered()) {
                        CellView cellview1 = cellview.getParentView();
                        Object obj = cellview1 == null ? null : cellview1.getCell();
                        boolean flag = roots.contains(cellview);
                        if(obj == null && !flag) {
                            roots.add(cellview);
                        } else
                            if(obj != null && flag) {
                                roots.remove(cellview);
                            }
                    }
                }
            }
            
        }
        Object aobj2[] = graphmodelchange.getInserted();
        if(aobj2 != null && aobj2.length > 0) {
            for(int k = 0; k < aobj2.length; k++) {
                factory.updateAutoSize(getMapping(aobj2[k], false));
            }
            
        }
        refresh(getMapping(graphmodelchange.getContext(), false), false);
        Object aobj3[] = graphmodelchange.getRemoved();
        if(aobj3 != null && aobj3.length > 0 || aobj2 != null && aobj2.length > 0 || !isOrdered()) {
            updatePorts();
        }
    }
    
    /** 
     * Prevent cells from being changed.
     * @param graphmodelchange  A variable of GraphModelChange data type.
     * @see GraphModelChange
     */
    protected void hideCellsForChange(GraphModelEvent.GraphModelChange graphmodelchange) {
        Object aobj[] = graphmodelchange.getRemoved();
        HashSet hashset = new HashSet();
        if(aobj != null) {
            for(int i = 0; i < aobj.length; i++) {
                hashset.add(aobj[i]);
            }
            
        }
        if(hideEdgesOnBecomeInvisible) {
            Object aobj1[] = graphmodelchange.getChanged();
            for(int j = 0; j < aobj1.length; j++) {
                CellView cellview = getMapping(aobj1[j], false);
                if(cellview instanceof EdgeView) {
                    EdgeView edgeview = (EdgeView)cellview;
                    Object obj = edgeview.getSource() != null ? edgeview.getSource().getCell() : null;
                    Object obj1 = edgeview.getTarget() != null ? edgeview.getTarget().getCell() : null;
                    Object obj2 = graphModel.getSource(aobj1[j]);
                    Object obj3 = graphModel.getTarget(aobj1[j]);
                    if(hashset.contains(obj) || hashset.contains(obj1) || obj2 != null && !isVisible(obj2) || obj3 != null && !isVisible(obj3)) {
                        setVisibleImpl(new Object[] {
                            aobj1[j]
                        }, false);
                    }
                }
            }
            
        }
    }
    
    /** 
     * Display the changes in cells.
     * @param graphmodelchange  A variable of GraphModelChange data type.
     * @see GraphModelChange
     */
    protected void showCellsForChange(GraphModelEvent.GraphModelChange graphmodelchange) {
        if(showAllEdgesForVisibleVertices) {
            Object aobj[] = graphmodelchange.getInserted();
            if(aobj != null) {
                for(int i = 0; i < aobj.length; i++) {
                    if(!isVisible(aobj[i]) && showAllEdgesForVisibleVertices) {
                        Object obj = graphModel.getSource(aobj[i]);
                        Object obj1 = graphModel.getTarget(aobj[i]);
                        if((obj != null || obj1 != null) && isVisible(obj) && isVisible(obj1)) {
                            setVisible(aobj[i], true);
                        }
                    }
                }
                
            }
            Object aobj1[] = graphmodelchange.getChanged();
            if(aobj1 != null) {
                for(int j = 0; j < aobj1.length; j++) {
                    if(!isVisible(aobj1[j])) {
                        Object obj2 = graphModel.getSource(aobj1[j]);
                        Object obj3 = graphModel.getTarget(aobj1[j]);
                        if((obj2 != null || obj3 != null) && isVisible(obj2) && isVisible(obj3)) {
                            setVisible(aobj1[j], true);
                        }
                    }
                }
                
            }
        }
    }
    
    /** 
     * Adds the specified model root cells to the view.
     * @param acellview  A variable of CellView data type.
     * @see CellView
     */
    public void insertRoots(CellView acellview[]) {
        if(acellview != null) {
            refresh(acellview, true);
            for(int i = 0; i < acellview.length; i++) {
                if(acellview[i] != null && getMapping(acellview[i].getCell(), false) != null) {
                    CellView cellview = acellview[i].getParentView();
                    Object obj = cellview == null ? null : cellview.getCell();
                    if(!(acellview[i] instanceof PortView) && !roots.contains(acellview[i]) && obj == null) {
                        roots.add(acellview[i]);
                    }
                }
            }
            
        }
    }
    
    /** 
     * Removes the specified model root cells from the view by removing
     * the mapping between the cell and its view and makes the cells
     * invisible.
     * @param aobj  A variable of Object data type.
     * @return A CellView[] data type.
     * @see Object
     * @see CellView
     */
    public CellView[] removeRoots(Object aobj[]) {
        if(aobj != null) {
            CellView acellview[] = new CellView[aobj.length];
            for(int i = 0; i < aobj.length; i++) {
                acellview[i] = removeMapping(aobj[i]);
                if(acellview[i] != null) {
                    acellview[i].removeFromParent();
                    roots.remove(acellview[i]);
                }
            }
            
            setVisibleImpl(aobj, false);
            return acellview;
        } else {
            return null;
        }
    }
    
    /** 
     * Takes an array of views and returns the array of the corresponding
     * cells by using <code>getCell</code> for each view.
     * @param acellview  A variable of CellView data type.
     * @return An Object[] data type.
     * @see CellView
     * @see Object
     */
    public Object[] getCells(CellView acellview[]) {
        if(acellview != null) {
            Object aobj[] = new Object[acellview.length];
            for(int i = 0; i < acellview.length; i++) {
                if(acellview[i] != null) {
                    aobj[i] = acellview[i].getCell();
                }
            }
            
            return aobj;
        } else {
            return null;
        }
    }
    
    /** 
     * Returns the view for the specified cell. If create is true
     * and no view is found then a view is created using
     * createView(Object).
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @return A CellView data type.
     * @see Object
     * @see CellView
     */
    public CellView getMapping(Object obj, boolean flag) {
        if(obj == null) {
            return null;
        }
        CellView cellview = (CellView)mapping.get(obj);
        if(cellview == null && flag) {
            cellview = (CellView)hiddenSet.get(obj);
            if(cellview != null && isVisible(obj)) {
                putMapping(obj, cellview);
                hiddenSet.remove(obj);
            } else {
                cellview = factory.createView(obj, this);
            }
        }
        return cellview;
    }
    
    /** 
     * Returns the views for the specified array of cells without
     * creating these views on the fly.
     * @param aobj  A variable of Object data type.
     * @return A CellView[] data type.
     * @see Object
     * @see CellView
     */
    public CellView[] getMapping(Object aobj[]) {
        return getMapping(aobj, false);
    }
    
    /** 
     * Returns the views for the specified array of cells. Returned
     * array may contain null pointers if the respective cell is not
     * mapped in this view and <code>create</code> is <code>false</code>.
     * @param aobj  A variable of Object data type.
     * @param flag  A variable of boolean data type.
     * @return A CellView[] daa type.
     * @see Object
     * @see CellView
     */
    public CellView[] getMapping(Object aobj[], boolean flag) {
        if(aobj != null) {
            CellView acellview[] = new CellView[aobj.length];
            for(int i = 0; i < aobj.length; i++) {
                acellview[i] = getMapping(aobj[i], flag);
            }
            
            return acellview;
        } else {
            return null;
        }
    }
    
    /** 
     * Associates the specified model cell with the specified view.
     * Updates the portlist if necessary.
     * @param obj  A variable of Object data type.
     * @param cellview  A variable of CellView data type.
     * @see Object
     * @see CellView
     */
    public void putMapping(Object obj, CellView cellview) {
        if(obj != null && cellview != null && isVisible(obj)) {
            mapping.put(obj, cellview);
        }
    }
    
    /** 
     * Removes the associaten for the specified model cell and
     * returns the view that was previously associated with the cell.
     * Updates the portlist if necessary.
     * @param obj  A variable of Object data type.
     * @return A Cell data type.
     * @see Object
     * @see Cell
     */
    public CellView removeMapping(Object obj) {
        if(obj != null) {
            CellView cellview = (CellView)mapping.remove(obj);
            return cellview;
        } else {
            return null;
        }
    }
    
    /** 
     * Null is always visible.
     * @param obj  A variable of Object data type.
     * @return A boolean data type.
     * @see Object
     */
    public boolean isVisible(Object obj) {
        return !isPartial() || visibleSet.contains(obj) || obj == null;
    }
    
    /** 
     * Return all the visible objects.
     * @return A Set data type.
     * @see Set
     */
    public Set getVisibleSet() {
        return new HashSet(visibleSet);
    }
    
    /** 
     * Set all the visible objects.
     * @param set A variable of Set data type.
     * @see Set
     */
    public void setVisibleSet(Set set) {
        visibleSet = set;
    }
    
    /** 
     * Set an object to be visible or not.
     * @param obj  A variable of Object data type.
     * @param flag  A variable of boolean data type.
     * @see Object
     */
    public void setVisible(Object obj, boolean flag) {
        setVisible(new Object[] {
            obj
        }, flag);
    }
    
    /** 
     * Set an array of objects to visible or not.
     * @param aobj  A variable of Object data type.
     * @param flag  A variable of boolean data type.
     * @see Object
     */
    public void setVisible(Object aobj[], boolean flag) {
        if(flag) {
            setVisible(aobj, ((Object []) (null)));
        } else {
            setVisible(((Object []) (null)), aobj);
        }
    }
    
    /** 
     * Set first set to visible, second set to invisible.
     * @param aobj  A variable of Object data type.
     * @param aobj1  A variable of Object data type.
     * @see Object
     */
    public void setVisible(Object aobj[], Object aobj1[]) {
        aobj = addVisibleDependencies(aobj, true);
        aobj1 = addVisibleDependencies(aobj1, false);
        GraphViewEdit graphviewedit = new GraphViewEdit(null, aobj, aobj1);
        graphviewedit.end();
        graphModel.edit(null, null, null, new UndoableEdit[] {
            graphviewedit
        });
    }
    
    /** 
     * Adds visibility dependencies.
     * @param aobj  A variable of Object data type.
     * @param flag  A variable of boolean data type.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] addVisibleDependencies(Object aobj[], boolean flag) {
        if(aobj != null) {
            if(flag) {
                HashSet hashset = new HashSet();
                for(int i = 0; i < aobj.length; i++) {
                    hashset.add(aobj[i]);
                    hashset.addAll(getPorts(aobj[i]));
                    hashset.addAll(getParentPorts(graphModel.getSource(aobj[i])));
                    hashset.addAll(getParentPorts(graphModel.getTarget(aobj[i])));
                }
                
                if(showEdgesOnShow) {
                    Set set = DefaultGraphModel.getEdges(getModel(), aobj);
                    for(Iterator iterator = set.iterator(); iterator.hasNext();) {
                        Object obj = iterator.next();
                        Object obj1 = graphModel.getSource(obj);
                        Object obj2 = graphModel.getTarget(obj);
                        if((isVisible(obj1) || hashset.contains(obj1)) && (isVisible(obj2) || hashset.contains(obj2))) {
                            hashset.add(obj);
                        }
                    }
                    
                }
                hashset.removeAll(visibleSet);
                return hashset.toArray();
            }
            if(hideEdgesOnHide) {
                HashSet hashset1 = new HashSet();
                for(int j = 0; j < aobj.length; j++) {
                    hashset1.addAll(getPorts(aobj[j]));
                    hashset1.add(aobj[j]);
                }
                
                hashset1.addAll(DefaultGraphModel.getEdges(graphModel, aobj));
                hashset1.retainAll(visibleSet);
                return hashset1.toArray();
            }
        }
        return null;
    }
    
    /** 
     * Set the visibility.
     * You must call update ports if this method returns true
     * @param aobj A variable of Object data type.
     * @param flag  A variable of boolean data type.
     * @return A boolean data type.
     * @see Object
     */
    public boolean setVisibleImpl(Object aobj[], boolean flag) {
        if(aobj != null && isPartial()) {
            boolean flag1 = false;
            for(int i = 0; i < aobj.length; i++) {
                if(aobj[i] != null) {
                    if(flag) {
                        visibleSet.add(aobj[i]);
                    } else {
                        visibleSet.remove(aobj[i]);
                    }
                }
            }
            
            for(int j = 0; j < aobj.length; j++) {
                if(aobj[j] != null) {
                    if(!flag) {
                        CellView cellview = getMapping(aobj[j], false);
                        if(cellview != null) {
                            cellview.removeFromParent();
                            cellview.refresh(false);
                            removeMapping(aobj[j]);
                            roots.remove(cellview);
                            if(graphModel.contains(aobj[j]) && rememberCellViews) {
                                hiddenSet.put(cellview.getCell(), cellview);
                            }
                            flag1 = true;
                        }
                    } else {
                        CellView cellview1 = getMapping(aobj[j], true);
                        CellView acellview[] = AbstractCellView.getDescendantViews(new CellView[] {
                            cellview1
                        });
                        for(int k = 0; k < acellview.length; k++) {
                            roots.remove(acellview[k]);
                        }
                        
                        cellview1.refresh(false);
                        factory.updateAutoSize(cellview1);
                        CellView cellview2 = cellview1.getParentView();
                        if(cellview2 != null) {
                            cellview2.refresh(true);
                        } else {
                            insertRoots(new CellView[] {
                                cellview1
                            });
                        }
                        flag1 = true;
                    }
                }
            }
            
            return flag1;
        } else {
            return false;
        }
    }
    
    /** 
     * Gets the parent ports.
     * @param obj A variable of Object data type.
     * @return A Collection data type.
     * @see Object
     * @see Connection
     */
    protected Collection getParentPorts(Object obj) {
        Object obj1 = graphModel.getParent(obj);
        Collection collection = getPorts(obj1);
        collection.add(obj1);
        return collection;
    }
    
    /** 
     * Get the ports.
     * @param obj A variable of Object data type.
     * @return A Collection data type.
     * @see Object
     * @see Connection
     */
    protected Collection getPorts(Object obj) {
        LinkedList linkedlist = new LinkedList();
        for(int i = 0; i < graphModel.getChildCount(obj); i++) {
            Object obj1 = graphModel.getChild(obj, i);
            if(obj1 instanceof Port) {
                linkedlist.add(obj1);
            }
        }
        
        return linkedlist;
    }
    
    /** 
     * Return the boolean flag for ordered.
     * @return A boolean data type.
     */
    public boolean isOrdered() {
        return ordered;
    }
    
    /** 
     * Return the boolean flag for partial.
     * @return A boolean data type.
     */
    public boolean isPartial() {
        return partial;
    }
    
    /** 
     * Inserts the <code>cells</code> and connections into the model,
     * and absorbs the local attributes. This implementation sets the
     * inserted cells visible. (Note: No undo required for this visibility
     * change.)
     * @param aobj A variable of Object data type.
     * @param map A variable of Map data type.
     * @param connectionset A variable of ConnectionSet data type.
     * @param parentmap A variable of ParentMap data type.
     * @param aundoableedit A variable of UndoableEdit data type.
     * @see Object
     * @see Map
     * @see ConnectionSet
     * @see ParentMap
     * @see UndoableEdit
     */
    public void insert(Object aobj[], Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[]) {
        Object aobj1[] = DefaultGraphModel.getDescendants(graphModel, aobj).toArray();
        Object aobj2[] = null;
        if(isPartial()) {
            HashSet hashset = new HashSet(DefaultGraphModel.getDescendants(graphModel, aobj));
            hashset.removeAll(visibleSet);
            if(!hashset.isEmpty()) {
                aobj2 = hashset.toArray();
            }
        }
        GraphViewEdit graphviewedit = createLocalEdit(map, aobj2, null);
        setVisibleImpl(aobj2, true);
        if(graphviewedit != null) {
            aundoableedit = augment(aundoableedit, graphviewedit);
        }
        graphModel.insert(aobj, map, connectionset, parentmap, aundoableedit);
    }
    
    /** 
     * Removes <code>cells</code> from the model. If <code>removeChildren</code>
     * is <code>true</code>, the children are also removed.
     * Notifies the model- and undo listeners of the change.
     * @param aobj A variable of Object data type.
     * @see Object
     */
    public void remove(Object aobj[]) {
        graphModel.remove(aobj);
    }
    
    /** 
     * Applies the <code>propertyMap</code> and the connection changes to
     * the model. The initial <code>edits</code> that triggered the call
     * are considered to be part of this transaction.
     * Notifies the model- and undo listeners of the change.
     * Note: The passed in attributes may contain PortViews.
     * @param map A variable of Map data type.
     * @param connectionset A variable of ConnectioinSet data type.
     * @param parentmap A variable of ParentMap data type.
     * @param aundoableedit A variable of UndoableEdit data type.
     * @see Map
     * @see ConnectionSet
     * @see ParentMap
     * @see UndoableEdit
     */
    public void edit(Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[]) {
        Object aobj[] = null;
        if(isPartial()) {
            HashSet hashset = new HashSet(map.keySet());
            hashset.removeAll(visibleSet);
            if(!hashset.isEmpty()) {
                aobj = hashset.toArray();
            }
        }
        GraphViewEdit graphviewedit = createLocalEdit(map, aobj, null);
        if(graphviewedit != null) {
            aundoableedit = augment(aundoableedit, graphviewedit);
        }
        graphModel.edit(map, connectionset, parentmap, aundoableedit);
    }
    
    /** 
     * Undo augment
     * @param aundoableedit A variable of UndoableEdit data type.
     * @param undoableedit A variable of UndoableEdit data type.
     * @return An UndoableEdit data type.
     * @see UndoableEdit
     */
    protected UndoableEdit[] augment(UndoableEdit aundoableedit[], UndoableEdit undoableedit) {
        if(undoableedit != null) {
            int i = aundoableedit == null ? 1 : aundoableedit.length + 1;
            UndoableEdit aundoableedit1[] = new UndoableEdit[i];
            if(aundoableedit != null) {
                System.arraycopy(aundoableedit, 0, aundoableedit1, 0, i - 2);
            }
            aundoableedit1[i - 1] = undoableedit;
            return aundoableedit1;
        } else {
            return aundoableedit;
        }
    }
    
    /** 
     * Sends <code>cells</code> to back. Note: This expects an array of cells!
     * @param aobj A variable of Object data type.
     */
    public void toBack(Object aobj[]) {
        if(aobj != null && aobj.length > 0) {
            if(isOrdered()) {
                CellView acellview[] = getMapping(aobj, false);
                GraphViewLayerEdit graphviewlayeredit = new GraphViewLayerEdit(this, acellview, -2);
                graphModel.edit(null, null, null, new UndoableEdit[] {
                    graphviewlayeredit
                });
            } else {
                graphModel.toBack(aobj);
            }
        }
    }
    
    /** 
     * Brings <code>cells</code> to front. Note: This expects an array of cells.
     * @param aobj A variable of Object data type.
     * @see Object
     */
    public void toFront(Object aobj[]) {
        if(aobj != null && aobj.length > 0) {
            if(isOrdered()) {
                CellView acellview[] = getMapping(aobj, false);
                GraphViewLayerEdit graphviewlayeredit = new GraphViewLayerEdit(this, acellview, -1);
                graphModel.edit(null, null, null, new UndoableEdit[] {
                    graphviewlayeredit
                });
            } else {
                graphModel.toFront(aobj);
            }
        }
    }
    
    /** 
     * Create a new GraphViewEdit with passed in arrays of objects
     * @param map A variable of Map data type.
     * @param aobj A variable of Object data type.
     * @param aobj1 A variable of Object data type.
     * @return A GraphViewEdit data type.
     * @see Map
     * @see Object
     * @see GraphViewEdit
     */
    protected GraphViewEdit createLocalEdit(Map map, Object aobj[], Object aobj1[]) {
        if(aobj != null || aobj1 != null) {
            GraphViewEdit graphviewedit = new GraphViewEdit(null, aobj, aobj1);
            graphviewedit.end();
            return graphviewedit;
        } else {
            return null;
        }
    }
    
    /**
     * Returns the specified cells in view-order if the model is not
     * ordered.
     * @param aobj A variable of Object data type.
     * @return An Object data type.
     * @see Object
     */
    public Object[] order(Object aobj[]) {
        if(aobj != null) {
            if(graphModel != null && isOrdered()) {
                HashSet hashset = new HashSet();
                for(int i = 0; i < aobj.length; i++) {
                    hashset.add(aobj[i]);
                }
                
                CellView acellview[] = getRoots();
                Stack stack = new Stack();
                for(int k = 0; k < acellview.length; k++) {
                    stack.add(acellview[k]);
                }
                
                ArrayList arraylist = new ArrayList();
                while(!stack.isEmpty()) {
                    CellView cellview = (CellView)stack.pop();
                    if(hashset.contains(cellview.getCell())) {
                        arraylist.add(cellview.getCell());
                    }
                    CellView acellview1[] = cellview.getChildViews();
                    for(int i1 = 0; i1 < acellview1.length; i1++) {
                        stack.add(acellview1[i1]);
                    }
                    
                }
                int l = arraylist.size();
                Object aobj2[] = new Object[l];
                for(Iterator iterator = arraylist.iterator(); iterator.hasNext();) {
                    aobj2[--l] = iterator.next();
                }
                
                return aobj2;
            }
            Object aobj1[] = new Object[aobj.length];
            for(int j = 0; j < aobj.length; j++) {
                aobj1[aobj.length - j - 1] = aobj[j];
            }
            
            return aobj1;
        } else {
            return aobj;
        }
    }
    
    /**
     * Set attributes for the cells.
     * @param map A variable of Map data type.
     * @return A Map data type.
     * @see Map
     */
    protected Map handleAttributes(Map map) {
        Hashtable hashtable = new Hashtable();
        CellView acellview[] = new CellView[map.size()];
        Iterator iterator = map.entrySet().iterator();
        int i = 0;
        while(iterator.hasNext()) {
            java.util.Map.Entry entry = (java.util.Map.Entry)iterator.next();
            CellView cellview = getMapping(entry.getKey(), false);
            acellview[i] = cellview;
            i++;
            if(cellview != null) {
                Map map1 = (Map)entry.getValue();
                Map map2 = cellview.setAttributes(map1);
                cellview.refresh(false);
                hashtable.put(cellview.getCell(), map2);
                factory.updateAutoSize(cellview);
            }
        }
        update(acellview);
        return hashtable;
    }
    
    /**
     * Translate the attributes of cell views.
     * @param acellview A variable of CellView data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @see CellView
     */
    public static void translateViews(CellView acellview[], int i, int j) {
        acellview = AbstractCellView.getDescendantViews(acellview);
        for(int k = 0; k < acellview.length; k++) {
            if(acellview[k].isLeaf()) {
                GraphConstants.translate(acellview[k].getAllAttributes(), i, j);
            }
        }
        
    }
    
    /**
     * Get all descendants.
     * @param acellview A variable of CellView data type.
     * @return A CellView data type.
     * @see CellView
     */
    public CellView[] getAllDescendants(CellView acellview[]) {
        Stack stack = new Stack();
        for(int i = 0; i < acellview.length; i++) {
            if(acellview[i] != null) {
                stack.add(acellview[i]);
            }
        }
        
        ArrayList arraylist = new ArrayList();
        while(!stack.isEmpty()) {
            CellView cellview = (CellView)stack.pop();
            CellView acellview2[] = cellview.getChildViews();
            for(int j = 0; j < acellview2.length; j++) {
                stack.add(acellview2[j]);
            }
            
            arraylist.add(cellview);
            for(int k = 0; k < graphModel.getChildCount(cellview.getCell()); k++) {
                Object obj = graphModel.getChild(cellview.getCell(), k);
                if(obj instanceof Port) {
                    CellView cellview1 = getMapping(obj, false);
                    if(cellview1 != null) {
                        stack.add(cellview1);
                    }
                }
            }
            
        }
        CellView acellview1[] = new CellView[arraylist.size()];
        arraylist.toArray(acellview1);
        return acellview1;
    }
    
    /**
     * Get the hidden set.
     * @return A Map data type.
     * @see Map
     */
    public Map getHiddenSet() {
        return hiddenSet;
    }
    
    /**
     * Return boolean for hide edges invisible.
     * @return A boolean data type.
     */
    public boolean isHideEdgesOnBecomeInvisible() {
        return hideEdgesOnBecomeInvisible;
    }
    
    /**
     * Checks hide edges on hide.
     * @return A boolean data type.
     */
    public boolean isHideEdgesOnHide() {
        return hideEdgesOnHide;
    }
    
    /**
     * Set if remember cell views.
     * @return A boolean data type.
     */
    public boolean isRememberCellViews() {
        return rememberCellViews;
    }
    
    /**
     * A boolean for showing edges for visible vertices.
     * @return A boolean data type.
     */
    public boolean isShowAllEdgesForVisibleVertices() {
        return showAllEdgesForVisibleVertices;
    }
    
    /**
     * Checks edges on show.
     * @return A boolean data type.
     */
    public boolean isShowEdgesOnShow() {
        return showEdgesOnShow;
    }
    
    /**
     * Set the hiddent set.
     * @param map A variable of Map data type.
     * @see Map
     */
    public void setHiddenSet(Map map) {
        hiddenSet = map;
    }
    
    /**
     * Set hide edges on become invisible.
     * @param flag boolean data type.
     */
    public void setHideEdgesOnBecomeInvisible(boolean flag) {
        hideEdgesOnBecomeInvisible = flag;
    }
    
    /**
     * Set hide edges on hide.
     * @param flag A boolean data type.
     */
    public void setHideEdgesOnHide(boolean flag) {
        hideEdgesOnHide = flag;
    }
    
    /**
     * Set remember cell views.
     * @param flag A variable of boolean data type.
     */
    public void setRememberCellViews(boolean flag) {
        rememberCellViews = flag;
    }
    
    /**
     * Set show all edges for visible vertices.
     * @param flag A variable of boolean data type.
     */
    public void setShowAllEdgesForVisibleVertices(boolean flag) {
        showAllEdgesForVisibleVertices = flag;
    }
    
    /**
     * Set show edges on show.
     * @param flag A variable of boolean data type.
     */
    public void setShowEdgesOnShow(boolean flag) {
        showEdgesOnShow = flag;
    }
    
    
}
