
import java.util.Iterator;
import java.util.Map;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoableEdit;

/**
 * The interface that defines a suitable data model for a JGraph.
 *
 * Referenced classes of package com.jgraph.graph are
 * ConnectionSet and ParentMap.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 */

public interface GraphModel
{
  /**
   * Retrieves the number of roots in the model.  Returns 0 if the model is empty.
   * @return An integer data type.
   * @see integer
   */
  
  public abstract int getRootCount();
  
  /** 
   * Returns the root at index in the model.
   * This should not return null if index is a valid
   * index for the model (that is index >= 0 &&
   * index < getRootCount()).
   * @param i A variable of integer data type.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getRootAt(int i);
  
  /** 
   * Returns the index of root in the model.
   * If root is null, returns -1.
   * @param obj A variable of Object data type.
   * @return An integer data type.   
   * @see Object
   */
  public abstract int getIndexOfRoot(Object obj);
  
  /**
   * Returns true if node or one of its
   * ancestors is in the model.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean contains(Object obj);
  
  /**
   * Returns a Map that represents the properties for
   * the specified cell. This properties have precedence over each
   * view's attributes, regardless of isAttributeStore.
   * @param obj A variable of Object data type.
   * @return A Map data type.
   * @see Object
   * @see Map
   */
  public abstract Map getAttributes(Object obj);
  
  /** 
   * Retrieves the source of edge. Edge must be an object
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getSource(Object obj);
  
  /** 
   * Retrieves the target of edge. Edge must be an object
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getTarget(Object obj);
  
  /** 
   * Retrieves true if port is a valid source
   * for edge. Edge and port must be
   * objects previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean acceptsSource(Object obj, Object obj1);
  
  /** 
   * Retrieves true if port is a valid target for edge. 
   * Edge and port must be
   * objects previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean acceptsTarget(Object obj, Object obj1);
  
  /** 
   * Retrieves an iterator of the edges connected to port.
   * Port must be a object previously obtained from
   * this data source. This method never returns null.
   * @param obj A variable of Object data type.
   * @return An Iterator data type.
   * @see Object
   * @see Iterator
   */
  public abstract Iterator edges(Object obj);
  
  /** 
   * Retrieves the parent of child in the model.
   * Child must be a node previously obtained from
   * this data source. This returns null if child is
   * a root in the model.
   * @param obj A variable of Object data type.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getParent(Object obj);
  
  /** 
   * Retrieves the index of child in parent.
   * If either the parent or child is null, returns -1.
   * @param obj A variable of Object data type.
   * @param obj1 A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  public abstract int getIndexOfChild(Object obj, Object obj1);
  
  /** 
   * Retrieves the child of parent at index in the parent's
   * child array. Parent must be a node previously obtained from
   * this data source. This should not return null if index
   * is a valid index for parent (that is <i>index</i> >= 0 &&
   * index < getChildCount(parent)).
   * @param obj A variable of Object data type.
   * @param i A variable of integer data type.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getChild(Object obj, int i);
  
  /** 
   * Retrieves the number of children of parent.  Returns 0 if the node
   * is a leaf or if it has no children. Parent must be a node
   * previously obtained from this data source.
   * @param obj A variable of Object data type.
   * @return An integer data type.
   * @see Object
   */
  public abstract int getChildCount(Object obj);
  
  /** 
   * Checks whether the specified node is a leaf node.
   * The way the test is performed depends on the
   * askAllowsChildren setting.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean isLeaf(Object obj);
  
  /** 
   * Inserts the cells and connections into the model,
   * and passes attributes to the views.
   * Notifies the model- and undo listeners of the change.
   * @param aobj A variable of an array of objects data type.
   * @param map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of an array of UndoableEdit data type.
   * @see Object
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   */
  public abstract void insert(Object aobj[], Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[]);
  
  /** 
   * Removes cells from the model. If removeChildren
   * is true, the children are also removed.
   * Notifies the model- and undo listeners of the change.
   * @param aobj A variable of an array of Objects data type.
   * @see Object
   */
  public abstract void remove(Object aobj[]);
  
  /** 
   * Applies the propertyMap and the connection changes to
   * the model. The initial edits that triggered the call
   * are considered to be part of this transaction.
   * Notifies the model- and undo listeners of the change.
   * Note: If only edits is non-null, the
   * edits are directly passed to the UndoableEditListeners.
   * @param map A variable of Map data type.
   * @param connectionset A variable of ConnectionSet data type.
   * @param parentmap A variable of ParentMap data type.
   * @param aundoableedit A variable of an array of UbdoableEdit data type.
   * @see Map
   * @see ConnectionSet
   * @see ParentMap
   * @see UndoableEdit
   */
  public abstract void edit(Map map, ConnectionSet connectionset, ParentMap parentmap, UndoableEdit aundoableedit[]);
  
  /** 
   * Sends cells to back.
   * @param aobj A variable of an array of Object data type.
   * @see Object
   */
  public abstract void toBack(Object aobj[]);
  
  /** 
   * Brings cells to front.
   * @param aobj A variable of an array of Object data type.
   * @see Object
   */
  public abstract void toFront(Object aobj[]);
  
  /** 
   * Adds a listener for the GraphModelEvent posted after the model changes.
   * @param graphmodellistener A variable of GraphModelListener data type.
   * @see GraphModeListener
   */
  public abstract void addGraphModelListener(GraphModelListener graphmodellistener);
  
  /** 
   * Removes a listener previously added with addGraphModelListener().
   * @param graphmodellistener A variable of GraphModelListener data type.
   * @see GraphModeListener
   */
  public abstract void removeGraphModelListener(GraphModelListener graphmodellistener);
  
  /** 
   * Adds an undo listener for notification of any changes.
   * Undo/Redo operations performed on the UndoableEdit
   * will cause the appropriate ModelEvent to be fired to keep
   * the view(s) in sync with the model.
   * @param undoableeditlistener A variable of UndoableEditListener data type.
   * @see UndoableEditListener
   */
  public abstract void addUndoableEditListener(UndoableEditListener undoableeditlistener);
  
  /** 
   * Removes an undo listener removing the given parameter from the current graph.
   * @param undoableeditlistener A variable of UndoableEditListener data type.
   * @see UndoableEditListener
   */
  public abstract void removeUndoableEditListener(UndoableEditListener undoableeditlistener);
}
