
import java.util.EventObject;
import java.util.Map;

/**
 * This class defines the event that is raised when a graph is changed. It also defines three interfaces within it.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GraphModelEvent extends EventObject
{
  /**
   * This interface describes the changes that occurs in the graph when the user performs an operation in it.
   *
   * @author TerpPresent
   * @version 2.0 TerpPresent
   */
    public static interface GraphModelChange
        extends GraphViewChange
    {

      /**
       * Returns the new attributes that were inserted into the graph.
       *
       * @return The new attributes inserted into the graph.
       */
        public abstract Object[] getInserted();

        /**
         * Returns the attributes that were removed from the graph.
         *
         * @return The attributes removed from the graph.
         */
        public abstract Object[] getRemoved();

        /**
         * Returns the attributes of the graph before the changes were made to it.
         *
         * @return A map representing the attributes of the graph prior to the change.
         */
        public abstract Map getPreviousAttributes();

        /**
         * Add all the information about the specified cells to the graphlayoutcache, which defines the view of
         * the graph model.
         *
         * @param graphlayoutcache The graph layout which is to be modified.
         * @param acellview An array of cells whose information is to be modified in the graph.
         * @see com.jgraph.graph#GraphLayoutCache
         * @see com.jgraph.graph#CellView
         */
        public abstract void putViews(GraphLayoutCache graphlayoutcache, CellView acellview[]);

        /**
         * Returns the views of all the cells in the graph.
         *
         * @param graphlayoutcache The graph layout from where the cell information is desired.
         * @return An array of CellView objects, with each element of the array describing one cell in the graph.
         */
        public abstract CellView[] getViews(GraphLayoutCache graphlayoutcache);
    }

    /**
     * An interface that describes changes made to the view of the graph. Classes implementing this interface will describe
     * the changes made to the Graph when its view is changed.
     *
     * @author TerpPresent
     * @version 2.0 TerpPresent
     */
    public static interface GraphViewChange
    {
      /**
       * Returns the cell that caused the graph view to change.
       *
       * @return The cell that caused the graph view to change.
       */
        public abstract Object getSource();

        /**
         * Returns the cells changed because of the change in the graph view.
         *
         * @return An array of all the cells that changed.
         */
        public abstract Object[] getChanged();

        /**
         * Returns a map describing all the attributes of the graph.
         *
         * @return Map describing the graph attributes.
         */
        public abstract Map getAttributes();

        /**
         * Returns the current context of the graph.
         *
         * @return An array of objects describing the current graph context.
         */
        public abstract Object[] getContext();
    }

    /**
     * This interface is used to execute the operation that was performed on the graph and was reponsible for causing the
     * graph in the graph model.
     *
     * @author TerpPresent
     * @version 2.0 TerpPresent
     */
    public static interface ExecutableGraphChange
    {
      /**
       * This method when executed changes the state of the graph.
       */
      public abstract void execute();
    }


    /**
     * Describes the changes that have been made to the graph.
     */
    protected GraphModelChange change;

    /**
     * Constructor for the graph model event.
     *
     * @param obj The component of the graph that caused the event to be fired. This is the event object.
     * @param graphmodelchange The changes made to the graph model.
     */
    public GraphModelEvent(Object obj, GraphModelChange graphmodelchange)
    {
        super(obj);
        change = graphmodelchange;
    }

    /**
     * Returns a GraphModelChange object describing the changes to the graph model.
     *
     * @return The changes made to the graph model.
     */
    public GraphModelChange getChange()
    {
        return change;
    }
}
