import java.util.EventObject;

/**
 * Describes the event that gets fired when there is a new selection in the graph.
 *
 * @author TerpPresent team
 * @version 2.0 TerpPresent
 */
public class GraphSelectionEvent extends EventObject
{
  /**
   * An array of cells in the graph.
   */
  protected Object cells[];

  /**
   * An array of boolean variables which stores whether a cell is new or not.
   */
  protected boolean areNew[];

  /**
   * Constructor. Creates a new GraphSelectionEvent object.
   *
   * @param obj An EventObject associated with this event. This is an object of GraphSelectionEvent's superclass.
   * @param aobj The array of cells that make up the graph.
   * @param aflag An array of boolean values that specifies which cells are new and which aren't.
   */
    public GraphSelectionEvent(Object obj, Object aobj[], boolean aflag[])
    {
        super(obj);
        cells = aobj;
        areNew = aflag;
    }

    /**
     * Returns all the cells in the graph.
     *
     * @return A new array of objects where each element is one cell in the graph.
     */
    public Object[] getCells()
    {
        int i = cells.length;
        Object aobj[] = new Object[i];
        System.arraycopy(((Object) (cells)), 0, ((Object) (aobj)), 0, i);
        return aobj;
    }

    /**
     * Returns the first cell in the graph.
     *
     * @return The first cell in graph.
     */
    public Object getCell()
    {
        return cells[0];
    }

    /**
     * Determines whether any cells have been added to the graph.
     *
     * @return True if there are one or more cells on the graph, false otherwise.
     */
    public boolean isAddedCell()
    {
        return areNew[0];
    }


    /**
     * Determines whether the specified cell exists in the graph.
     *
     * @param obj Cell that has to be found in the graph.
     *
     * @return True if the cell exists on the graph, false otherwise.
     * @throws IllegalArgumentException If the cell is not found in the graph.
     */
    public boolean isAddedCell(Object obj)
    {
        for(int i = cells.length - 1; i >= 0; i--)
        {
            if(cells[i].equals(obj))
            {
                return areNew[i];
            }
        }

        throw new IllegalArgumentException("cell is not a cell identified by the GraphSelectionEvent");
    }

    /**
     * Determines whether a cell has just been added at the specified index.
     *
     * @param i The index at which a cell is being searched for.
     * @return True if the cell at index i is newly added, false otherwise..
     * @exception IllegalArgumentException If the index is greater than the number of cells on the board or is less than zero.
     */
    public boolean isAddedCell(int i)
    {
        if(cells == null || i < 0 || i >= cells.length)
        {
            throw new IllegalArgumentException("index is beyond range of added cells identified by GraphSelectionEvent");
        } else
        {
            return areNew[i];
        }
    }

    /**
     * Returns a clone of the object specified.
     *
     * @param obj The object to be cloned.
     * @return The cloned object.
     */
    public Object cloneWithSource(Object obj)
    {
        return new GraphSelectionEvent(obj, cells, areNew);
    }
}
