
import java.beans.PropertyChangeListener;

/**
 * This interface represents the current state of the selection for
 * the graph component. 
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 */

public interface GraphSelectionModel
  
{
  
  /** 
   * Selection can only contain one cell at a time. 
   */
  public static final int SINGLE_GRAPH_SELECTION = 1;

  /** 
   * Selection can contain any number of items. 
   */
  public static final int MULTIPLE_GRAPH_SELECTION = 4;
  
  /** 
   * Sets the selection model, which must be either
   * SINGLE_GRAPH_SELECTION or MULTIPLE_GRAPH_SELECTION.
   * This may change the selection if the current selection is not valid
   * for the new mode.
   * @param i A variable of integer data type.
   * @see integer
   */
  public abstract void setSelectionMode(int i);
  
  /** 
   * Sets if the selection model allows the selection
   * of children.
   * @param flag A variable of boolean data type.
   */
  public abstract void setChildrenSelectable(boolean flag);
  
  /** 
   * Retrieves true if the selection model allows the selection
   * of children.
   * @return A boolean data type.
   */
  public abstract boolean isChildrenSelectable();
  
  /** 
   * Retrieves the current selection mode, either
   * SINGLE_GRAPH_SELECTION or MULTIPLE_GRAPH_SELECTION.
   * @return An integer data type.
   */
  public abstract int getSelectionMode();
  
  /** 
   * Sets the selection to cell. If this represents a change, then
   * the GraphSelectionListeners are notified. If cell is
   * null, this has the same effect as invoking clearSelection.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public abstract void setSelectionCell(Object obj);
  
  /** 
   * Sets the selection to cells. If this represents a change, then
   * the GraphSelectionListeners are notified. If cells is
   * null, this has the same effect as invoking clearSelection.
   * @param aobj A variable of Object data type.
   * @see Object
   */
  public abstract void setSelectionCells(Object aobj[]);
  
  /** 
   * Adds cell to the current selection. If cell is not currently
   * in the selection the GraphSelectionListeners are notified. This has
   * no effect if cell is null.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public abstract void addSelectionCell(Object obj);
  
  /** 
   * Adds cells to the current selection.  If any of the cells are
   * not currently in the selection the GraphSelectionListeners
   * are notified. This has no effect if cells is null.
   * @param obj A variable of an array of Object data type.
   * @see Object
   */
  public abstract void addSelectionCells(Object aobj[]);
  
  /** 
   * Removes cell from the selection. If cell is in the selection
   * the GraphSelectionListeners are notified. This has no effect if
   * cell is null.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public abstract void removeSelectionCell(Object obj);
  
  /** 
   * Removes cells from the selection.  If any of the cells in
   * cells are in the selection, the GraphSelectionListeners 
   * are notified. This method has no effect if cells is null.
   * @param obj A variable of Object data type.
   * @see Object
   */
  public abstract void removeSelectionCells(Object aobj[]);
  
  /** 
   * Retrieves the cells that are currently selectable.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object[] getSelectables();
  
  /** 
   * Retrieves the first cell in the selection. How first is defined is
   * up to implementors.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object getSelectionCell();
  
  /** 
   * Retrieves the cells in the selection. This will return null (or an
   * empty array) if nothing is currently selected.
   * @return An Object data type.
   * @see Object
   */
  public abstract Object[] getSelectionCells();
  
  /** 
   * Retrieves the number of cells that are selected.
   * @return An integer data type.
   * @see integer
   */
  public abstract int getSelectionCount();
  
  /** 
   * Retrieves true if the cell, cell is in the current
   * selection.
   * @param A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean isCellSelected(Object obj);
  
  /** 
   * Retrieves true if the cell, cell
   * has selected children.
   * @param A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean isChildrenSelected(Object obj);
  
  /** Retrieves true if the selection is currently empty.
   * @return A boolean data type.
   */
  public abstract boolean isSelectionEmpty();
  
  /**
   * Empties the current selection.  If this represents a change in the
   * current selection, the selection listeners are notified.
   */
  public abstract void clearSelection();
  
  /** 
   * Adds a PropertyChangeListener to the listener list.
   * The listener is registered for all properties.
   * A PropertyChangeEvent will get fired when the selection mode
   * changes.
   * @param propertychangelistener A variable of PropertyChangeListener data type.
   * @see PropertyChangeListener
   */
  public abstract void addPropertyChangeListener(PropertyChangeListener propertychangelistener);
  
  /** 
   * Removes a PropertyChangeListener from the listener list.
   * This removes a PropertyChangeListener that was registered
   * for all properties.
   * @param propertychangelistener A variable of PropertyChangeListener data type.
   * @see PropertyChangeListener 
   */
  public abstract void removePropertyChangeListener(PropertyChangeListener propertychangelistener);
  
  /** 
   * Adds x to the list of listeners that are notified each time the
   * set of selected Objects changes.
   * @param graphselectionlistener A variable of GraphSelectionListener data type
   * @see GraphSelectionListener
   */
  public abstract void addGraphSelectionListener(GraphSelectionListener graphselectionlistener);
  
  /** 
   * Removes x from the list of listeners that are notified each time
   * the set of selected Objects changes.
   * @param graphselectionlistener A variable of GraphSelectionListener data type
   * @see GraphSelectionListener
   */
  public abstract void removeGraphSelectionListener(GraphSelectionListener graphselectionlistener);
}
