/*
 * @(#)GraphTransferable.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Rectangle;
import java.awt.datatransfer.*;
import java.io.Serializable;
import java.util.Map;

/**
 * An object that represents the clipboard contents for a graph selection.
 * The object has three representations:
 * <p>
 * 1. Richer: The cells, view attributes and connections for this selection are
 * stored as separate datastructures, which can be inserted using
 * the GraphModel.insert() method.
 * 2. HTML: If one cell is selected, the userObject is returned as HTML.
 * 3. Plain: The userObject of the selected cell is returned as plain text.
 *
 * Referenced classes of package com.jgraph.graph are
 * DefaultGraphCell and ConnectionSet.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see BasicTransferable
 * @see Serializable
 * @see ClipboardOwner
 */
public class GraphTransferable extends BasicTransferable
implements Serializable, ClipboardOwner {
    
    /**
     * An instance of DataFlavor.
     * Local Machine Reference Data Flavor.
     */
    public static DataFlavor dataFlavor ;
    
    /**
     * An instance of Object.
     * Selected cells.
     */
    protected Object cells[];
    
    /**
     * An instance of ConnectionSet.
     * Object that describes the connection between cells.
     */
    protected ConnectionSet cs;
    
    /**
     * An instance of Map.
     * (Cell, Map) entries that hold the view attributes for the cells.
     */
    protected Map attributeMap;
    
    /**
     * An instance of Rectangle.
     * Rectangle that defines the former bounds of the views.
     */
    protected Rectangle bounds;
    
    /** 
     * Constructs a new transferable selection for <code>cells</code>,
     * <code>cs</code>and <code>attrMap</code>.
     * @param aobj A variable of Object data type.
     * @param connectionset A variable of ConnectionSet data type.
     * @param map A variable of Map data type.
     * @param rectangle A variable of Rectangle data type.
     * @see Object
     * @see ConnectionSet
     * @see Map
     * @see Rectangle
     */
    public GraphTransferable(Object aobj[], ConnectionSet connectionset, Map map, Rectangle rectangle) {
        attributeMap = map;
        bounds = rectangle;
        cells = aobj;
        cs = connectionset;
    }
    
    /** 
     * Returns the <code>cells</code> that represent the selection.
     * @return An Object[] data type.
     * @see Object
     */
    public Object[] getCells() {
        return cells;
    }
    
    /** 
     * Returns the connections between <code>cells</code> (and possibly
     * other, unselected cells).
     * @return A ConnectionSet data type.
     * @see ConnectionSet
     */
    public ConnectionSet getConnectionSet() {
        return cs;
    }
    
    /** 
     * Returns a map of (GraphCell, Map)-pairs that represent the
     * view attributes for the respecive cells.
     * @return A Map data type.
     * @see Map
     */
    public Map getAttributeMap() {
        return attributeMap;
    }
    
    /** 
     * Get the rectangle bounds
     * @return A Rectangle data type.
     * @see Rectangle
     */
    public Rectangle getBounds() {
        return bounds;
    }
    
    /** 
     * Empty function.
     * @param clipboard A variable of Clipboard data type.
     * @param transferable A variable of Transferable data type.
     * @see Clipboard
     * @see Transferable
     */
    public void lostOwnership(Clipboard clipboard, Transferable transferable) {
    }
    
    /** 
     * Returns the jvm-localreference flavors of the transferable.
     * @return A DataFlavor[] data type.
     * @see DataFlavor
     */
    public DataFlavor[] getRicherFlavors() {
        return (new DataFlavor[] {dataFlavor});
    }
    
    /** 
     * Fetch the data in a jvm-localreference format.
     * @param dataflavor A variable of DataFlavor data type.
     * @throws UnsupportedFlavorException if dataflavor not equal.
     * @return An Object data type.
     * @see DataFlavor
     * @see UnsupportedFlavorException
     * @see Object
     */
    public Object getRicherData(DataFlavor dataflavor)
    throws UnsupportedFlavorException {
        if(dataflavor.equals(dataFlavor)) {
            return this;
        } else {
            throw new UnsupportedFlavorException(dataflavor);
        }
    }
    
    /** 
     * Returns true if the transferable support a text/plain format.
     * @return A boolean data type.
     */
    public boolean isPlainSupported() {
        return cells != null && cells.length == 1;
    }
    
    /** 
     * Fetch the data in a text/plain format.
     * @return A String data type.
     */
    public String getPlainData() {
        StringBuffer stringbuffer = new StringBuffer();
        if(cells[0] instanceof DefaultGraphCell) {
            Object obj = ((DefaultGraphCell)cells[0]).getUserObject();
            if(obj != null) {
                return obj.toString();
            }
        }
        return cells[0].toString();
    }
    
    /** 
     * Returns true if the transferable support a text/html format.
     * @return A boolean data type.
     */
    public boolean isHTMLSupported() {
        return isPlainSupported();
    }
    
    /** 
     * Fetch the data in a text/html format.
     * @return A String data type.
     */
    public String getHTMLData() {
        StringBuffer stringbuffer = new StringBuffer();
        stringbuffer.append("<html><body><p>");
        stringbuffer.append(getPlainData());
        stringbuffer.append("</p></body></html>");
        return stringbuffer.toString();
    }
    
    /*
     * Local Machine Reference Data Flavor.
     */
    static {
        DataFlavor dataflavor;
        dataFlavor = DataFlavor.imageFlavor ;
        try {
            dataflavor = new DataFlavor("application/x-java-jvm-local-objectref; class=GraphTransferable");
        }
        catch(ClassNotFoundException classnotfoundexception) {
            dataflavor = null;
        }
        dataFlavor = dataflavor;
    }
}
