/*
 * @(#)GraphUndoManager.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

//import java.util.Vector;
import javax.swing.undo.*;

/**
 * An UndoManager that may be shared among multiple GraphLayoutCache's.
 *
 * Referenced class of package com.jgraph.graph is
 * GraphLayoutCache.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see UndoManager
 */
public class GraphUndoManager extends UndoManager {
    
    /**
     * Empty method.
     */
    public GraphUndoManager() {
    }
    
    /**
     * Overridden to preserve usual semantics: returns true if an undo
     * operation would be successful now for the given view, false otherwise
     * @param obj A variable of  data type.
     * @return A boolean data type.
     * @see Object
     */
    public synchronized boolean canUndo(Object obj) {
        if(isInProgress()) {
            UndoableEdit undoableedit = editToBeUndone(obj);
            return undoableedit != null && undoableedit.canUndo();
        } else {
            return super.canUndo();
        }
    }
    
    /** 
     * Overridden to preserve usual semantics: returns true if a redo
     * operation would be successful now for the given view, false otherwise
     * @param obj A variable of Object data type.
     * @return A boolean data type.
     * @see Object
     */
    public synchronized boolean canRedo(Object obj) {
        if(isInProgress()) {
            UndoableEdit undoableedit = editToBeRedone(obj);
            return undoableedit != null && undoableedit.canRedo();
        } else {
            return super.canRedo();
        }
    }
    
    /** 
     * If this UndoManager is inProgress, undo the last significant
     * UndoableEdit wrt. to source, and all insignificant edits back to
     * it. Updates indexOfNextAdd accordingly.
     * <p>If not inProgress, indexOfNextAdd is ignored and super's routine is
     * called.</p>
     * @param obj A variable of Object data type.
     * @throws CannotUndoException
     * @see Object
     * @see CannotUndoException
     */
    public void undo(Object obj) {
        if(obj == null || !isInProgress()) {
            super.undo();
        } else {
            UndoableEdit undoableedit = editToBeUndone(obj);
            if(undoableedit == null) {
                throw new CannotUndoException();
            }
            undoTo(undoableedit);
        }
    }
    
    /** 
     * Get the edit to be undone.
     * @param obj A variable of Object data type.
     * @return An UndoableEdit data type.
     * @see Object
     * @see UndoableEdit
     */
    protected UndoableEdit editToBeUndone(Object obj) {
        UndoableEdit undoableedit = null;
        Object obj1 = null;
        do {
            undoableedit = nextEditToBeUndone(undoableedit);
            if(undoableedit instanceof GraphModelEvent.GraphViewChange) {
                obj1 = ((GraphModelEvent.GraphViewChange)undoableedit).getSource();
            }
            if(!(obj1 instanceof GraphLayoutCache)) {
                obj1 = null;
            }
        } while(undoableedit != null && obj1 != null && obj1 != obj);
        return undoableedit;
    }
    
    /** 
     * Returns the the next significant edit wrt. to current
     * to be undone if undo is called. May return null.
     * @param undoableedit A variable of UndoableEdit data type.
     * @return An UndoableEdit data type.
     * @see UndoableEdit
     */
    protected UndoableEdit nextEditToBeUndone(UndoableEdit undoableedit) {
        if(undoableedit == null) {
            return editToBeUndone();
        }
        int i = edits.indexOf(undoableedit) - 1;
        if(i >= 0) {
            return (UndoableEdit)edits.get(i);
        } else {
            return null;
        }
    }
    
    /** 
     * If this <code>UndoManager</code> is <code>inProgress</code>,
     * redoes the last significant <code>UndoableEdit</code> with
     * respect to source or after, and all insignificant
     * edits up to it. Updates <code>indexOfNextAdd</code> accordingly.
     * <p>If not <code>inProgress</code>, <code>indexOfNextAdd</code>
     * is ignored and super's routine is called.</p>
     * @param obj A variable of Object data type.
     * @see Object
     */
    public void redo(Object obj) {
        if(obj == null || !isInProgress()) {
            super.redo();
        } else {
            UndoableEdit undoableedit = editToBeRedone(obj);
            if(undoableedit == null) {
                throw new CannotRedoException();
            }
            redoTo(undoableedit);
        }
    }
    
    /** 
     * Get the edit to be redone.
     * @param obj A variable of Object data type.
     * @return An UndoableEdit
     * @see UndoableEDit
     */
    protected UndoableEdit editToBeRedone(Object obj) {
        UndoableEdit undoableedit = nextEditToBeRedone(null);
        UndoableEdit undoableedit1 = null;
        Object obj1 = null;
        do {
            undoableedit1 = undoableedit;
            undoableedit = nextEditToBeRedone(undoableedit);
            if(undoableedit instanceof GraphModelEvent.GraphViewChange) {
                obj1 = ((GraphModelEvent.GraphViewChange)undoableedit).getSource();
            }
            if(!(obj1 instanceof GraphLayoutCache)) {
                obj1 = null;
            }
        } while(undoableedit != null && obj1 != null && obj1 != obj);
        return undoableedit1;
    }
    
    /** 
     * Returns the the next significant edit wrt. to current
     * to be redone if redo is called. May return null.
     * @param undoableedit A variable od UndoAbleEdit data type.
     * @return An UndoableEdit.
     * @see UndoableEdit
     */
    protected UndoableEdit nextEditToBeRedone(UndoableEdit undoableedit) {
        if(undoableedit == null) {
            return editToBeRedone();
        }
        int i = edits.indexOf(undoableedit) + 1;
        if(i < edits.size()) {
            return (UndoableEdit)edits.get(i);
        } else {
            return null;
        }
    }
}
