/*
 * @(#)ImageLoader.java	1.0 23/01/02
 *
 * Copyright (C) 2003 Sven Luzar
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
import java.awt.Image;
import java.net.*;
import java.util.*;

import javax.swing.ImageIcon;

/**Loader for the resource images. The class
 * uses the getResource Method to get the
 * Resource from the relative path.
 *
 * @author sven.luzar
 * @version 2.0 TerpPresent
 * @see java.lang.ClassLoader#getResource
 *
 */
public class ImageLoader {

  /** A stack of paths where the program searches for images.
   */
  protected static Stack searchPath = new Stack();

  /** Pushes the default searh path resources to the stack.
   *  NOTE: The searchpath that gets pushed inside the code will only for unix. This needs to be modified such that it works
   *  on all the other platforms too.
   *
   *  @see #pushSearchPath
   */
  static {
    pushSearchPath(".");
  }

  /** Returns an Image from the same path.
   *  @param imageName An image name with the file extension .gif
   *
   *  @returns Returns the image specfied by imageName.
   *
   */
  public static Image getImage(String imageName){
    return getImageIcon(imageName).getImage() ;
  }

  /** Returns the ImageIcon from the top of the stack.
   *  @param imageName An image name with the file extension .gif
   *
   *  @return ImageIcon object of the image specified in the parameter.
   *
   */
  public static ImageIcon getImageIcon(String imageName){
    return getImageIcon(searchPath.size()-1, imageName);
  }

  /** Returns an ImageIcon from the search path specified.
   *
   *  @param imageName An image name with the file extension .gif
   *  @param searchPathIndex The index of the searchPath on the stack that is to be used to search for the image.
   *
   *  @return An ImageIcon object at the specified searchpath and with name imageName.
   *
   */
  public static ImageIcon getImageIcon(int searchPathIndex , String imageName){
    if (searchPathIndex < searchPath.size() && searchPathIndex >= 0){
      URL url = ImageLoader.class.getResource(((String)searchPath.get(searchPathIndex)) + imageName) ;
      if (url != null){
        return new ImageIcon(url);
      } else {
        return getImageIcon(searchPathIndex - 1, imageName);
      }
    } else {
        return null;
    }
  }

  /** If the path is not null, pushes the specified path on the stack of search paths. Else, it does nothing.
   *
   *  A sample search path is '/'.
   *
   * @param path An additional location to searh for files.
   *
   */
  public static void pushSearchPath(String path){
    if (path == null)
      return;

    if (!path.endsWith("/")){
      path = path + "/";
    }

    searchPath.push(path);
  }

  /** Removes the searchpath at the specified index from the stack.
   *
   *  @param index The index on the stack at which the path should be removed.
   */
  public static void removeSearchPath(int index){
    searchPath.remove(index);
  }

  /** Pops the top most search path from the stack.
   */
  public static void popSearchPath(){
    searchPath.pop() ;
  }
}
