
import java.awt.*;
import java.awt.event.MouseEvent;
import java.io.*;
import java.util.*;
import javax.accessibility.Accessible;
import javax.swing.*;
//import javax.swing.event.EventListenerList;
//import javax.swing.plaf.ComponentUI;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * This class describes the operations that can be performed on the graph. It is one of the most important classes for
 * the application.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class JGraph extends JComponent
    implements CellViewFactory, Scrollable, Accessible, Serializable
{
    /**
     * The GraphSelectionRedirector class handles the events when a value in the graph changes.
     *
     * @author TerpPresent
     * @version 2.0 TerpPresent.
     */
    protected class GraphSelectionRedirector
        implements Serializable, GraphSelectionListener
    {

      /**
       * This method is called when a value on the graph changes. A clone of the event is used to call the fireValueChanged
       * method.
       *
       * @param graphselectionevent The graphselectionevent that is fired when a value in the graph is changed.
       */
      public void valueChanged(GraphSelectionEvent graphselectionevent)
      {
        GraphSelectionEvent graphselectionevent1 = (GraphSelectionEvent)graphselectionevent.cloneWithSource(JGraph.this);
        fireValueChanged(graphselectionevent1);
      }

      /**
       * Creates a new GraphSelectionRedirector object. NOTE: THIS METHOD IS EMPTY.
       */
      protected GraphSelectionRedirector()
      {
      }
    }

    /**
     * This class is used to describe the cells that the user selects on the graph.
     *
     * @author TerpPresent
     * @version 2.0 TerpPresent
     */
    public static class EmptySelectionModel extends DefaultGraphSelectionModel
    {
      /**
       * A variable used to describe the selection model.
       *
       * @see #setSelectionCells (Object [])
       * @see #addSelectionCells (Object [])
       * @see #removeSelectionCells (Object [])
       */
      protected static final EmptySelectionModel sharedInstance = new EmptySelectionModel();

      /**
       * Returns the sharedInstance variable used in this class to describe the selected cells.
       *
       * @return The sharedInstance object.
       */
        public static EmptySelectionModel sharedInstance()
        {
            return sharedInstance;
        }

        /**
         * Sets the cells specified in the parameter as the ones selected.
         * NOTE: THIS METHOD IS EMPTY.
         *
         * @param aobj An array of cells that have been selected.
         */
        public void setSelectionCells(Object aobj[])
        {
        }

        /**
          * Adds the cells specified in the parameter to the list of selected cells.
          * NOTE: THIS METHOD IS EMPTY.
          *
          * @param aobj An array of cells that have been selected.
          */
        public void addSelectionCells(Object aobj[])
        {
        }

        /**
         * Removes the cells specified in the parameter from the list of selected cells.
         * NOTE: THIS METHOD IS EMPTY.
         *
         * @param aobj The list of cells to be removed.
         */
        public void removeSelectionCells(Object aobj[])
        {
        }


        /**
         * Creates a new EmptySelectionModel object.
         */
        public EmptySelectionModel()
        {
            super(null);
        }
    }

    /**
     * The version of this application.
     */
    public static final String VERSION = "JGraph Geneva (v2.0)";

    /**
     * The id for the class that describes the UI for the application.
     */
    private static final String uiClassID = "GraphUI";

    /**
     * A GraphSelectionRedirector object. This redirector is used whenever a selection in the graph changes.
     */
    protected transient GraphSelectionRedirector selectionRedirector;

    /**
     * A GraphModel object that describes the graph.
     */
    protected transient GraphModel graphModel;

    /**
     * A GraphLayoutCache object that describes how the UI for the graph has been laid out.
     */
    protected transient GraphLayoutCache graphLayoutCache;

    /**
     * A MarqueeHandler for the class.
     */
    protected transient BasicMarqueeHandler marquee;

    /**
     * A GraphSelectionModel object that describes how this application handles selections.
     */
    protected transient GraphSelectionModel selectionModel;

    /**
     * A double variable that stores the current scale of the graphpad.
     */
    protected double scale;

    /**
     * A boolean value that is true or false depending on whether the graph has been aliased or not.
     */
    protected boolean antiAliased;

    /**
     * A boolean variable that indicates whether the graphpad is editable or not.
     */
    protected boolean editable;

    /**
     * A boolean variable that determines whether the grid is visible or not.
     */
    protected boolean gridVisible;


    /**
     * An int variable that stores the size of the grid.
     */
    protected int gridSize;

    /**
     * A boolean variable that determines whether the ports are visible.
     */
    protected boolean portsVisible;

    /**
     * The color that is used to indicate highlighted stuff.
     */
    protected Color highlightColor;

    /**
     * The color of the graphpad handle.
     */
    protected Color handleColor;

    /**
     * Color of the lockedHandle.
     */
    protected Color lockedHandleColor;

    /**
     * Color of the marquee.
     */
    protected Color marqueeColor;

    /**
     * Color of the grid.
     */
    protected Color gridColor;

    /**
     * A boolean value that determines whether drag is enabled or not.
     */
    protected boolean dragEnabled;


    /**
     * A boolean variable that determines whether drop is enabled or not.
     */
    protected boolean dropEnabled;

    /**
     * The number of times the user has clicked on edit.
     */
    protected int editClickCount;

    /**
     * NOTE: THE USE OF THIS VARIABLE IS VERY UNCLEAR.
     */
    protected boolean enabled;

    /**
     * A boolean value to determine whether the grid is enabled or not.
     */
    protected boolean gridEnabled;

    /**
     * An int variable to store the size of the handle.
     */
    protected int handleSize;

    /**
     * The tolerance level for the application.
     * NOTE: THE USE OF TOLERANCE IS NOT VERY CLEAR.
     */
    protected int tolerance;

    /**
     * The minimum number of cells that a component has to be moved when performing a move operation.
     */
    protected int minimumMove;

    /**
     * A boolean value to determine whether new cells can be selected or not.
     */
    protected boolean selectNewCells;

    /**
     * A boolean value to determine whether two components that are connected to each other get disconnected with they are moved.
     */
    protected boolean disconnectOnMove;

    /**
     * A boolean value to determine whether the components on the grid are moveable.
     */
    protected boolean moveable;

    /**
     *A boolean value to determine whether the grid is clonable.
     */
    protected boolean cloneable;

    /**
     * A boolean value to determine whether the grid is re-sizable.
     */
    protected boolean sizeable;

    /**
     * A boolean value to determine whether the grid is bendable.
     */
    protected boolean bendable;

    /**
     * A boolean value to determine whether components can be connected to each other.
     */
    protected boolean connectable;

    /**
     * A boolean value to determine whether components can be dis-connected from each other.
     */
    protected boolean disconnectable;

    /**
     * A boolean value to determine whether cell editing should be stopped or not.
     */
    protected boolean invokesStopCellEditing;

    /**
     * A boolean value to determine whether the UI of the graph is being set or not.
     */
    private boolean settingUI;

    /** The key for the graph model property */
    public static final String GRAPH_MODEL_PROPERTY = "model";

    /** The key for the GRAPH_LAYOUT_CACHE_PROPERTY */
    public static final String GRAPH_LAYOUT_CACHE_PROPERTY = "view";

    /** Key for the MARQUEE_HANDLER_PROPERTY */
    public static final String MARQUEE_HANDLER_PROPERTY = "marquee";

    /** Key for the EDITABLE_PROPERTY */
    public static final String EDITABLE_PROPERTY = "editable";

    /** Key for the SCALE_PROPERTY */
    public static final String SCALE_PROPERTY = "scale";

    /** Key for the ANTIALIASED_PROPERTY */
    public static final String ANTIALIASED_PROPERTY = "antiAliased";

    /** Key for the GRID_SIZE_PROPERTY */
    public static final String GRID_SIZE_PROPERTY = "gridSize";

    /** Key for the GRID_VISIBLE_PROPERTY */
    public static final String GRID_VISIBLE_PROPERTY = "gridVisible";

    /** Key for the PORTS_VISIBLE_PROPERTY */
    public static final String PORTS_VISIBLE_PROPERTY = "portsVisible";

    /** Key for the SELECTION_MODEL_PROPERTY */
    public static final String SELECTION_MODEL_PROPERTY = "selectionModel";

    /** Key for the INVOKES_STOP_CELL_EDITING_PROPERTY */
    public static final String INVOKES_STOP_CELL_EDITING_PROPERTY = "invokesStopCellEditing";

    /**
     * Returns a new GraphLayoutCache object, describing the UI layout for the graph specified in the parameters.
     *
     * @param jgraph The graph whose layout we are interested in.
     * @return The GraphLayoutCache object for the graph specified.
     */
    protected static GraphLayoutCache createDefaultGraphView(JGraph jgraph)
    {
        return new GraphLayoutCache(jgraph.getModel(), jgraph);
    }

    /**
     * Adds sample data for the graph. This is like the default setting.
     *
     * @param graphmodel The model of the current graph.
     */
    public static void addSampleData(GraphModel graphmodel)
    {
        ConnectionSet connectionset = new ConnectionSet();
        Hashtable hashtable = new Hashtable();
        Map map = GraphConstants.createMap();
        GraphConstants.setLineBegin(map, 2);
        GraphConstants.setBeginSize(map, 10);
        GraphConstants.setDashPattern(map, new float[] {
            3F, 3F
        });
        GraphConstants.setFontSize(map, 10F);
        Map map1 = GraphConstants.createMap();
        GraphConstants.setLineBegin(map1, 2);
        GraphConstants.setBeginFill(map1, true);
        GraphConstants.setBeginSize(map1, 10);
        GraphConstants.setFontSize(map1, 10F);
        Map map2 = GraphConstants.createMap();
        GraphConstants.setLineBegin(map2, 9);
        GraphConstants.setBeginFill(map2, true);
        GraphConstants.setBeginSize(map2, 6);
        GraphConstants.setLineEnd(map2, 4);
        GraphConstants.setEndSize(map2, 8);
        GraphConstants.setLabelPosition(map2, new Point(500, 1200));
        GraphConstants.setFontSize(map2, 10F);
        DefaultGraphCell defaultgraphcell = new DefaultGraphCell("GraphModel");
        hashtable.put(defaultgraphcell, createBounds(20, 100, Color.blue));
        defaultgraphcell.add(new DefaultPort("GraphModel/Center"));
        DefaultGraphCell defaultgraphcell1 = new DefaultGraphCell("DefaultGraphModel");
        hashtable.put(defaultgraphcell1, createBounds(20, 180, Color.blue));
        defaultgraphcell1.add(new DefaultPort("DefaultGraphModel/Center"));
        DefaultEdge defaultedge = new DefaultEdge("implements");
        connectionset.connect(defaultedge, defaultgraphcell.getChildAt(0), defaultgraphcell1.getChildAt(0));
        hashtable.put(defaultedge, map);
        DefaultGraphCell defaultgraphcell2 = new DefaultGraphCell("ModelGroup");
        defaultgraphcell2.add(defaultgraphcell);
        defaultgraphcell2.add(defaultgraphcell1);
        defaultgraphcell2.add(defaultedge);
        DefaultGraphCell defaultgraphcell3 = new DefaultGraphCell("JComponent");
        hashtable.put(defaultgraphcell3, createBounds(150, 20, Color.green));
        defaultgraphcell3.add(new DefaultPort("JComponent/Center"));
        DefaultGraphCell defaultgraphcell4 = new DefaultGraphCell("JGraph");
        hashtable.put(defaultgraphcell4, createBounds(150, 100, Color.green));
        defaultgraphcell4.add(new DefaultPort("JGraph/Center"));
        DefaultEdge defaultedge1 = new DefaultEdge("extends");
        connectionset.connect(defaultedge1, defaultgraphcell3.getChildAt(0), defaultgraphcell4.getChildAt(0));
        hashtable.put(defaultedge1, map1);
        DefaultGraphCell defaultgraphcell5 = new DefaultGraphCell("ComponentUI");
        hashtable.put(defaultgraphcell5, createBounds(280, 20, Color.red));
        defaultgraphcell5.add(new DefaultPort("ComponentUI/Center"));
        DefaultGraphCell defaultgraphcell6 = new DefaultGraphCell("GraphUI");
        hashtable.put(defaultgraphcell6, createBounds(280, 100, Color.red));
        defaultgraphcell6.add(new DefaultPort("GraphUI/Center"));
        DefaultGraphCell defaultgraphcell7 = new DefaultGraphCell("BasicGraphUI");
        hashtable.put(defaultgraphcell7, createBounds(280, 180, Color.red));
        defaultgraphcell7.add(new DefaultPort("BasicGraphUI/Center"));
        DefaultEdge defaultedge2 = new DefaultEdge("extends");
        connectionset.connect(defaultedge2, defaultgraphcell5.getChildAt(0), defaultgraphcell6.getChildAt(0));
        hashtable.put(defaultedge2, map1);
        DefaultEdge defaultedge3 = new DefaultEdge("implements");
        connectionset.connect(defaultedge3, defaultgraphcell6.getChildAt(0), defaultgraphcell7.getChildAt(0));
        hashtable.put(defaultedge3, map);
        DefaultGraphCell defaultgraphcell8 = new DefaultGraphCell("UIGroup");
        defaultgraphcell8.add(defaultgraphcell5);
        defaultgraphcell8.add(defaultgraphcell6);
        defaultgraphcell8.add(defaultgraphcell7);
        defaultgraphcell8.add(defaultedge3);
        defaultgraphcell8.add(defaultedge2);
        DefaultEdge defaultedge4 = new DefaultEdge("model");
        connectionset.connect(defaultedge4, defaultgraphcell4.getChildAt(0), defaultgraphcell.getChildAt(0));
        hashtable.put(defaultedge4, map2);
        DefaultEdge defaultedge5 = new DefaultEdge("ui");
        connectionset.connect(defaultedge5, defaultgraphcell3.getChildAt(0), defaultgraphcell5.getChildAt(0));
        hashtable.put(defaultedge5, map2);
        DefaultGraphCell defaultgraphcell9 = new DefaultGraphCell("ViewGroup");
        Object aobj[] = {
            defaultedge4, defaultedge5, defaultgraphcell2, defaultgraphcell3, defaultgraphcell4, defaultedge1, defaultgraphcell8
        };
        graphmodel.insert(aobj, hashtable, connectionset, null, null);
    }

    /**
     * This method creates bounds for the graph and returns a map that of all the graph constants and their corresponding keys.
     * It also sets the forecolor, backgroundcolor, border styles, fontstyles and opacity for the graphpad.
     *
     * @param i The x coordinate where the graph starts.
     * @param j The y coordinate where the graph starts.
     * @param color The background color for the graphpad.
     * @return The map of all the constants for this graph and their associated keys.
     */
    public static Map createBounds(int i, int j, Color color)
    {
        Map map = GraphConstants.createMap();
        GraphConstants.setBounds(map, new Rectangle(i, j, 90, 30));
        GraphConstants.setBorder(map, BorderFactory.createRaisedBevelBorder());
        GraphConstants.setBackground(map, color.darker());
        GraphConstants.setForeground(map, Color.white);
        GraphConstants.setFontStyle(map, 1);
        GraphConstants.setOpaque(map, true);
        return map;
    }

    /**
     * Creates a new Jgraph object.
     */
    public JGraph()
    {
        this(null);
    }

    /**
     * Creates a new JGraph object with the specified graphmodel.
     *
     * @param graphmodel The model to be used for the newly created JGraph.
     */
    public JGraph(GraphModel graphmodel)
    {
        this(graphmodel, (GraphLayoutCache)null);
    }

    /**
     * Creates a new jgraph with the specified graphmodel and associated graphlayoutcache.
     *
     * @param graphmodel The model to be used for the graph.
     * @param graphlayoutcache The layout to be used for the new graph.
     */
    public JGraph(GraphModel graphmodel, GraphLayoutCache graphlayoutcache)
    {
        this(graphmodel, graphlayoutcache, new BasicMarqueeHandler());
    }

    /**
     * Creates a new graph with the specified graphmodel and the associated marqueehandler.
     *
     * @param graphmodel The graph model to be used by the new graph.
     * @param basicmarqueehandler The basicmarqueehandler for the new graph.
     */
    public JGraph(GraphModel graphmodel, BasicMarqueeHandler basicmarqueehandler)
    {
        this(graphmodel, null, basicmarqueehandler);
    }

    /**
     * reates a new graph with the specified graphmodel, layout  and the associated marqueehandler.
     *
     * @param graphmodel The model to be used for the graph.
     * @param graphlayoutcache The UI layout for the graph.
     * @param basicmarqueehandler The basicmarqueehandler for the graph.
     */
    public JGraph(GraphModel graphmodel, GraphLayoutCache graphlayoutcache, BasicMarqueeHandler basicmarqueehandler)
    {
        scale = 1.0D;
        antiAliased = false;
        editable = true;
        gridVisible = false;
        gridSize = 10;
        portsVisible = false;
        highlightColor = Color.green;
        dragEnabled = false;
        dropEnabled = true;
        editClickCount = 2;
        enabled = true;
        gridEnabled = false;
        handleSize = 3;
        tolerance = 4;
        minimumMove = 5;
        selectNewCells = false;
        disconnectOnMove = false;
        moveable = true;
        cloneable = true;
        sizeable = true;
        bendable = true;
        connectable = true;
        disconnectable = true;
        selectionModel = new DefaultGraphSelectionModel(this);
        setLayout(null);
        marquee = basicmarqueehandler;
        if(graphlayoutcache == null)
        {
            graphlayoutcache = createDefaultGraphView(this);
        }
        setGraphLayoutCache(graphlayoutcache);
        updateUI();
        if(graphmodel == null)
        {
            graphmodel = new DefaultGraphModel();
            setModel(graphmodel);
            addSampleData(graphmodel);
        } else
        {
            setModel(graphmodel);
        }
        setDoubleBuffered(true);
    }

    /**
     * Returns the UI of the graph.
     *
     * @return A GraphUI object describing the UI for this graph.
     */
    public GraphUI getUI()
    {
        return (GraphUI)ui;
    }

    /**
     * This method is used to modify the UI for the graph. It sets the settingUI variable to true when it tries to
     * change the UI and resets it to false once the change is done.
     *
     * @param graphui The GraphUI that needs to be used in the graph instead of the currently existing one.
     * @see #settingUI
     */
    public void setUI(GraphUI graphui)
    {
        if((GraphUI)ui != graphui)
        {
            settingUI = true;
            try
            {
                super.setUI(graphui);
            }
            finally
            {
                settingUI = false;
            }
        }
    }

    /**
     * Updates the UI of the graph to the one specified by BasicGraphUI.
     *
     * @see BasicGraphUI
     */
    public void updateUI()
    {
        setUI(new BasicGraphUI());
        invalidate();
    }

    /**
     * Returns the Class ID for the UI class.
     *
     * @return A string representing the class ID. For example, "GraphUI".
     */
    public String getUIClassID()
    {
        return "GraphUI";
    }

    /**
     * Returns all the roots (the cells) in the graphModel of this class.
     *
     * @return An array of object where each one represents a root for this graph's model.
     */
    public Object[] getRoots()
    {
        return DefaultGraphModel.getRoots(graphModel);
    }

    /**
     * Returns the roots (the cells) in the rectangle region specified.
     *
     * @param rectangle The rectangle region within which we want the roots (cells).
     * @return An array of roots (cells).
     */
    public Object[] getRoots(Rectangle rectangle)
    {
        CellView acellview[] = graphLayoutCache.getRoots(rectangle);
        Object aobj[] = new Object[acellview.length];
        for(int i = 0; i < acellview.length; i++)
        {
            aobj[i] = acellview[i].getCell();
        }

        return aobj;
    }

    /**
     * Returns an array of objects containing all the cells, and each cell's roots and descendants.
     *
     * @param aobj The cells whose descendants we want.
     * @return A array containing all cells and their roots and descendants.
     */
    public Object[] getDescendants(Object aobj[])
    {
        Set set = DefaultGraphModel.getDescendants(getModel(), aobj);
        return set.toArray();
    }

    /**
     * Returns a clone of the cells specified in the parameter list.
     *
     * @param aobj The list of cells that we want to be cloned.
     * @return A hashtable where the key is the original cell and the data is the cloned cell.
     */
    public Map cloneCells(Object aobj[])
    {
        Hashtable hashtable = new Hashtable();
        ArrayList arraylist = new ArrayList();
        for(int i = 0; i < aobj.length; i++)
        {
            arraylist.add(aobj[i]);
        }

        while(!arraylist.isEmpty())
        {
            Object obj = arraylist.remove(0);
            if(obj instanceof DefaultGraphCell)
            {
                for(int j = 0; j < graphModel.getChildCount(obj); j++)
                {
                    arraylist.add(graphModel.getChild(obj, j));
                }

                DefaultGraphCell defaultgraphcell = (DefaultGraphCell)obj;
                DefaultGraphCell defaultgraphcell1 = (DefaultGraphCell)defaultgraphcell.clone();
                Object obj2 = graphModel.getParent(defaultgraphcell);
                if(obj2 != null)
                {
                    DefaultMutableTreeNode defaultmutabletreenode = (DefaultMutableTreeNode)hashtable.get(obj2);
                    if(defaultmutabletreenode != null)
                    {
                        defaultmutabletreenode.add(defaultgraphcell1);
                    }
                }
                hashtable.put(defaultgraphcell, defaultgraphcell1);
            }
        }
        for(Iterator iterator = hashtable.values().iterator(); iterator.hasNext();)
        {
            Object obj1 = iterator.next();
            if(obj1 instanceof Port)
            {
                Port port = ((Port)obj1).getAnchor();
                if(port != null)
                {
                    ((Port)obj1).setAnchor((Port)hashtable.get(port));
                }
            }
        }

        return hashtable;
    }

    /**
     * Returns the first cell at the location specified.
     *
     * @param i The x co-ordinate of the cell.
     * @param j The y co-ordnate of the cell.
     * @return The first cell in the location (i, j)
     */
    public Object getFirstCellForLocation(int i, int j)
    {
        return getNextCellForLocation(null, i, j);
    }

    /**
     * Gets the next cell at the location specified. null if such a cell does not exist.
     *
     * @param obj The array of cells after which another cell has to be found.
     * @param i The x co-ordinate where a cell is desired.
     * @param j The y co-ordinate where a cell is desired.
     * @return The next cell found. <code>null</code> if there is no such cell.
     */
    public Object getNextCellForLocation(Object obj, int i, int j)
    {
        i = (int)((double)i / scale);
        j = (int)((double)j / scale);
        CellView cellview = graphLayoutCache.getMapping(obj, false);
        CellView cellview1 = getNextViewAt(cellview, i, j);
        if(cellview1 != null)
        {
            return cellview1.getCell();
        } else
        {
            return null;
        }
    }

    /**
     * Gets the rectangle binding the cell specified. <code>null</code> if there is no such binding rectangle.
     *
     * @param obj The cell whose binding rectangle we want to find.
     * @return A Rectangle describing the bounding box around the cell.
     */
    public Rectangle getCellBounds(Object obj)
    {
        CellView cellview = graphLayoutCache.getMapping(obj, false);
        if(cellview != null)
        {
            return cellview.getBounds();
        } else
        {
            return null;
        }
    }

    /**
     * Returns the rectangle binding all the cells specified. <code>null</code> if there is no such rectangle.
     *
     * @param aobj An array of cells around all of which we want to find a binding rectangle.
     * @return The binding rectangle. <code>null</code> if none like this exist.
     */
    public Rectangle getCellBounds(Object aobj[])
    {
        if(aobj != null && aobj.length > 0)
        {
            Rectangle rectangle = new Rectangle(getCellBounds(aobj[0]));
            for(int i = 1; i < aobj.length; i++)
            {
                Rectangle rectangle1 = getCellBounds(aobj[i]);
                SwingUtilities.computeUnion(rectangle1.x, rectangle1.y, rectangle1.width, rectangle1.height, rectangle);
            }

            return rectangle;
        } else
        {
            return null;
        }
    }

    /**
     * Returns a CellView object describing the next view at position (i,j)
     *
     * @param cellview The CellView after which we want to find another one.
     * @param i The x co-ordinate of the point.
     * @param j The y co-ordinate of the point.
     * @return The next cellView.
     */
    public CellView getNextViewAt(CellView cellview, int i, int j)
    {
        Object aobj[] = graphLayoutCache.order(getSelectionModel().getSelectables());
        CellView acellview[] = graphLayoutCache.getMapping(aobj);
        CellView cellview1 = getNextViewAt(acellview, cellview, i, j);
        return cellview1;
    }

    /**
     * Returns a cellview enclosing all the cells specified in the array acellview, but that are next to the cell
     * specified by cellview.
     * i and j are the x co-ordinates of the desired bounding rectangle.
     *
     * @param acellview An array of cellview objects at which the new view is desired.
     * @param cellview The next cellview.
     * @param i The x co-ordinate of the desired bounding rectangle.
     * @param j The y co-ordinate of the desired bounding rectangle.
     * @return The CellView object for all the cells.
     */
    public CellView getNextViewAt(CellView acellview[], CellView cellview, int i, int j)
    {
        if(acellview != null)
        {
            Rectangle rectangle = new Rectangle(i - tolerance, j - tolerance, 2 * tolerance, 2 * tolerance);
            CellView cellview1 = null;
            boolean flag = cellview == null;
            java.awt.Graphics g = getGraphics();
            for(int k = acellview.length - 1; k >= 0; k--)
            {
                if(acellview[k] != null && acellview[k].intersects(g, rectangle))
                {
                    if(flag && !selectionModel.isChildrenSelected(acellview[k].getCell()))
                    {
                        return acellview[k];
                    }
                    if(cellview1 == null)
                    {
                        cellview1 = acellview[k];
                    }
                    flag |= acellview[k] == cellview;
                }
            }

            return cellview1;
        } else
        {
            return null;
        }
    }

    /**
     * Gets the port line on which the point (i,j) lies.
     *
     * @param i x co-ordinate a point
     * @param j y co-ordiante of a point
     * @return The port on which the specified point resides.
     */
    public Port getPortForLocation(int i, int j)
    {
        PortView portview = getPortViewAt(i, j);
        if(portview != null)
        {
            return (Port)portview.getCell();
        } else
        {
            return null;
        }
    }

    /**
     * Gets the portview in which the point (i,j) lies.
     *
     * @param i x co-ordinate a point
     * @param j y co-ordiante of a point
     * @return The portview in which the specified point resides.
     */
    public PortView getPortViewAt(int i, int j)
    {
        Rectangle rectangle = new Rectangle(i - tolerance, j - tolerance, 2 * tolerance, 2 * tolerance);
        PortView aportview[] = graphLayoutCache.getPorts();
        for(int k = aportview.length - 1; k >= 0; k--)
        {
            if(aportview[k] != null && aportview[k].intersects(getGraphics(), rectangle))
            {
                return aportview[k];
            }
        }

        return null;
    }

    /**
     * Returns the string representation of the object specified. If obj is null, it returns null.
     *
     * @param obj This a cellview object, or if it is a cell, it is mapped to a cellview by the method.
     * @return The string representation of obj.
     */
    public String convertValueToString(Object obj)
    {
        CellView cellview = (obj instanceof CellView) ? (CellView)obj : getGraphLayoutCache().getMapping(obj, false);
        if(cellview != null)
        {
            Object obj1 = GraphConstants.getValue(cellview.getAllAttributes());
            if(obj1 != null)
            {
                obj = obj1;
            } else
            {
                obj = cellview.getCell();
            }
        }
        if((obj instanceof DefaultMutableTreeNode) && ((DefaultMutableTreeNode)obj).getUserObject() != null)
        {
            return ((DefaultMutableTreeNode)obj).getUserObject().toString();
        }
        if(obj != null)
        {
            return obj.toString();
        } else
        {
            return null;
        }
    }

    /**
     * If the grid is enabled, the point is converted to the closest position on the grid where there is a '.'
     * If grid is not enabled, the specified point is just returned.
     *
     * @param point A point representing a point in the graph.
     * @return The point in the graph that is closest to the specified point, if it is a grid.
     */
    public Point snap(Point point)
    {
        if(gridEnabled && point != null)
        {
            point.x += gridSize / 2;
            point.y += gridSize / 2;
            point.x = Math.round(Math.round(point.x / gridSize) * gridSize);
            point.y = Math.round(Math.round(point.y / gridSize) * gridSize);
        }
        return point;
    }

    /**
     * If the grid is enabled, the dimension is converted to the closest dimension on the grid where there is a '.'
     * If grid is not enabled, the specified dimension is just returned.
     *
     * @param dimension A dimension representing a dimension in the graph.
     * @return The dimension in the dimension that is closest to the specified dimension, if it is a grid.
     */
    public Dimension snap(Dimension dimension)
    {
        if(gridEnabled && dimension != null)
        {
            dimension.width += gridSize / 2;
            dimension.height += gridSize / 2;
            dimension.width = Math.round(Math.round(dimension.width / gridSize) * gridSize);
            dimension.height = Math.round(Math.round(dimension.height / gridSize) * gridSize);
        }
        return dimension;
    }

    /**
     * Converts the specified point to the point on the screen based on the scale. The x and y co-ordinates of the point
     * are just multiplied by the scale.
     *
     * @param point The input point.
     * @return The position of the point on the screen, with the zoon.
     */
    public Point toScreen(Point point)
    {
        if(point == null)
        {
            return null;
        } else
        {
            point.x = (int)Math.round((double)point.x * scale);
            point.y = (int)Math.round((double)point.y * scale);
            return point;
        }
    }

    /**
     * Converts the specified point from the point on the screen based on the scale. The x and y co-ordinates of the point
     * are just divided by the scale.
     *
     * @param point The input point.
     * @return The position of the point on the graph, without the zoon.
     */
    public Point fromScreen(Point point)
    {
        if(point == null)
        {
            return null;
        } else
        {
            point.x = (int)Math.round((double)point.x / scale);
            point.y = (int)Math.round((double)point.y / scale);
            return point;
        }
    }

    /**
      * Converts the specified Rectangle to the Rectangle on the screen based on the scale. The x and y co-ordinates and width and height
      * of the Rectangle are just multiplied by the scale.
      *
      * @param rectangle The input Rectangle.
      * @return The position of the Rectangle on the screen, with the zoom.
      */
    public Rectangle toScreen(Rectangle rectangle)
    {
        if(rectangle == null)
        {
            return null;
        } else
        {
            rectangle.x *= scale;
            rectangle.y *= scale;
            rectangle.width *= scale;
            rectangle.height *= scale;
            return rectangle;
        }
    }

    /**
     * Converts the specified Rectangle to the Rectangle from the screen based on the scale. The x and y co-ordinates and width and height
     * of the Rectangle are just divided by the scale.
     *
     * @param rectangle The input Rectangle.
     * @return The position of the Rectangle on the graph, without the zoon.
     */
    public Rectangle fromScreen(Rectangle rectangle)
    {
        if(rectangle == null)
        {
            return null;
        } else
        {
            rectangle.x /= scale;
            rectangle.y /= scale;
            rectangle.width /= scale;
            rectangle.height /= scale;
            return rectangle;
        }
    }

    /**
     * Creates a cellView based on the cell and the mapper of the cell to a point.
     *
     * @param obj The cell whose cellview we desire.
     * @param cellmapper The mapper of the cell to a cellView.
     * @return The cellView for the cell.
     */
    public CellView createView(Object obj, CellMapper cellmapper)
    {
        Object obj1 = null;
        if(obj instanceof Port)
        {
            obj1 = createPortView((Port)obj, cellmapper);
        } else
        if(obj instanceof Edge)
        {
            obj1 = createEdgeView((Edge)obj, cellmapper);
        } else
        {
            obj1 = createVertexView(obj, cellmapper);
        }
        cellmapper.putMapping(obj, ((CellView) (obj1)));
        ((CellView) (obj1)).refresh(true);
        ((CellView) (obj1)).update();
        return ((CellView) (obj1));
    }

    /**
     * Updates the dimensions of a cell.
     *
     * @param cellview The cellview of the cell whose dimensions have changed.
     */
    public void updateAutoSize(CellView cellview)
    {
        if(cellview != null && GraphConstants.isAutoSize(cellview.getAllAttributes()))
        {
            Rectangle rectangle = cellview.getBounds();
            if(rectangle != null)
            {
                Dimension dimension = getUI().getPreferredSize(this, cellview);
                rectangle.setSize(dimension);
            }
        }
    }

    /**
     * Creates a new VertexView object of the cell specified. Returns a new VertexView object.
     *
     * @param obj A cell in the screen.
     * @param cellmapper The mapper of the cell.
     * @return The vertexView for the cell.
     */
    protected VertexView createVertexView(Object obj, CellMapper cellmapper)
    {
        return new VertexView(obj, this, cellmapper);
    }

    /**
     * Creates an EdgeView for the cell. Returns a new EdgeView object.
     *
     * @param edge The edge for the cell.
     * @param cellmapper The cellmapper for the cell.
     * @return An edgeView object for the cell.
     */
    protected EdgeView createEdgeView(Edge edge, CellMapper cellmapper)
    {
        return new EdgeView(edge, this, cellmapper);
    }

    /**
     * Creates a PortView object of the cell.
     *
     * @param port The current port of the cell.
     * @param cellmapper The mapper, that maps the cell to a cellView.
     * @return A new Portview object.
     */
    protected PortView createPortView(Port port, CellMapper cellmapper)
    {
        return new PortView(port, this, cellmapper);
    }

    /**
     * Returns the number of times the edit button has been clicked.
     *
     * @return An int representing the number of edit button clicks.
     */
    public int getEditClickCount()
    {
        return editClickCount;
    }

    /**
     * Sets the number of edit button clicks.
     *
     * @param i The number to edit button clicks to be stored by the application.
     */
    public void setEditClickCount(int i)
    {
        editClickCount = i;
    }

    /**
     * Determine whether a component can be dropped on the graphpad.
     *
     * @return <code>true</code> if a component can be dropped on the graphpad <code>false</code> otherwise.
     */
    public boolean isDropEnabled()
    {
        return dropEnabled;
    }

    /**
     * Set whether a component can be dropped on the graphpad.
     *
     * @param flag <code>true</code> if a component can be dropped on the graphpad <code>false</code> otherwise.
     */
    public void setDropEnabled(boolean flag)
    {
        dropEnabled = flag;
    }

    /**
     * Determine whether a component can be dragged in the application.
     *
     * @return <code>true</code> if a component can be dragged <code>false</code> otherwise.
     */
    public boolean isDragEnabled()
    {
        return dragEnabled;
    }

    /**
     * Set whether a component can be dragged in the application.
     *
     * @param flag <code>true</code> if a component can be dragged <code>false</code> otherwise.
     */
    public void setDragEnabled(boolean flag)
    {
        dragEnabled = flag;
    }

    /**
     * Determine whether components can be moved or its position is frozen.
     *
     * @return <code>true</code> True if components can move <code>false</otherwise>
     */
    public boolean isMoveable()
    {
        return moveable;
    }

    /**
     * Sets whether components can move or they are fixed.
     *
     * @param flag <code>true</code> True if components can move <code>false</otherwise>
     */
    public void setMoveable(boolean flag)
    {
        moveable = flag;
    }

    /**
     * Determines whether components are bendable.
     *
     * @return <code>true</code> True if components are bendable <code>false</otherwise>
     */
    public boolean isBendable()
    {
        return bendable;
    }

    /**
     * Sets whether components are bendable.
     *
     * @param flag <code>true</code> True if components can bend <code>false</otherwise>
     */
    public void setBendable(boolean flag)
    {
        bendable = flag;
    }

    /**
     * Determines whether components can be connected to each other. This is associated with the "group objects" feature.
     *
     * @return <code>true</code> True if components are connectable <code>false</otherwise>
     */
    public boolean isConnectable()
    {
        return connectable;
    }

    /**
     * Sets whether components are bendable. This is associated with the "group objects" feature.
     *
     * @param flag <code>true</code> True if components are connectable <code>false</otherwise>
     */
    public void setConnectable(boolean flag)
    {
        connectable = flag;
    }

    /**
     * Determines whether components can be dis-connected from each other. This is associated with the "group objects" feature.
     *
     * @return <code>true</code> True if components can be dis-connected <code>false</otherwise>
     */
    public boolean isDisconnectable()
    {
        return disconnectable;
    }

    /**
     * Sets whether components are dis-connectable. This is associated with the "group objects" feature.
     *
     * @param flag <code>true</code> True if components are dis-connectable, <code>false</otherwise>
     */
    public void setDisconnectable(boolean flag)
    {
        disconnectable = flag;
    }

    /**
     * Determines whether the application can be cloned.
     *
     * @return <code>true</code> True if the application can be cloned <code>false</otherwise>
     */
    public boolean isCloneable()
    {
        return cloneable;
    }

    /**
     * Sets whether application can be cloned.
     *
     * @param flag <code>true</code> True if components are dis-connectable, <code>false</otherwise>
     */
    public void setCloneable(boolean flag)
    {
        cloneable = flag;
    }

    /**
     * Determines whether the application can be re-sized.
     *
     * @return <code>true</code> if the application can be resized, false otherwise.
     */
    public boolean isSizeable()
    {
        return sizeable;
    }

    /**
     * Sets whether an application can be re-sized.
     *
     * @param flag true if it can be resized, false otherwise.
     */
    public void setSizeable(boolean flag)
    {
        sizeable = flag;
    }

    /**
     * Determines whether components are dis-connected when they move. This is associated with the "group objects" feature.
     *
     * @return true if the components are dis-connected, false otherwise.
     */
    public boolean isDisconnectOnMove()
    {
        return disconnectOnMove && disconnectable;
    }

    /**
     * Sets whether new cells that have been selected by the user.
     *
     * @param flag true if new cells have been set, false otherwise.
     */
    public void setSelectNewCells(boolean flag)
    {
        selectNewCells = flag;
    }


    /**
     * Determines  whether new cells have been selected by the user.
     *
     * @return true if new cells have been selected, false otherwise.
     */
    public boolean isSelectNewCells()
    {
        return selectNewCells;
    }

    /**
     * Sets a flag so that components are dis-connected when they are moved.
     *
     * @param flag true if components are to be dis-connected, false otherwise.
     */
    public void setDisconnectOnMove(boolean flag)
    {
        disconnectOnMove = flag;
    }

    /**
     * Determines whether the user can see the grid on the drawing pad. Asoociated with the View->Grid option.
     *
     * @return true, If the grid lines are visible, false otherwise.
     */
    public boolean isGridEnabled()
    {
        return gridEnabled;
    }

    /**
     * Sets whether the grid is visible on the drawing pad. Asoociated with the View->Grid option.
     *
     * @param flag true, If the grid lines are to be visible, false otherwise.
     */
    public void setGridEnabled(boolean flag)
    {
        gridEnabled = flag;
    }

    /**
     * Sets the tolerance level for the application.
     * NOTE : It is not very clear in what context this tolerance is used.
     *
     * @return The tolerance level desired.
     */
    public int getTolerance()
    {
        return tolerance;
    }

    /**
     * Sets the tolerance level for the application.
     * NOTE : It is not very clear in what context this tolerance is used.
     *
     * @param i The tolerance level for this application.
     */
    public void setTolerance(int i)
    {
        tolerance = i;
    }

    /**
     * The size of the handle (also known as the cursor) for this application.
     *
     * @return An int representing the size of the handle.
     */
    public int getHandleSize()
    {
        return handleSize;
    }

    /**
     * Sets the handle size (cursor size) for this application.
     *
     * @param i The size of the cursor.
     */
    public void setHandleSize(int i)
    {
        handleSize = i;
    }

    /**
     * Returns the minimum number of cells that a component has to be moved for the move operation to be successful.
     *
     * @return An int representing the minimum number of cells a component has to move.
     */
    public int getMinimumMove()
    {
        return minimumMove;
    }

    /**
     * Sets the minimum number of cells that a component has to be moved for the move operation to be successful.
     *
     * @param i An int representing the minimum number of cells a component has to move.
     */
    public void setMinimumMove(int i)
    {
        minimumMove = i;
    }


    /**
     * The background color on the slide. Format -> Background -> Color.
     *
     * @return The background color used by the application.
     */
    public Color getGridColor()
    {
        return gridColor;
    }

    /**
     * Sets the background color for the current slide. Format -> Background -> Color.
     *
     * @param color The background color for the slides.
     */
    public void setGridColor(Color color)
    {
        gridColor = color;
    }

    /**
     * Determoine the color of the handle.
     *
     * @return The color of the handle.
     */
    public Color getHandleColor()
    {
        return handleColor;
    }

    /**
     * Sets the handle color.
     *
     * @param color The color of the handle.
     */
    public void setHandleColor(Color color)
    {
        handleColor = color;
    }

    /**
     * Determines the color of the handle when it is locked at a position.
     *
     * @return The color of the handle when it is locked.
     */
    public Color getLockedHandleColor()
    {
        return lockedHandleColor;
    }

    /**
     * The color to be used for the handle.
     *
     * @param color The handle color.
     */
    public void setLockedHandleColor(Color color)
    {
        lockedHandleColor = color;
    }

    /**
     * Returns the marquee color for this application.
     * NOTE: The use of "marquee" is not very clear.
     *
     * @return The color of the marquee.
     */
    public Color getMarqueeColor()
    {
        return marqueeColor;
    }

    /**
     * Sets the marquee color for this application.
     * NOTE: The use of "marquee" is not very clear.
     *
     * @param color The color of the marquee.
     */
    public void setMarqueeColor(Color color)
    {
        marqueeColor = color;
    }

    /**
     * The color to be used when highlighting text in the slide.
     *
     * @return The color used for highlighting.
     */
    public Color getHighlightColor()
    {
        return highlightColor;
    }

    /**
     * Sets the color to be used for highlighting.
     *
     * @param color The color used for highlighting.
     */
    public void setHighlightColor(Color color)
    {
        highlightColor = color;
    }

    /**
     * Returns the scale (the maximum width and height, which are equal) used by the application in meters.
     *
     * @return The maximum width and height in meters.
     */
    public double getScale()
    {
        return scale;
    }

    /**
     * Sets the scale (the maximum width and height, which are equal) used by the application in meters.
     *
     * @param d The maximum width and height in meters.
     */
    public void setScale(double d)
    {
        if(d > 0.0D)
        {
            double d1 = scale;
            scale = d;
            firePropertyChange("scale", d1, d);
        }
    }

    /**
     * Returns the grid size, i.e. the number of grid points that are visible horizontally and vertically.
     *
     * @return The number of grid points.
     */
    public int getGridSize()
    {
        return gridSize;
    }

    /**
     * Sets the grid size, i.e. the number of grid points that are visible horizontally and vertically.
     *
     * @param i The number of grid points.
     */
    public void setGridSize(int i)
    {
        int j = gridSize;
        gridSize = i;
        firePropertyChange("gridSize", j, i);
    }

    /**
     * Determines whether the grid format is seen on the slide.
     *
     * @return True if the grid is visible, false otherwise.
     */
    public boolean isGridVisible()
    {
        return gridVisible;
    }

    /**
     * Sets the grid to be visible or invisble on the slide.
     *
     * @param flag True if the grid is to be visible, false otherwise.
     */
    public void setGridVisible(boolean flag)
    {
        boolean flag1 = gridVisible;
        gridVisible = flag;
        firePropertyChange("gridVisible", flag1, flag);
    }

    /**
     * Determines whether ports are visible for each component. Ports are eight squares visible for each component, four
     * on each of the four corners of the square that encloses the component and one port at the center of each of
     * the four edges. It makes navigation easy. View -> Ports
     *
     * @return True if ports are visible, false otherwise.
     */
    public boolean isPortsVisible()
    {
        return portsVisible;
    }


    /**
     * Sets whether ports are visible or not for each component. Ports are eight squares visible for each component, four
     * on each of the four corners of the square that encloses the component and one port at the center of each of
     * the four edges. It makes navigation easy. View -> Ports
     *
     * @param flag True if ports are visible, false otherwise.
     */
    public void setPortsVisible(boolean flag)
    {
        boolean flag1 = portsVisible;
        portsVisible = flag;
        firePropertyChange("portsVisible", flag1, flag);
    }

    /**
     * Determines whether the components are anti-aliased or not. View -> AntiAlias.
     * NOTE : The purpose of this feature is not clear. No change is visible in the application when a change is made
     * to allow for anti-aliasing or disallow it.
     *
     * @return True if anti-aliasing is allowed, false if it is not.
     */
    public boolean isAntiAliased()
    {
        return antiAliased;
    }

    /**
     * Sets whether the components are anti-aliased or not. View -> AntiAlias.
     *
     * NOTE : The purpose of this feature is not clear. No change is visible in the application when a change is made
     * to allow for anti-aliasing or disallow it.
     *
     * @param flag True is anti-aliasing is to be enabled, false otherwise.
     */
    public void setAntiAliased(boolean flag)
    {
        boolean flag1 = antiAliased;
        antiAliased = flag;
        firePropertyChange("antiAliased", flag1, flag);
    }

    /**
     * Determines whether components are editable. To edit a component, Edit -> Edit.
     *
     * @return True is components are editable, false otherwise.
     */
    public boolean isEditable()
    {
        return editable;
    }

    /**
     * Sets whether components are editable. To edit a component, Edit -> Edit.
     *
     * @param flag True is components are editable, false otherwise.
     */
    public void setEditable(boolean flag)
    {
        boolean flag1 = editable;
        editable = flag;
        firePropertyChange("editable", flag1, flag);
    }

    /**
     * Returns the GraphModel associated with this application.
     *
     * @return The application's graphModel.
     */
    public GraphModel getModel()
    {
        return graphModel;
    }

    /**
     * Sets the graphModel for the application. It also fires the property change event to indicate a change in the
     * graph by calling the firePropertyChange () method. Also calls the invalidate () method.
     *
     * @param graphmodel The new graphModel for the application.
     */
    public void setModel(GraphModel graphmodel)
    {
        GraphModel graphmodel1 = graphModel;
        graphModel = graphmodel;
        firePropertyChange("model", graphmodel1, graphModel);
        if(graphLayoutCache != null && graphLayoutCache.getModel() != graphModel)
        {
            graphLayoutCache.setModel(graphModel);
        }
        invalidate();
    }

    /**
     * Returns the GraphLayoutCache associated with this application.
     *
     * @return The application's graphLayoutCache.
     */
    public GraphLayoutCache getGraphLayoutCache()
    {
        return graphLayoutCache;
    }

    /**
     * Sets the graphLayoutCache for this application. It then fires the property change event by calling the
     * firePropertyChange () method. Also calls the invalidate () method.
     *
     * @param graphlayoutcache The graphLayoutCache for this application.
     */
    public void setGraphLayoutCache(GraphLayoutCache graphlayoutcache)
    {
        GraphLayoutCache graphlayoutcache1 = graphLayoutCache;
        graphLayoutCache = graphlayoutcache;
        firePropertyChange("view", graphlayoutcache1, graphLayoutCache);
        if(graphLayoutCache != null && graphLayoutCache.getModel() != getModel())
        {
            graphLayoutCache.setModel(getModel());
        }
        invalidate();
    }

    /**
     * Returns the BasicMarqueeHandler associated with this application.
     *
     * @return The application's BasicMarqueeHandler.
     */
    public BasicMarqueeHandler getMarqueeHandler()
    {
        return marquee;
    }

    /**
     * Sets the BasicMarqueeHandler for this application. It then fires the property change event by calling the
     * firePropertyChange () method. Also calls the invalidate () method.
     *
     * @param basicmarqueehandler The BasicMarqueeHandler for this application.
     */
    public void setMarqueeHandler(BasicMarqueeHandler basicmarqueehandler)
    {
        BasicMarqueeHandler basicmarqueehandler1 = marquee;
        marquee = basicmarqueehandler;
        firePropertyChange("marquee", basicmarqueehandler1, basicmarqueehandler);
        invalidate();
    }

    /**
     * Sets whether the StopCellEditing method is invoked through the specified parameter. The property change event is
     * fired by calling the firePropertyChange method .
     *
     * @param flag True or false depending on whether the StopCellEditing should be involked or not.
     * @see #StopCellEditing
     * @see #invokesStopCellEditing
     */
    public void setInvokesStopCellEditing(boolean flag)
    {
        boolean flag1 = invokesStopCellEditing;
        invokesStopCellEditing = flag;
        firePropertyChange("invokesStopCellEditing", flag1, flag);
    }

    /**
     * Gets the boolean value indicating whether the StopCellEditing method should be involked or not.
     *
     * @return Returns the value of <code>invokesStopCellEditing</code>
     * @see invokesStopCellEditing.
     */
    public boolean getInvokesStopCellEditing()
    {
        return invokesStopCellEditing;
    }

    /**
     * Determines whether the specified cell is editable.
     *
     * @param obj The specifed cell.
     * @return True if the cell is editable, false otherwise.
     */
    public boolean isCellEditable(Object obj)
    {
        CellView cellview = graphLayoutCache.getMapping(obj, false);
        return isEditable() && GraphConstants.isEditable(cellview.getAllAttributes());
    }

    /**
     * Returns the ToolTipText that appears whenever the mouse is moved over a component. The method determines the
     * component over which the mouse moved and returns the appropriate message.
     *
     * @param mouseevent The mouseevent that caused this method to be called.
     * @return The tooptip for the component above which the mouse is. Returns null if the mouseevent is null.
     */
    public String getToolTipText(MouseEvent mouseevent)
    {
        if(mouseevent != null)
        {
            Object obj = getFirstCellForLocation(mouseevent.getX(), mouseevent.getY());
            String s = convertValueToString(obj);
            return s == null || s.length() <= 0 ? null : s;
        } else
        {
            return null;
        }
    }

    /**
     * Sets the graphselectionmodel for the application.
     * If the specified graphselectionmodel is null, the application's  graphselectionmodel is set to the emptySelectionModel.
     *
     * Also modifies the GraphSelectionListener based on the GraphSelectionModel.
     *
     * The property change event is also fired by calling the firePropertyChange () method.
     *
     * @param graphselectionmodel The new graphselectionModel.
     */
    public void setSelectionModel(GraphSelectionModel graphselectionmodel)
    {
        if(graphselectionmodel == null)
        {
            graphselectionmodel = EmptySelectionModel.sharedInstance();
        }
        GraphSelectionModel graphselectionmodel1 = selectionModel;
        if(selectionModel != null && selectionRedirector != null)
        {
            selectionModel.removeGraphSelectionListener(selectionRedirector);
        }
        selectionModel = graphselectionmodel;
        if(selectionRedirector != null)
        {
            selectionModel.addGraphSelectionListener(selectionRedirector);
        }
        firePropertyChange("selectionModel", graphselectionmodel1, selectionModel);
    }

    /**
     * Gets the SelectionModel for the graph.
     *
     * @return The selectionModel for this graph.
     */
    public GraphSelectionModel getSelectionModel()
    {
        return selectionModel;
    }

    /**
     * Empties the current selection by calling GraphSelectionModel.clearSelection ()
     */
    public void clearSelection()
    {
        getSelectionModel().clearSelection();
    }

    /**
     * Determines whether there are any selections for the graph.
     *
     * @return True is the selection is currently empty.
     */
    public boolean isSelectionEmpty()
    {
        return getSelectionModel().isSelectionEmpty();
    }

    /**
     * Adds a graphselectionlistener for this graph. Whenever a selection on the graph changes, all of the selectionlisteners
     * are called.
     *
     * @param graphselectionlistener The listener to be added to the existing list.
     */
    public void addGraphSelectionListener(GraphSelectionListener graphselectionlistener)
    {
        listenerList.add(GraphSelectionListener.class, graphselectionlistener);
        if(listenerList.getListenerCount(GraphSelectionListener.class) != 0 && selectionRedirector == null)
        {
            selectionRedirector = new GraphSelectionRedirector();
            selectionModel.addGraphSelectionListener(selectionRedirector);
        }
    }

    /**
     * Removes a graphselectionlistener for this graph.
     *
     * @param graphselectionlistener The listener to be removed from the existing list.
     */
    public void removeGraphSelectionListener(GraphSelectionListener graphselectionlistener)
    {
        listenerList.remove(GraphSelectionListener.class, graphselectionlistener);
        if(listenerList.getListenerCount(GraphSelectionListener.class) == 0 && selectionRedirector != null)
        {
            selectionModel.removeGraphSelectionListener(selectionRedirector);
            selectionRedirector = null;
        }
    }

    /**
     * This method is called whenever a selection in the graph changes. It calls the valueChanged() method for each
     * of the listeners that exist for this graph.
     *
     * @param graphselectionevent The event raised when a selection in the graph changed.
     */
    protected void fireValueChanged(GraphSelectionEvent graphselectionevent)
    {
        Object aobj[] = listenerList.getListenerList();
        for(int i = aobj.length - 2; i >= 0; i -= 2)
        {
            if(aobj[i] == (GraphSelectionListener.class))
            {
                ((GraphSelectionListener)aobj[i + 1]).valueChanged(graphselectionevent);
            }
        }

    }

    /**
     * Sets in the SelectionModel the cell that has been selected.
     *
     * @param obj The selected cell.
     */
    public void setSelectionCell(Object obj)
    {
        getSelectionModel().setSelectionCell(obj);
    }

    /**
     * Sets in the SelectionModel all the cells that have been selected.
     *
     * @param aobj An array of cells that have been selected.
     */
    public void setSelectionCells(Object aobj[])
    {
        getSelectionModel().setSelectionCells(aobj);
    }

    /**
     * Adds to the selectionModel a cell that has been selected.
     *
     * @param obj The cell that has been selected.
     */
    public void addSelectionCell(Object obj)
    {
        getSelectionModel().addSelectionCell(obj);
    }

    /**
     * Adds in the SelectionModel all the cells that have been selected.
     *
     * @param aobj An array of cells that have been selected.
     */
    public void addSelectionCells(Object aobj[])
    {
        getSelectionModel().addSelectionCells(aobj);
    }

    /**
     * Removes from the selectionModel a cell that has been selected.
     *
     * @param obj The cell that has been selected.
     */
    public void removeSelectionCell(Object obj)
    {
        getSelectionModel().removeSelectionCell(obj);
    }

    /**
     * Returns from the selectionModel the cell that has been selected.
     *
     * @return The selected cell.
     */
    public Object getSelectionCell()
    {
        return getSelectionModel().getSelectionCell();
    }

    /**
     * Returns from the selectionModel an array of cells that are selected.
     *
     * @return The selected cells.
     */
    public Object[] getSelectionCells()
    {
        return getSelectionModel().getSelectionCells();
    }

    /**
     * Returns the number of cells that have been selected on the slide.
     *
     * @return An int representing the number of cells that have been selected.
     */
    public int getSelectionCount()
    {
        return selectionModel.getSelectionCount();
    }

    /**
     * Determines whether the specified cell is selected.
     *
     * @param obj The cell about which we want to get the selection information.
     * @return True is cell <code>obj</code> is selected, false otherwise.
     */
    public boolean isCellSelected(Object obj)
    {
        return getSelectionModel().isCellSelected(obj);
    }

    /**
     * Scrolls the slide window so that the specified cell becomes visible on the application.
     *
     * @param obj The cell that is to be seen on the screen.
     */
    public void scrollCellToVisible(Object obj)
    {
        Rectangle rectangle = getCellBounds(obj);
        if(rectangle != null)
        {
            rectangle = new Rectangle(rectangle);
            scrollRectToVisible(toScreen(rectangle));
        }
    }

    /**
     * Scrolls the slide window so that the specified point becomes visible. The method automatically does the
     * translation from the point to a cell on the slide.
     *
     * @param point The point that is to be made visible on the screen.
     */
    public void scrollPointToVisible(Point point)
    {
        if(point != null)
        {
            Rectangle rectangle = new Rectangle(point);
            if(rectangle != null)
            {
                scrollRectToVisible(rectangle);
            }
        }
    }

    /**
     * Determines whether the graphpad (the slide) is currently being edited by the user.
     *
     * @return True if any component in the slide is being edited, false otherwise.
     */
    public boolean isEditing()
    {
        GraphUI graphui = getUI();
        if(graphui != null)
        {
            return graphui.isEditing(this);
        } else
        {
            return false;
        }
    }

    /**
     * Stops editing of the cells in the slide.
     *
     * @return True if it is time to stop editing. (Taken from GraphUI).
     */
    public boolean stopEditing()
    {
        GraphUI graphui = getUI();
        if(graphui != null)
        {
            return graphui.stopEditing(this);
        } else
        {
            return false;
        }
    }

    /**
     * Cancles editing of the current graphpad.
     *
     */
    public void cancelEditing()
    {
        GraphUI graphui = getUI();
        if(graphui != null)
        {
            graphui.cancelEditing(this);
        }
    }

    /**
     * Specifies the cell on the graph at which the editing of the cells should start.
     *
     * @param obj The cell at which editing should start.
     */
    public void startEditingAtCell(Object obj)
    {
        GraphUI graphui = getUI();
        if(graphui != null)
        {
            graphui.startEditingAtCell(this, obj);
        }
    }

    /**
     * Returns the cell on the graph that is currently being edited.
     *
     * @return The cells that is being edited.
     */
    public Object getEditingCell()
    {
        GraphUI graphui = getUI();
        if(graphui != null)
        {
            return graphui.getEditingCell(this);
        } else
        {
            return null;
        }
    }

    /**
     * This method is called when an event caused changes to the graph layout or its components. The graph is then
     * revalidated to check for any errors and repainted so that the user sees the changes.
     */
    public void graphDidChange()
    {
        revalidate();
        repaint();
    }

    /**
     * Serializes the entire graphpad application. The objects that are serialized are the graphmodel, graphlayoutcahe,
     * selectionmodel and the graphUI.
     *
     * @param objectoutputstream The stream to which the serialized data should be written.
     * @throws IOException Throws an exception if the data cannot be written to the objectoutputstream.
     */
    private void writeObject(ObjectOutputStream objectoutputstream)
        throws IOException
    {
        Vector vector = new Vector();
        objectoutputstream.defaultWriteObject();
        if(graphModel instanceof Serializable)
        {
            vector.addElement("graphModel");
            vector.addElement(graphModel);
        }
        if(graphLayoutCache instanceof Serializable)
        {
            vector.addElement("graphLayoutCache");
            vector.addElement(graphLayoutCache);
        }
        if(selectionModel instanceof Serializable)
        {
            vector.addElement("selectionModel");
            vector.addElement(selectionModel);
        }
        objectoutputstream.writeObject(vector);
        if(getUIClassID().equals("GraphUI") && ui != null)
        {
            ui.installUI(this);
        }
    }

    /**
     * Reads objects from a serialized object input stream and resets the application to the objects that it reads in.
     * The objects read in are the graphModel, the graphlayoutcache, selectionmodel and the graphUI.
     *
     * @param objectinputstream The stream from which the serialized input is read.
     * @throws IOException Thrown if there is trouble reading data from the objectinputstream.
     * @throws java.lang.ClassNotFoundException Thrown if the objectinputstream reader is unable to locate the classes
     * to which the serialized objects that are read in from the input stream belong.
     */
    private void readObject(ObjectInputStream objectinputstream)
        throws IOException, ClassNotFoundException
    {
        objectinputstream.defaultReadObject();
        Vector vector = (Vector)objectinputstream.readObject();
        int i = 0;
        int j = vector.size();
        if(i < j && vector.elementAt(i).equals("graphModel"))
        {
            graphModel = (GraphModel)vector.elementAt(++i);
            i++;
        }
        if(i < j && vector.elementAt(i).equals("graphLayoutCache"))
        {
            graphLayoutCache = (GraphLayoutCache)vector.elementAt(++i);
            i++;
        }
        if(i < j && vector.elementAt(i).equals("selectionModel"))
        {
            selectionModel = (GraphSelectionModel)vector.elementAt(++i);
            i++;
        }
        if(listenerList.getListenerCount(GraphSelectionListener.class) != 0)
        {
            selectionRedirector = new GraphSelectionRedirector();
            selectionModel.addGraphSelectionListener(selectionRedirector);
        }
    }

    /**
     * Returns the preferred size of the ports when scrolling.
     *
     * @return A dimension object that returns the size of the scrollable area.
     */
    public Dimension getPreferredScrollableViewportSize()
    {
        return getPreferredSize();
    }

    /**
     * Gets the number of units that the application scrolls down on a single click. One "unit" is the distance between
     * two adjacent grid lines.
     *
     * The number units returned is 2 is i==1 and 4 otherwise. More specifically, if i==1, it gives the vertical increment
     * but if i==1, it gives the horizontal increment.
     *
     * NOTE: The rectangle variable and j are not used anywhere.
     *
     * @param rectangle NOT USED
     * @param i A flag to choose the number of units to scroll. If i == 1, it returns the vertical units, else horizontal units.
     * @param j NOT USED
     * @return The number of units moved on a single scroll. <code>return i != 1 ? 4 : 2;</code>
     */
    public int getScrollableUnitIncrement(Rectangle rectangle, int i, int j)
    {
        return i != 1 ? 4 : 2;
    }

    /**
     * Gets the increment for a single scroll in terms of blocks. Blocks are determined in terms of the size of the
     * rectangle specified.
     *
     * if i ==1, it returns the height of the specified rectangle, else specifies the width of the rectangle.
     *
     * Variable j is never used.
     *
     * @param rectangle The rectangle in terms of which the increments are defined.
     * @param i Chooses between height and width. If i==1, returns the vertical increment, else the horizontal one.
     * @param j NEVER USED.
     * @return The number of blocks that make up the increment.
     */
    public int getScrollableBlockIncrement(Rectangle rectangle, int i, int j)
    {
        return i != 1 ? rectangle.width : rectangle.height;
    }

    /**
     * Determines whether the width of the component's parent is greater than the preferred width.
     *
     * @return True if width of the component's parent is greater than the preferred width, false otherwise.
     */
    public boolean getScrollableTracksViewportWidth()
    {
        if(getParent() instanceof JViewport)
        {
            return ((JViewport)getParent()).getWidth() > getPreferredSize().width;
        } else
        {
            return false;
        }
    }

    /**
     * Determines whether the height of the component's parent is greater than the preferred height.
     *
     * @return True if height of the component's parent is greater than the preferred height, false otherwise.
     */
    public boolean getScrollableTracksViewportHeight()
    {
        if(getParent() instanceof JViewport)
        {
            return ((JViewport)getParent()).getHeight() > getPreferredSize().height;
        } else
        {
            return false;
        }
    }

    /**
     * Returns a string that describes the options for the component. The string contains information about it's
     * parents's parameters, whether it is editable and which the editing of the cells should stop.
     *
     * The use of this method is not very clear.
     *
     * @return <code>return super.paramString() + ",editable=" + s + ",invokesStopCellEditing=" + s1;</code>
     */
    protected String paramString()
    {
        String s = editable ? "true" : "false";
        String s1 = invokesStopCellEditing ? "true" : "false";
        return super.paramString() + ",editable=" + s + ",invokesStopCellEditing=" + s1;
    }

    /**
     * This method produces a java application describing the class heirarchies for this application.
     *
     * @param args The command line arguments to the application.
     */
    public static void main(String args[])
    {
        if(args.length > 0 && args[0].equals("-version"))
        {
            System.out.println("JGraph Geneva (v2.0)");
        } else
        {
            JFrame jframe = new JFrame("JGraph Geneva (v2.0)");
            JGraph jgraph = new JGraph();
            jgraph.setDragEnabled(true);
            jframe.getContentPane().add(new JScrollPane(jgraph));
            jframe.pack();
            jframe.show();
        }
    }
}
