/*
 * @(#)ParentMap.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.io.Serializable;
import java.util.*;
import javax.swing.tree.TreeNode;

/**
 * An object that describes relations between childs and parents.
 *
 * Referenced classes of package com.jgraph.graph are
 * Port and GraphModel.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see Serializable
 */
public class ParentMap
implements Serializable {
    
    /**
     * Object that represents the relation between a child an a parent.
     */
    public class Entry
    implements Serializable {
      
        /** 
         * An instance of Object.
         * Child and parent of the relation this entry describes. 
         */
        protected Object child;
        
        /** 
         * An instance of Object.
         * Child and parent of the relation this entry describes. 
         */
        protected Object parent;
        
        /** 
         * Returns the child of the relation.
         * @return An Object data type.
         * @see Object
         */
        public Object getChild() {
            return child;
        }
        
        /** 
         * Returns the parent of the relation.
         * @return An Object data type.
         * @see Object
         */
        public Object getParent() {
            return parent;
        }
        
        /** 
         * Constructs a new relation between child
         * and parent.
         * @param obj A variable of Object data type.
         * @param obj1 A variable of Object data type.
         * @see Object
         */
        public Entry(Object obj, Object obj1) {
            child = obj;
            parent = obj1;
        }
    }
    
    
    /** 
     * An instance of ArrayList.
     * Contents of the parent map. 
     */
    protected ArrayList entries;
    
    /**
     * An instance of Set. 
     * Set of changed changedNodes for the parent map. Includes childs and parents. 
     */
    protected Set changedNodes;
    
    /**
     * An instance of Map.  
     * Maps parents to integers with the future number of childs. 
     */
    protected Map childCount;
    
    /** 
     * Constructor for the ParentMap.
     * Creates a new ArrayList, HashSet, and HashTable
     */
    public ParentMap() {
        entries = new ArrayList();
        changedNodes = new HashSet();
        childCount = new Hashtable();
    }
    
    /** 
     * Returns a parent map that represents the insertion or
     * removal of cells in model based on remove.
     * Unselected childs of selected nodes are moved to the first
     * unselected parent of that node.
     * Note: Consequently, cells "move up" one level
     * when their parent is removed.
     * @param graphmodel A variable of GraphModel data type.
     * @param aobj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @return A ParentMap data type.
     * @see GraphModel
     * @see Object
     * @see ParentMap
     */
    public static ParentMap create(GraphModel graphmodel, Object aobj[], boolean flag) {
        HashSet hashset = new HashSet();
        for(int i = 0; i < aobj.length; i++) {
            hashset.add(aobj[i]);
        }
        
        ParentMap parentmap = new ParentMap();
        for(int j = 0; j < aobj.length; j++) {
            // Collect Parent Information
            Object obj = graphmodel.getParent(aobj[j]);
            if(obj != null && !hashset.contains(obj)) {
                parentmap.addEntry(aobj[j], flag ? null : obj);
            }
            if(flag) {
                // Move Orphans to First Unselected Parent
                for(; hashset.contains(obj); obj = graphmodel.getParent(obj)) { }
                for(int k = 0; k < graphmodel.getChildCount(aobj[j]); k++) {
                    Object obj1 = graphmodel.getChild(aobj[j], k);
                    if(!hashset.contains(obj1)) {
                        parentmap.addEntry(obj1, obj);
                    }
                }
                
            }
        }
        
        return parentmap;
    }
    
    /** 
     * Set parent of child in the parent map.
     * Note: This reduces changes the number of childs for old parent
     * by -1 and new parent to +1 only if child is not an istanceof Port.
     * (This is required to avoid removing parent vertices when all
     * ports are removed.)
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @see Object
     */
    public void addEntry(Object obj, Object obj1) {
        if(obj != null) {
            Object obj2 = null;
            if(obj instanceof TreeNode) {
                TreeNode treenode = ((TreeNode)obj).getParent();
                if(!(obj instanceof Port)) {
                    changeChildCount(treenode, -1);
                }
                if(treenode != null) {
                    changedNodes.add(treenode);
                }
            }
            entries.add(new Entry(obj, obj1));
            if(!(obj instanceof Port)) {
                changeChildCount(obj1, 1);
            }
            // Update Changed Nodes
            changedNodes.add(obj);
            if(obj1 != null) {
                changedNodes.add(obj1);
            }
        }
    }
    
    /** 
     * Returns an Iterator for the entries in the map.
     * @param obj A variable of Object data type.
     * @param i A variable of integer data type.
     * @see Object
     */
    protected void changeChildCount(Object obj, int i) {
        if(obj != null) {
            Integer integer = (Integer)childCount.get(obj);
            if(integer == null) {
                if(obj instanceof TreeNode) {
                    integer = new Integer(((TreeNode)obj).getChildCount());
                } else {
                    integer = new Integer(0);
                }
            }
            int j = integer.intValue() + i;
            childCount.put(obj, new Integer(j));
        }
    }
    
    /** 
     * Returns the parents that will be empty when
     * this parent map is applied.
     * @return An Object data type.
     * @see Object
     */
    public Object[] emptyParents() {
        ArrayList arraylist = new ArrayList();
        for(Iterator iterator = childCount.entrySet().iterator(); iterator.hasNext();) {
            java.util.Map.Entry entry = (java.util.Map.Entry)iterator.next();
            if((entry.getValue() instanceof Integer) && ((Integer)entry.getValue()).intValue() == 0) {
                arraylist.add(entry.getKey());
            }
        }
        
        return arraylist.toArray();
    }
    
    /** 
     * Returns the number of entries.
     * @return An integer data type.
     */
    public int size() {
        return entries.size();
    }
    
    /** 
     * Returns an Iterator for the entries in the map.
     * @return An Iterator data type.
     * @see Iterator
     */
    public Iterator entries() {
        return entries.iterator();
    }
    
    /** 
     * Returns a Set for the nodes, childs and parents,
     * in this parent map.
     * @return A Set data type.
     * @see Set
     */
    public Set getChangedNodes() {
        return changedNodes;
    }
    
    
    /** 
     * Returns a string with child and parent information.
     * @return A String data type.
     */
    public String toString() {
        String s = super.toString() + "\n";
        for(Iterator iterator = entries(); iterator.hasNext();) {
            Entry entry = (Entry)iterator.next();
            s = s + " child=" + entry.getChild() + " parent=" + entry.getParent() + "\n";
        }
        
        return s;
    }
}
