/*
 * @(#)PortRenderer.java	1.0 28/11/01
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.*;
import java.io.Serializable;
import javax.swing.JComponent;
import javax.swing.UIManager;

/**
 * This renderer displays entries that implement the CellView interface
 * and supports the following attributes:
 * <li>
 * GraphConstants.OFFSET
 * GraphConstants.ABSOLUTE
 * </li
 *
 * Referenced classes of package com.jgraph.graph are
 * PortView, CellViewRenderer, GraphConstants and CellView.
 *
 * @author Gaudenz Alder
 * @version 1.0 28/11/01
 * 
 * @see JComponent
 * @see CellViewRenderer
 * @see Serializable
 */

public class PortRenderer extends JComponent
implements CellViewRenderer, Serializable {
    /**
     * An instance of JGraph.
     * Cache the current graph for drawing.
     */
    protected transient JGraph graph;
    
    /**
     * An instance of PortView.
     * Cache the current edgeview for drawing.
     */
    protected transient PortView view;
    
    /**
     * An instance of boolean.
     * Cached hasFocus and selected value.
     */
    protected transient boolean hasFocus;
    
    /**
     * An instance of boolean.
     * Cached hasFocus and selected value.
     */
    protected transient boolean selected;
    
    /**
     * An instance of boolean.
     * Cached hasFocus and selected value.
     */
    protected transient boolean preview;
    
    /**
     * Constructs a renderer that may be used to render ports.
     */
    public PortRenderer() {
        setForeground(UIManager.getColor("MenuItem.selectionBackground"));
        setBackground(UIManager.getColor("Tree.selectionBorderColor"));
    }
    
    /**
     * Configure and return the renderer based on the passed in
     * components. The value is typically set from messaging the
     * graph with convertValueToString.
     * @param jgraph A variable of JGraph data type.
     * @param cellview A variable of CellView data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @param flag2 A variable of boolean data type.
     * @return A Component data type.
     * @see JGraph
     * @see CellView
     * @see Component
     */
    public Component getRendererComponent(JGraph jgraph, CellView cellview, boolean flag, boolean flag1, boolean flag2) {
        if((cellview instanceof PortView) && jgraph != null) {
            graph = jgraph;
            view = (PortView)cellview;
            //this.hasFocus = hasFocus;
            selected = flag;
            preview = flag2;
            return this;
        } else {
            return null;
        }
    }
    
    /**
     * Paint the renderer. Overrides superclass paint
     * to add specific painting. Note: The preview flag
     * is interpreted as "highlight" in this context. (This
     * is used to highlight the port if the mouse is over it.)
     * @param g A variable of Graphics data type.
     * @see Graphics
     */
    public void paint(Graphics g) {
        Dimension dimension = getSize();
        g.setColor(graph.getBackground());
        g.setXORMode(graph.getBackground());
        if(preview) {
            g.setColor(getBackground());
            g.drawRect(1, 1, dimension.width - 3, dimension.height - 3);
            g.drawRect(2, 2, dimension.width - 5, dimension.height - 5);
        }
        boolean flag = GraphConstants.getOffset(view.getAllAttributes()) != null;
        java.awt.Color color = GraphConstants.getForeground(view.getAllAttributes());
        g.setColor(getForeground());
        if(flag) {
            g.drawLine(2, 2, dimension.width - 2, dimension.height - 2);
            g.drawLine(2, dimension.width - 2, dimension.height - 2, 2);
        } else
            if(!preview) {
                g.drawRect(3, 3, dimension.width - 4, dimension.height - 4);
            }
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void validate() {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void revalidate() {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param l A variable of long data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @param k A variable of integer data type.
     * @param i1 A variable of integer data type.
     */
    public void repaint(long l, int i, int j, int k, int i1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param rectangle A variable of Rectangle data type.
     * @see Rectangle
     */
    public void repaint(Rectangle rectangle) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @see Object
     */
    protected void firePropertyChange(String s, Object obj, Object obj1) {
        // Strings get interned...
        if(s == "text") {
            super.firePropertyChange(s, obj, obj1);
        }
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param byte0 A variable of byte data type.
     * @param byte1 A variable of byte data type.
     */
    public void firePropertyChange(String s, byte byte0, byte byte1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param c A variable of char data type.
     * @param c1 A variable of char data type.
     */
    public void firePropertyChange(String s, char c, char c1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param word0 A variable of short data type.
     * @param word1 A variable of short data type.
     */
    public void firePropertyChange(String s, short word0, short word1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     */
    public void firePropertyChange(String s, int i, int j) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param l A variable of long data type.
     * @param l1 A variable of long data type.
     */
    public void firePropertyChange(String s, long l, long l1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param f A variable of float data type.
     * @param f1 A variable of float data type.
     */
    public void firePropertyChange(String s, float f, float f1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param d A variable of double data type.
     * @param d1 A variable of double data type.
     */
    public void firePropertyChange(String s, double d, double d1) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     */
    public void firePropertyChange(String s, boolean flag, boolean flag1) {
    }
}
