/*
 * @(#)PortView.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Point;
import java.awt.Rectangle;

/**
 * The default implementation of a port view.
 *
 * Referenced classes of package com.jgraph.graph are
 * AbstractCellView, Port, VertexView, CellView, PortRenderer, CellMapper,
 * GraphConstants, EdgeView, CellViewRenderer, GraphContext and CellHandle.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see AbstactCellView
 */

public class PortView extends AbstractCellView {
    
    /** 
     * Default size for all ports is 6. 
     */
    protected static int size = 6;
    
    /** 
     * Renderer for the class. 
     */
    public static PortRenderer renderer = new PortRenderer();
    
    /** 
     * Cache of the last valid parent. //FIX: Better solution? 
     */
    protected transient CellView lastParent;
    
    /** 
     * Constructs a view that holds a reference to the specified cell,
     * anchor and parent vertex.
     * @param obj A variable of Object data type.
     * @param jgraph A variable of JGraph data type.
     * @param cellmapper A variable of CellMapper data type.
     * @see Object
     * @see JGraph
     * @see CellMapper
     */
    public PortView(Object obj, JGraph jgraph, CellMapper cellmapper) {
        super(obj, jgraph, cellmapper);
    }
    
    //
    // CellView interface
    //
    
    /** 
     * This method ensures a non-null value. If the super method
     * returns null then the last valid parent is returned.
     * Note: If a vertex is removed, all ports will be replaced
     * in connected edges. The ports are replaced by the center
     * point of the last valid vertex view.
     * @return A CellView data type.
     * @see CellView
     */
    public CellView getParentView() {
        CellView cellview = super.getParentView();
        if(cellview == null) {
            cellview = lastParent;
        } else {
            lastParent = cellview;
        }
        return cellview;
    }
    
    /** 
     * Returns the bounds for the port view.
     * @return A Rectangle data type.
     * @see Rectangle
     */
    public Rectangle getBounds() {
        Rectangle rectangle = new Rectangle(getLocation(null));
        rectangle.x = rectangle.x - size / 2;
        rectangle.y = rectangle.y - size / 2;
        rectangle.width = rectangle.width + size;
        rectangle.height = rectangle.height + size;
        return rectangle;
    }
    
    /** 
     * Returns a renderer for the class.
     * @return A CellViewRenderer data type.
     * @see CellViewRenderer
     */
    public CellViewRenderer getRenderer() {
        return renderer;
    }
    
    /** 
     * GetHandle() functions - Returns null
     * @param graphcontext A variable of GraphContext data type.
     * @return null
     * @see GraphContext
     */
    public CellHandle getHandle(GraphContext graphcontext) {
        return null;
    }
    
    //
    // Special Methods
    //
    
    /** 
     * Returns the point that the port represents with respect
     * to edge. Edge may be > null.
     * @param edgeview A variable of EdgeVew data type.
     * @return A Point data type.
     * @see EdgeView
     * @see Point
     */
    public Point getLocation(EdgeView edgeview) {
        Port port = null;
        if(cell instanceof Port) {
            port = ((Port)cell).getAnchor();
        }
        PortView portview = (PortView)mapper.getMapping(port, false);
        Point point = null;
        boolean flag = GraphConstants.isAbsolute(allAttributes);
        Point point1 = GraphConstants.getOffset(allAttributes);
        VertexView vertexview = (VertexView)getParentView();
        // If No Edge Return Center
        if(vertexview != null) {
            if(edgeview == null && point1 == null) {
                point = vertexview.getCenterPoint();
            }
            // Apply Offset
            if(point1 != null) {
                int i = point1.x;
                int j = point1.y;
                Rectangle rectangle = vertexview.getBounds();
                // Absolute Offset
                if(!flag) {
                    i = (i * (rectangle.width - 1)) / 1000;
                    j = (j * (rectangle.height - 1)) / 1000;
                } // Offset from Anchor
                point = portview == null ? rectangle.getLocation() : portview.getLocation(edgeview);
                point = new Point(point.x + i, point.y + j);
            } else
                if(edgeview != null) {
                    // Floating Port
                    Point point2 = getNextPoint(edgeview);
                    if(point2 == null) {
                        // If "Dangling" Port Return Center
                        return vertexview.getCenterPoint();
                    }
                    point = vertexview.getPerimeterPoint(point, point2);
                }
        }
        return point;
    }
    
    /** 
     * Returns the point that is closest to the port view on
     * edge. Returns null if edge has less than 2 points.
     * @param edgeview A variable of EdgeVew data type.
     * @return A Point data type.
     * @see EdgeView
     * @see Point
     */
    protected Point getNextPoint(EdgeView edgeview) {
        int i = edgeview.getPointCount();
        if(i > 1) {
            if(edgeview.getSource() == this) {
                return getEdgePoint(edgeview, 1);
            }
            if(edgeview.getTarget() == this) {
                return getEdgePoint(edgeview, i - 2);
            }
        }
        return null;
    }
    
    /** 
     * Returns the point of edge at index.
     * Avoids calling getLocation on the opposite port of
     * edge (possible infinite recursion).
     * @param edgeview A variable of EdgeView data type.
     * @param i A variable of integer data type.
     * @return A Point data type.
     * @see EdgeView
     * @see Point
     */
    protected Point getEdgePoint(EdgeView edgeview, int i) {
        Object obj = edgeview.points.get(i);
        if(obj instanceof Point) {
            return (Point)obj;
        }
        if(obj instanceof PortView) {
            VertexView vertexview = (VertexView)((CellView)obj).getParentView();
            if(vertexview != null) {
                return vertexview.getCenterPoint();
            }
        }
        return null;
    }
    
}
