import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.Hashtable;



/** This is the Touch class.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class Touch implements Runnable {

        /** The main graph object.
         */
	private GPGraph graph;
	/** A hashtable for the deltas.
         */
        private Hashtable deltas = new Hashtable();
        /** A hashtable for the positions.
         */
	private Hashtable positions = new Hashtable();
        /** A relaxer thread.
         */
	private Thread relaxer;
        /** Boolean value if it is allow to run.
         */
	private boolean allowedToRun = false;
        /** Boolean value if repaint is needed.
         */
	private boolean repaintNeeded = false;
        /** A double value for damper.
         */
	private double damper = 1.0;
	// A low damper value causes the graph to move slowly
        /** A double for the max value for motion.
         */
	private double maxMotion = 0;
	// Keep an eye on the fastest moving node to see if the graph is stabilizing
        /** Keep the value of the last max motion.
         */
	private double lastMaxMotion = 0;
        /** The ratio value for the motion.
         */
	private double motionRatio = 0;
	// It'buttonSelect sort of a ratio, equal to lastMaxMotion/maxMotion-1
        /** A boolean for the damping.
         */
	private boolean damping = true;
	// When damping is true, the damper value decreases
        /** A value for the rigidity.
         */
	private double rigidity = 1.5;
	// Rigidity has the same effect as the damper, except that it'buttonSelect a constant
	// a low rigidity value causes things to go slowly.
	// a value that'buttonSelect too high will cause oscillation
        /** The new value for the rigidity.
         */
	private double newRigidity = 1;

	// ............

	/** Constructor with a supplied TGPanel <tt>tgp</tt>.
         * @param graph This GPGraph will allow the creation of a Touch object */
	public Touch(GPGraph graph) {
		this.graph = graph;
		relaxer = null;
	}

        /** This set the regidity.
         */
	void setRigidity(double r) {
		newRigidity = r; //update rigidity at the end of the relax() thread
	}

	//relaxEdges is more like tense edges up.  All edges pull nodes closer together;
        /** This relax the edges.  The nodes are pull closer together.
         */
	private synchronized void relaxEdges() {
		Object[] edges = graph.getEdges(graph.getAll());
		//System.out.println("edges="+edges.length);
		for (int i = 0; i < edges.length; i++) {
			CellView from = graph.getSourceView(edges[i]);
			CellView to = graph.getTargetView(edges[i]);
			CellView fromV =
				graph.getGraphLayoutCache().getMapping(
					((Edge) edges[i]).getSource(),
					false);
			CellView toV =
				graph.getGraphLayoutCache().getMapping(
					((Edge) edges[i]).getTarget(),
					false);

			if (from != null && to != null) {
				Rectangle bf = fromV.getBounds();
				// GraphConstants.getBounds(fromV.getAttributes());
				Rectangle bt = toV.getBounds();
				//GraphConstants.getBounds(toV.getAttributes());
				//Point2D.Double bf = getPosition(from);
				//Point2D.Double bt = getPosition(to);
				double vx = bt.x - bf.x;
				double vy = bt.y - bf.y;
				double len = Math.sqrt(vx * vx + vy * vy);
				double dx = vx * rigidity; //rigidity makes edges tighter
				double dy = vy * rigidity;
				double length = getLength(edges[i]) * 100;
				dx /= length;
				dy /= length;
				moveView(to, -dx * len, -dy * len);
				moveView(from, dx * len, dy * len);

			}
		}
	}

        /** This gets the length of an edge.
         * @param edge the edge of which to get the length
         * @return the length of the edge
         */        
	public double getLength(Object edge) {
		CellView view = graph.getGraphLayoutCache().getMapping(edge, false);
		return GPGraphTools.getLength(view);
	}

        /** This avoids the labels.
         */
	private synchronized void avoidLabels() {
		Object[] vertices = graph.getVertices(graph.getAll());
		for (int i = 0; i < vertices.length; i++) {
			for (int j = i + 1; j < vertices.length; j++) {
				CellView from = graph.getGraphLayoutCache().getMapping(vertices[i], false);
				CellView to = graph.getGraphLayoutCache().getMapping(vertices[j], false);
				if (from != null && to != null) {
					Point2D.Double bf = getPosition(from);
					Point2D.Double bt = getPosition(to);

					double dx = 0;
					double dy = 0;
					double vx = bf.x - bt.x;
					double vy = bf.y - bt.y;
					double len = vx * vx + vy * vy; //so it'buttonSelect length squared
					if (len == 0) {
						dx = 0.1;
						dy = 0.1;
					} else if (len < 200 * 200) {
						dx = vx / len;
						dy = vy / len;
					}

					int buffer = 200;

					double repX = Math.max(from.getBounds().getWidth(), to.getBounds().getWidth());
					double repY = Math.max(from.getBounds().getHeight(), to.getBounds().getHeight());
					repX=repX*4;
					repY=repY*4;
					repX = repX*repX/100;
					repY = repY*repY/100;

					repX = Math.max(repX,repY);
					if (Math.random()>0.3){
						moveView(from, dx * repX, dy * repY);
						moveView(to, -dx * repY, -dy * repX);
					}
					else {
						moveView(from, dx * repX*3, dy * repY*3);
						moveView(to, -dx * repX, -dy * repY*3);
					}
//					else {
//						moveView(from, dx * repX*15*Math.random(), dy * repY);
//						moveView(to, -dx * repX*10*Math.random(), -dy * repY*10*Math.random());
//					}
				}
			}
		}
	}

        /** This starts the damping effect.
         */        
	public void startDamper() {
		damping = true;
	}

        /** This stops the damping effect.
         */        
	public void stopDamper() {
		damping = false;
		damper = 1.0; //A value of 1.0 means no damping
	}

        /** This resets the damping effect.
         */        
	public void resetDamper() { //reset the damper, but don't keep damping.
		damping = true;
		damper = 1.0;
	}

        /** This creates a new value for the damping effect.
         * @param newValue 1.0 means no damping, a low damping value causes the graph to move slowly */        
	public void setDamper(double newValue) {
		damper = newValue;
	}

        /** This is where the damping effect is implemented.
         */        
	public void damp() {
		if (damping) {
			if (motionRatio <= 0.001) {
				//This is important.  Only damp when the graph starts to move faster
				//When there is noise, you damp roughly half the time. (Which is a lot)
				//
				//If things are slowing down, then you can let them do so on their own,
				//without damping.

				//If max motion<0.2, damp away
				//If by the time the damper has ticked down to 0.9, maxMotion is still>1, damp away
				//We never want the damper to be negative though
				if ((maxMotion < 0.2 || (maxMotion > 1 && damper < 0.9))
					&& damper > 0.01)
					damper -= 0.01;
				//If we've slowed down significanly, damp more aggresively (then the line two below)
				else if (maxMotion < 0.4 && damper > 0.003)
					damper -= 0.003;
				//If max motion is pretty high, and we just started damping, then only damp slightly
				else if (damper > 0.0001)
					damper -= 0.0001;
			}
		}
		if (maxMotion < 0.001 && damping)
			damper = 0;
		//System.out.println("damper="+damper);
	}

        /** This move the nodes.
         */
	private synchronized void moveNodes() {
		lastMaxMotion = maxMotion;
		double maxMotionA = 0;
		Object[] vertices = graph.getVertices(graph.getAll());
		for (int i = 0; i < vertices.length; i++) {
			CellView view = graph.getGraphLayoutCache().getMapping(vertices[i], false);
			if (view != null) {
				Rectangle bounds =
					GraphConstants.getBounds(view.getAllAttributes());
				Point2D.Double delta = getDelta(view);
				Point2D.Double position = getPosition(view);
				//deltas.remove(view);
				double dx = delta.getX();
				double dy = delta.getY();
				dx *= damper;
				dy *= damper;
				delta.setLocation(dx / 2, dy / 2);
				double distMoved = Math.sqrt(dx * dx + dy * dy);
				if (GraphConstants.isMoveable(view.getAllAttributes())
					&& !graph.isCellSelected(vertices[i])
					&& (dx != 0 || dy != 0)) {
					//System.out.println("dx: " + dx + " dy: "+dy);
					position.x += Math.max(-5, Math.min(5, dx));
					position.y += Math.max(-5, Math.min(5, dy));
					bounds.x = Math.max(0, (int) position.x - bounds.width/2);
					bounds.y =
						Math.max(0, (int) position.y - bounds.height/2);
					repaintNeeded = true;
					// Invalidate cached edge shapes
					Object[] edges = DefaultGraphModel.getEdges(graph.getModel(), new Object[]{view.getCell()}).toArray();
					CellView[] v = graph.getGraphLayoutCache().getMapping(edges, false);
					for (int j=0; j<v.length; j++)
					 if (v[j] instanceof EdgeView)
					 	((EdgeView) v[j]).update();
				}
				maxMotionA = Math.max(distMoved, maxMotionA);

			}
		};
		maxMotion = maxMotionA;
		if (maxMotion > 0)
			motionRatio = lastMaxMotion / maxMotion - 1;
		else
			motionRatio = 0;
		damp();
	}

        /** This relax the edges.
         */
	private synchronized void relax() {
		for (int i = 0; i < 10; i++) {
			relaxEdges();
			avoidLabels();
			moveNodes();
		}
		if (rigidity != newRigidity)
			rigidity = newRigidity; //update rigidity
		if (repaintNeeded) {
			graph.repaint();
			repaintNeeded = false;
		}
	}

        /** This runs the thread.
         */        
	public void run() {
		Thread me = Thread.currentThread();
		while (relaxer == me && allowedToRun) {
			relax();
			//System.out.println("D "+damper);
			try {
				relaxer.sleep(20);
			} catch (InterruptedException e) {
				break;
			}
		}
	}

        /** This starts the thread.
         */        
	public void start() {
		relaxer = new Thread(this);
		allowedToRun = true;
		relaxer.start();
	}

        /** This tells whether or not the thread is running.
         * @return true if it is running
         */        
	public boolean isRunning() {
		if (relaxer != null)
			return relaxer.isAlive();
		return false;
	}

        /** This stops the thread from running.
         */        
	public void stop() {
		allowedToRun = false;
		relaxer = null;
	}

        /** This gets the position of a CellView.
         * @param view the object to get the position of
         * @return the position
         */        
	public Point2D.Double getPosition(CellView view) {
		Point2D.Double p1 = (Point2D.Double) positions.get(view);

		Rectangle rect = GraphConstants.getBounds(view.getAllAttributes());
		Point2D.Double p2 =
			new Point2D.Double(
				rect.x + rect.width / 2,
				rect.y + rect.height / 2);

		if (p1 != null) {
			if (Math.abs(p1.x - p2.x) > 5 || Math.abs(p1.y - p2.y) > 5) {
				//System.out.println("p1.x " +p1.x +" p2.x " +p2.x);
				p1.setLocation(p2.x, p2.y);
			}
			return p1;
		} else {
			positions.put(view, p2);
			return p2;
		}
	}

        /** This gets the delta value of the CellView.
         * @param view the object to get the delta value from
         * @return the delta value
         */        
	public Point2D.Double getDelta(CellView view) {
		Point2D.Double p = (Point2D.Double) deltas.get(view);
		if (p == null) {
			p = new Point2D.Double(0, 0);
			deltas.put(view, p);
		}
		return p;
	}

        /** This allows a movement of the CellView.
         * @param view the object to move
         * @param dx movement in x axis
         * @param dy movement in y axis
         */        
	public void moveView(CellView view, double dx, double dy) {
		//System.out.println("set Delta for "+view+" to "+dx+", "+dy);
		Point2D.Double p = getDelta(view);
		p.setLocation(p.getX() + dx, p.getY() + dy);
	}

}