/**
 *  Building Composer, http://bc.cecer.army.mil
 *
 *  ERDC - Engineering Research and Development Center
 *  USACERL - US Army Construction Engineering Research Laboratory
 *
 *  All rights reserved, 2002.  Please see included license.
 */

import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * This class allows the user to edit and add properties.
 *
 * @author US Army Construction Engineering Research Laboratory
 * @created February 23, 2002
 * @version 2.0 TerpPresent
 */
public class UserProperties extends Properties {

	//protected static EnvProperties envProps = new EnvProperties();
    /** This is a vector that holds the properties.
     */    
	protected static Vector instances = new Vector();
        /** This string holds the username.
         */        
	protected String nameSpace;
        /** This is the file location.
         */        
	protected File file;


	/**
	 * Constructor for UserProperties.
	 */
	private UserProperties() {
	}


	/**
	 * Constructor for UserProperties.
	 *
	 * @param nameSpace This is the username
	 */
	private UserProperties(String nameSpace) {
		String USERHOME = System.getProperty("user.home");
		//envProps.getProperty("USERPROFILE");
		if (USERHOME == null) {
			USERHOME = "";
		}
		this.nameSpace = USERHOME + File.separator + nameSpace;
		String fullpath = this.nameSpace + File.separator + "user.properties";
		File file = new File(fullpath);

		// check for properties file
		if (!file.exists() || !file.isFile()) {
			try {
				createRecursiveFolder(new File(this.nameSpace));
				file.createNewFile();
			}
			catch (IOException e) {
				System.out.println(e);
				// throw an unchecked exception, and then create default
			}
		}
		// else if (!file.canRead() || !file.canWrite()) {
			//try to remedy, else create default
		// }

		this.file = file;

		// init properties
		readPropertiesFromFile(this, file, true);
	}


	/**
	 * Multi-Singleton instance factory method.
	 *
	 * @param nameSpace nameSpace is the name of the directory that user properties will be stored.
	 *      This acts as a 'namespace' by which sets of properties can collectively be stored. For each
	 *      new nameSpace, a new directory (prepended with '.') is created and properties stored in a
	 *      file named user.properties. For example, if 'example' is passed as nameSpace, the following
	 *      directory would be created C:\\Documents and Settings\\userlogin\\.testapp\\user.properties
	 *      (Win2k).
	 * @return the UserProperties instance
	 */
	public final static UserProperties getInstance(String nameSpace) {
		// check for existing instance with same nameSpace
		for (int i = 0; i < instances.size(); i++) {
			UserProperties instance = (UserProperties) instances.elementAt(i);
			if (nameSpace.equals(instance.nameSpace)) {
				return instance;
			}
		}

		// else, if not found, create
		UserProperties newInstance = new UserProperties(nameSpace);
		instances.add(newInstance);
		return newInstance;
	}


	/**
	 * This allows everything to be saved.
	 */
	public static void saveAll() {
		for (int i = 0; i < instances.size(); i++) {
			UserProperties instance = (UserProperties) instances.elementAt(i);
			instance.save();
		}
	}


	/**
	 * This allows the reading of properties from a specified file.
	 *
	 * @param properties the object in which the properties will be placed
	 * @param file the file from which to read properties
	 * @param clear if true, the properties will be cleared before reading
	 */
	protected static void readPropertiesFromFile(Properties properties, File file, boolean clear) {
		FileInputStream in = null;
		try {
			in = new FileInputStream(file);
			if (clear) {
				properties.clear();
			}
			properties.load(in);
		}
		catch (IOException e) {
			// do nothing, instance will just be empty
		}
		finally {
			if (in != null) {
				try {
					in.close();
				}
				catch (IOException e) {
				}
				in = null;
			}
		}
	}


	/**
	 * This allows for properties to be written to a file.
	 *
	 * @param properties These are the properties to write
	 * @param file This is the file to write them to
	 */
	protected static void writePropertiesToFile(Properties properties, File file) {
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(file);
			properties.store(out, " ---User Properties--- output from UserProperties.java");
		}
		catch (IOException e) {
			// do nothing
		}
		finally {
			if (out != null) {
				try {
					out.close();
				}
				catch (IOException e) {
				}
				out = null;
			}
		}
	}


	/**
	 * This allows for the creation of a recursive folder.
	 *
	 * @param f This is the file to create
	 * @return a boolean of whether or not f exists
	 */
	private static boolean createRecursiveFolder(File f) {
		if (f.exists()) {
			return true;
		}
		if (!f.isAbsolute()) {
			f = f.getAbsoluteFile();
		}
		String par = f.getParent();
		if (par == null) {
			return false;
		}
		if (!createRecursiveFolder(new File(par))) {
			return false;
		}
		f.mkdir();
		return f.exists();
	}


	/** Sets the rect attribute of the UserProperties object.
         *
         * @param r this is the rectangle to set
         * @param key The new key value
         */
	public void setRect(String key, Rectangle r) {
		String val = r.getX() + "," + r.getY() + "," + r.getWidth() + "," + r.getHeight();
		this.setProperty(key, val);
	}


	/**
	 * Gets the rect attribute of the UserProperties object.
	 *
	 * @param key the key to get the rect info
	 * @return The rect value
	 */
	public Rectangle getRect(String key) {
		int x = 0;
		int y = 0;
		int width = 0;
		int height = 0;
		String rect = this.getProperty(key);
		try {
			StringTokenizer st = new StringTokenizer(rect, ",");
			     x = (int)Double.parseDouble(st.nextToken());
			     y = (int)Double.parseDouble(st.nextToken());
			 width = (int)Double.parseDouble(st.nextToken());
			height = (int)Double.parseDouble(st.nextToken());
		}
		catch (Exception e) {
			return null;
		}
		return new Rectangle(x, y, width, height);
	}


	/**
	 * This allows saving of properties to a file.
	 */
	public void save() {
		writePropertiesToFile(this, file);
	}
}
