/*
 * @(#)VertexRenderer.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.*;
import java.io.Serializable;
import javax.swing.*;

/** 
 * This renderer displays entries that implement the CellView interface
 * and supports the following attributes. If the cell view is not a leaf,
 * this object is only visible if it is selected.
 *
 * GraphConstants.BOUNDS
 * GraphConstants.ICON
 * GraphConstants.FONT
 * GraphConstants.OPAQUE
 * GraphConstants.BORDER
 * GraphConstants.BORDERCOLOR
 * GraphConstants.LINEWIDTH
 * GraphConstants.FOREGROUND
 * GraphConstants.BACKGROUND
 * GraphConstants.VERTICAL_ALIGNMENT
 * GraphConstants.HORIZONTAL_ALIGNMENT
 * GraphConstants.VERTICAL_TEXT_POSITION
 * GraphConstants.HORIZONTAL_TEXT_POSITION
 *
 * Referenced classes of package com.jgraph.graph are
 * VertexView, CellViewRenderer, CellView, GraphSelectionModel
 * and GraphConstants.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see JLabel
 * @see CellViewRenderer
 * @see Serializable
 */

public class VertexRenderer extends JLabel
    implements CellViewRenderer, Serializable
{

    /**
     * An instance of boolean. 
     * Use this flag to control if groups should appear transparent. 
     */
    protected boolean hideGroups;
    
    /** 
     * An instance of JGraph.
     * Cache the current graph for drawing. 
     */
    protected transient JGraph graph;
    
    /** 
     * An instance of VertexView. 
     * Cache the current shape for drawing. 
     */
    protected transient VertexView view;
    
    /** 
     * An instance of boolean. 
     * Cached hasFocus and selected value. 
     */
    protected transient boolean hasFocus;
    
    /** 
     * An instance of boolean. 
     */
    protected transient boolean selected;
    
    /** 
     * An instance of boolean.
     */
    protected transient boolean preview;
    
    /** 
     * An instance of boolean.
     */
    protected transient boolean opaque;
    
    /**
     * An instance of boolean.
     */
    protected transient boolean childrenSelected;
    
    /** 
     * Cached default foreground. 
     */
    protected transient Color defaultForeground;
    
    /**
     * An instance of Color. 
     * Cached default background.
     */
    protected transient Color defaultBackground;
    
    /**
     * An instance of Color. 
     * Cached borderColor. 
     */
    protected transient Color bordercolor;
    
    /** 
     * An instance of integer.
     * Cached borderwidth. 
     */
    protected transient int borderWidth;

    /**
     * Constructs a renderer that may be used to render vertices.
     */
    public VertexRenderer()
    {
        hideGroups = true;
        defaultForeground = UIManager.getColor("Tree.textForeground");
        defaultBackground = UIManager.getColor("Tree.textBackground");
    }

    /** 
     * Configure and return the renderer based on the passed in
     * components. The value is typically set from messaging the
     * graph with convertValueToString.
     * We recommend you check the value's class and throw an
     * illegal argument exception if it's not correct.
     * @param jgraph A variable of JGraph data type.
     * @param cellview A variable of CellView data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @param flag2 A variable of boolean data type.
     * @return A Component data type.
     * @see JGraph
     * @see CellView
     * @see Component
     */
    public Component getRendererComponent(JGraph jgraph, CellView cellview, boolean flag, boolean flag1, boolean flag2)
    {
        if(cellview instanceof VertexView)
        {
            view = (VertexView)cellview;
            setComponentOrientation(jgraph.getComponentOrientation());
            if(jgraph.getEditingCell() != cellview.getCell())
            {
                String s = jgraph.convertValueToString(cellview);
                if(s != null)
                {
                    setText(s.toString());
                } else
                {
                    setText(null);
                }
            } else
            {
                setText(null);
            }
            graph = jgraph;
            hasFocus = flag1;
            childrenSelected = jgraph.getSelectionModel().isChildrenSelected(cellview.getCell());
            selected = flag;
            preview = flag2;
            if(view.isLeaf() || !hideGroups)
            {
                installAttributes(cellview);
            } else
            {
                setText(null);
                setBorder(null);
                setOpaque(false);
                setIcon(null);
            }
            return this;
        } else
        {
            return null;
        }
    }

    /** 
     * Install the attributes of specified cell in this
     * renderer instance. This means, retrieve every published
     * key from the cells hashtable and set global variables
     * or superclass properties accordingly.
     * @param cellview A variable of CellView data type.
     * @see CellView
     */
    
    protected void installAttributes(CellView cellview)
    {
        java.util.Map map = cellview.getAllAttributes();
        setIcon(GraphConstants.getIcon(map));
        setOpaque(GraphConstants.isOpaque(map));
        setBorder(GraphConstants.getBorder(map));
        setVerticalAlignment(GraphConstants.getVerticalAlignment(map));
        setHorizontalAlignment(GraphConstants.getHorizontalAlignment(map));
        setVerticalTextPosition(GraphConstants.getVerticalTextPosition(map));
        setHorizontalTextPosition(GraphConstants.getHorizontalTextPosition(map));
        bordercolor = GraphConstants.getBorderColor(map);
        borderWidth = Math.max(1, Math.round(GraphConstants.getLineWidth(map)));
        if(getBorder() == null && bordercolor != null)
        {
            setBorder(BorderFactory.createLineBorder(bordercolor, borderWidth));
        }
        Color color = GraphConstants.getForeground(map);
        setForeground(color == null ? defaultForeground : color);
        Color color1 = GraphConstants.getBackground(map);
        setBackground(color1 == null ? defaultBackground : color1);
        setFont(GraphConstants.getFont(map));
    }

    /** 
     * Paint the renderer. Overrides superclass paint
     * to add specific painting.
     * @param g A variable of Graphics data type.
     * @see Graphics
     */
    public void paint(Graphics g)
    {
        super.paint(g);
        paintSelectionBorder(g);
    }

    /** 
     * Provided for subclassers to paint a selection border.
     * @param g A variable of Graphics data type.
     * @see Graphics
     */
    protected void paintSelectionBorder(Graphics g)
    {
        ((Graphics2D)g).setStroke(GraphConstants.SELECTION_STROKE);
        if(childrenSelected)
        {
            g.setColor(graph.getGridColor());
        } else
        if(hasFocus && selected)
        {
            g.setColor(graph.getLockedHandleColor());
        } else
        if(selected)
        {
            g.setColor(graph.getHighlightColor());
        }
        if(childrenSelected || selected)
        {
            Dimension dimension = getSize();
            g.drawRect(0, 0, dimension.width - 1, dimension.height - 1);
        }
    }

    /** 
     * Returns the intersection of the bounding rectangle and the
     * straight line between the source and the specified point p.
     * The specified point is expected not to intersect the bounds.
     * @param vertexview A variable of VertexView data type.
     * @param point A variable of Point data type.
     * @param point1 A variable of Point data type.
     * @return A Point data type.
     * @see VertexView
     * @see Point
     */
    public Point getPerimeterPoint(VertexView vertexview, Point point, Point point1)
    {
        Rectangle rectangle = vertexview.getBounds();
        int i = rectangle.x;
        int j = rectangle.y;
        int k = rectangle.width;
        int l = rectangle.height;
        int i1 = i + k / 2;
        int j1 = j + l / 2;
        int k1 = point1.x - i1;   // Compute Angle
        int l1 = point1.y - j1;
        double d = Math.atan2(l1, k1);
        int i2 = 0;
        int j2 = 0;
        double d1 = 3.1415926535897931D;
        double d2 = 1.5707963267948966D;
        double d3 = d2 - d;
        double d4 = Math.atan2(l, k);
        if(d < -d1 + d4 || d > d1 - d4)          // Left edge
        {
            i2 = i;
            j2 = j1 - (int)(((double)k * Math.tan(d)) / 2D);
        } else            // Top Edge
        if(d < -d4)
        {
            j2 = j;
            i2 = i1 - (int)(((double)l * Math.tan(d3)) / 2D);
        } else             // Right Edge
        if(d < d4)
        {
            i2 = i + k;
            j2 = j1 + (int)(((double)k * Math.tan(d)) / 2D);
        } else                // Bottom Edge
        {
            j2 = j + l;
            i2 = i1 + (int)(((double)l * Math.tan(d3)) / 2D);
        }
        return new Point(i2, j2);
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void validate()
    {
    }

    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    public void revalidate()
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param l A variable of long data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @param k A variable of integer data type.
     * @param i1 A variable of integer data type.
     */
    public void repaint(long l, int i, int j, int k, int i1)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param rectangle A variable of Rectangle data type.
     * @see Rectangle
     */
    public void repaint(Rectangle rectangle)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @see Object
     */
    protected void firePropertyChange(String s, Object obj, Object obj1)
    {
        // Strings get interned...
        if(s == "text")
        {
            super.firePropertyChange(s, obj, obj1);
        }
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param byte0 A variable of byte data type.
     * @param byte1 A variable of byte data type.
     */
    public void firePropertyChange(String s, byte byte0, byte byte1)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param c A variable of char data type.
     * @param c1 A variable of char data type.
     */
    public void firePropertyChange(String s, char c, char c1)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param word0 A variable of short data type.
     * @param word1 A variable of short data type.
     */
    public void firePropertyChange(String s, short word0, short word1)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     */
    public void firePropertyChange(String s, int i, int j)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param l A variable of long data type.
     * @param l1 A variable of long data type.
     */
    public void firePropertyChange(String s, long l, long l1)
    {
    }

    /** Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param f A variable of float data type.
     * @param f1 A variable of float data type.
     */
    public void firePropertyChange(String s, float f, float f1)
    {
    }

    /** Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param d A variable of double data type.
     * @param d1 A variable of double data type.
     */
    public void firePropertyChange(String s, double d, double d1)
    {
    }

    /** 
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     * @param s A variable of String data type.
     * @param flag A variable of boolean data type.
     * @param flag1  A variable of boolean data type.
     */
    public void firePropertyChange(String s, boolean flag, boolean flag1)
    {
    }

    /**
     * Returns the hideGroups.
     * @return A boolean data type.
     */
    public boolean isHideGroups()
    {
        return hideGroups;
    }

    /** 
     * Sets the hideGroups.
     * @param flag A variable of boolean data type.
     */
    public void setHideGroups(boolean flag)
    {
        hideGroups = flag;
    }
}
