/*
 * @(#)VertexView.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

/**
 * A default implementation of vertex view extending AbstractCellView.
 *
 * Referenced classes of package com.jgraph.graph are
 * AbstractCellView, CellView, EdgeView, VertexRenderer, DefaultGraphCell,
 * GraphConstants, GraphModel, GraphContext, CellMapper, CellHandle,
 * GraphLayoutCache and PortView.
 *
 * @version 1.0 1/1/02
 * @author Gaudenz Alder
 */
public class VertexView extends AbstractCellView
{
    /** 
     * Class used to update the sizeHandle information.
     */    
    public class SizeHandle
        implements CellHandle, Serializable
    {
        
        /** 
         * An instance of Image.
         * Double Buffer used in the initOffscreen().
         */        
        protected transient Image offscreen;
        
        /**
         * An instance of Graphics. 
         * Used in the initOffscreen().
         */        
        protected transient Graphics offgraphics;
        
        /**
         * An instance of boolean. 
         * Used in the initOffscreen().
         */        
        protected transient boolean firstDrag;
        
        /**
         * An instance of JGraph. 
         * Contains the current attributes of jGraph.
         */        
        protected transient JGraph graph;
        
        /* 
         * An instance of VertexView.
         * Reference to the temporary view for this handle.
         * Contains the vertex viewpoint information.
         */        
        protected transient VertexView vertex;
        
        /**
         * An instance of PortView. 
         * Reference to the views for this handle.
         */        
        protected transient PortView portViews[];
        
        /** 
         * An instance of Rectangle.
         * Contains the information for the cached bounds.
         */        
        protected transient Rectangle cachedBounds;
        
        /**
         * An instance of  GraphContext.
         * Reference to the context for the specified view.
         */
        protected transient GraphContext context;
        
        /**
         * An instance of Rectangle. 
         * Reference to the context for the specified view. 
         */   
        protected transient Rectangle initialBounds;
        
        /**
         * An instance of CellView.
         * Information for the different views.
         */        
        protected transient CellView contextViews[];
        
        /**
         * An instance of integer. 
         * Index of the active control point. -1 if none is active. 
         */
        protected transient int index;
        
        /**
         * An instance of Rectangle. 
         * Array of control points represented as rectangles. 
         */
        protected transient Rectangle r[];
        
        /**
         * An instance of boolean. 
         * Used to check for firstOverlayInvocation.
         */
        protected boolean firstOverlayInvocation;
        
        /**
         * An instance of integer. 
         * Array that holds the cursors for the different control points. 
         */
        public transient int cursors[] = {
            6, 8, 7, 10, 11, 4, 9, 5
        };

        /** 
         * Paint the size handler.
         * @param g A variable of Graphics data type.
         * @see Graphics 
         */        
        public void paint(Graphics g)
        {
            invalidate();
            g.setColor(graph.getHandleColor());
            for(int i = 0; i < r.length; i++)
            {
                g.fill3DRect(r[i].x, r[i].y, r[i].width, r[i].height, true);
            }

        }

        // Double Buffers by David Larsson.
        /** 
         * Updates the screen with new image or color.
         */        
        protected void initOffscreen()
        {
            try
            {
                Rectangle rectangle = graph.getBounds();
                offscreen = new BufferedImage(rectangle.width, rectangle.height, 1);
                offgraphics = offscreen.getGraphics();
                offgraphics.setClip(0, 0, (int)rectangle.getWidth(), (int)rectangle.getHeight());
                offgraphics.setColor(graph.getBackground());
                offgraphics.fillRect(0, 0, (int)rectangle.getWidth(), (int)rectangle.getHeight());
                graph.getUI().paint(offgraphics, graph);
            }
            catch(Error error)
            {
                offscreen = null;
                offgraphics = null;
            }
        }

        /** 
         * Makes the new changes on the Graphics enviroments
         * such as the new bounds/rectangles.
         * @param g A variable of Graphics data type.
         * @see Graphics
         */        
        public void overlay(Graphics g)
        {
            if(!firstOverlayInvocation)
            {
                if(cachedBounds != null)
                {
                    g.setColor(Color.black);
                    Rectangle rectangle = graph.toScreen(new Rectangle(cachedBounds));
                    g.drawRect(rectangle.x, rectangle.y, rectangle.width - 2, rectangle.height - 2);
                } else
                if(!initialBounds.equals(vertex.getBounds()))
                {
                    Graphics2D graphics2d = (Graphics2D)g;
                    java.awt.geom.AffineTransform affinetransform = graphics2d.getTransform();
                    graphics2d.scale(graph.getScale(), graph.getScale());
                    graph.getUI().paintCell(g, vertex, vertex.getBounds(), true);
                    if(contextViews != null)
                    {
                        for(int i = 0; i < contextViews.length; i++)
                        {
                            graph.getUI().paintCell(g, contextViews[i], contextViews[i].getBounds(), true);
                        }

                    }
                    graphics2d.setTransform(affinetransform);
                    if(portViews != null && graph.isPortsVisible())
                    {
                        graph.getUI().paintPorts(g, portViews);
                    }
                }
            }
            firstOverlayInvocation = false;
        }

        /** 
         * Invoked when the mouse pointer has been moved on a component
         * (with no buttons down).
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */
        public void mouseMoved(MouseEvent mouseevent)
        {
            if(vertex != null)
            {
                for(int i = 0; i < r.length; i++)
                {
                    if(r[i].contains(mouseevent.getPoint()))
                    {
                        graph.setCursor(new Cursor(cursors[i]));
                        mouseevent.consume();
                        return;
                    }
                }

            }
        }
        
        /** 
         * Process mouse pressed event.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */
        public void mousePressed(MouseEvent mouseevent)
        {
            if(!graph.isSizeable())
            {
                return;
            }
            for(int i = 0; i < r.length; i++)
            {
                if(r[i].contains(mouseevent.getPoint()))
                {
                    HashSet hashset = new HashSet();
                    hashset.add(vertex.getCell());
                    contextViews = context.createTemporaryContextViews(hashset);
                    CellView acellview[] = AbstractCellView.getDescendantViews(new CellView[] {
                        vertex
                    });
                    if(acellview.length >= BasicGraphUI.MAXHANDLES)
                    {
                        cachedBounds = new Rectangle(initialBounds);
                    }
                    mouseevent.consume();
                    index = i;
                    return;
                }
            }

        }

        /** 
         * Process mouse dragged event.
         * @param mouseevent A variable of MouseEvent data type.
         * @see MouseEvent
         */
        public void mouseDragged(MouseEvent mouseevent)
        {
            if(firstDrag && graph.isDoubleBuffered() && cachedBounds == null)
            {
                initOffscreen();
                firstDrag = false;
            }
            Rectangle rectangle = null;
            Graphics g = offgraphics == null ? graph.getGraphics() : offgraphics;
            if(index == -1)
            {
                return;
            }
            Rectangle rectangle1 = computeBounds(mouseevent);
            g.setColor(graph.getForeground());
            g.setXORMode(graph.getBackground());
            overlay(g);
            if(offgraphics != null)
            {
                rectangle = graph.toScreen(new Rectangle(vertex.getBounds()));
                Rectangle rectangle2 = graph.toScreen(AbstractCellView.getBounds(contextViews));
                if(rectangle2 != null)
                {
                    rectangle.add(rectangle2);
                }
            }
            if(cachedBounds != null)
            {
                cachedBounds = rectangle1;
            } else
            {
                // Reset old Bounds
                CellView acellview[] = AbstractCellView.getDescendantViews(new CellView[] {
                    vertex
                });
                for(int i = 0; i < acellview.length; i++)
                {
                    CellView cellview = graph.getGraphLayoutCache().getMapping(acellview[i].getCell(), false);
                    java.util.Map map = GraphConstants.cloneMap(cellview.getAllAttributes());
                    acellview[i].setAttributes(map);
                    acellview[i].refresh(false);
                }

                vertex.setBounds(rectangle1);
                if(vertex != null)
                {
                    graph.getGraphLayoutCache().update(vertex);
                }
                if(contextViews != null)
                {
                    graph.getGraphLayoutCache().update(contextViews);
                }
            }
            overlay(g);
            if(offscreen != null)
            {
                rectangle.add(graph.toScreen(new Rectangle(vertex.getBounds())));
                Rectangle rectangle3 = graph.toScreen(AbstractCellView.getBounds(contextViews));
                if(rectangle3 != null)
                {
                    rectangle.add(rectangle3);
                }
                rectangle.grow(2, 2);
                int j = Math.max(0, rectangle.x);
                int k = Math.max(0, rectangle.y);
                int l = j + rectangle.width;
                int i1 = k + rectangle.height;
                graph.getGraphics().drawImage(offscreen, j, k, l, i1, j, k, l, i1, graph);
            }
        }

        /** 
         * Copmutes the bounds of the rectangle.
         * @param mouseevent  A variable of MouseEvent data type.
         * @return A Rectangle data type.
         * @see MouseEvent
         * @see Rectangle
         */        
        protected Rectangle computeBounds(MouseEvent mouseevent)
        {
            int i = initialBounds.x;
            int j = (initialBounds.x + initialBounds.width) - 1;
            int k = initialBounds.y;
            int l = (initialBounds.y + initialBounds.height) - 1;
            Point point = graph.fromScreen(graph.snap(new Point(mouseevent.getPoint())));
            // Bottom row
            if(index > 4)
            {
                l = point.y;
            } else      // Top row
            if(index < 3)
            {
                k = point.y;
            }
            if(index == 0 || index == 3 || index == 5)    // Left col
            {
                i = point.x;
            } else              // Right col
            if(index == 2 || index == 4 || index == 7)
            {
                j = point.x;
            }
            int i1 = j - i;
            int j1 = l - k;
            if(isConstrainedSizeEvent(mouseevent))
            {
                if(index == 3 || index == 4 || index == 5)
                {
                    j1 = i1;
                } else
                if(index == 1 || index == 6 || index == 2 || index == 7)
                {
                    i1 = j1;
                } else
                {
                    j1 = i1;
                    k = l - j1;
                }
            }
            if(i1 < 0)          // Flip over left side
            {
                i += i1;
                i1 = Math.abs(i1);
            }
            if(j1 < 0)          // Flip over top side
            {
                k += j1;
                j1 = Math.abs(j1);
            }
            return new Rectangle(i, k, i1 + 1, j1 + 1);
        }

        /** 
         * Dispatch the edit event
         * @param mouseevent  A variable of MouseEvent data type.
         * @see MouseEvent
         */        
        public void mouseReleased(MouseEvent mouseevent)
        {
            if(index != -1)
            {
                cachedBounds = computeBounds(mouseevent);
                vertex.setBounds(cachedBounds);
                CellView acellview[] = AbstractCellView.getDescendantViews(new CellView[] {
                    vertex
                });
                java.util.Map map = GraphConstants.createAttributes(acellview, null);
                graph.getGraphLayoutCache().edit(map, null, null, null);
            }
            mouseevent.consume();
            cachedBounds = null;
            initialBounds = null;
            firstDrag = true;
        }

        /** 
         * If an invalid event occurs, Retrieves current bounds and sets local vars.
         * After that, Updates control point positions.
         */        
        private void invalidate()
        {
            // Retrieve current bounds and set local vars
            Rectangle rectangle = graph.getCellBounds(vertex.getCell());
            if(rectangle != null)
            {
                rectangle = new Rectangle(rectangle);
                graph.toScreen(rectangle);
                int i = graph.getHandleSize();
                int j = 2 * i;
                int k = rectangle.x - i;
                int l = rectangle.y - i;
                int i1 = (rectangle.x + rectangle.width / 2) - i;
                int j1 = (rectangle.y + rectangle.height / 2) - i;
                int k1 = (rectangle.x + rectangle.width) - i;
                int l1 = (rectangle.y + rectangle.height) - i;
                // Update control point positions
                r[0].setBounds(k, l, j, j);
                r[1].setBounds(i1, l, j, j);
                r[2].setBounds(k1, l, j, j);
                r[3].setBounds(k, j1, j, j);
                r[4].setBounds(k1, j1, j, j);
                r[5].setBounds(k, l1, j, j);
                r[6].setBounds(i1, l1, j, j);
                r[7].setBounds(k1, l1, j, j);
            }
        }

        /** 
         * Creates the preview.
         * @param vertexview1 A variable of VertexView data type.
         * @param graphcontext A variable of GraphContext data type.
         * @see VertexView
         * @see GraphContext
         */        
        public SizeHandle(VertexView vertexview1, GraphContext graphcontext)
        {
            firstDrag = true;
            index = -1;
            r = new Rectangle[8];
            firstOverlayInvocation = true;
            graph = graphcontext.getGraph();
            vertex = vertexview1;
            // PortView Preview
            portViews = graphcontext.createTemporaryPortViews();
            initialBounds = new Rectangle(vertex.getBounds());
            context = graphcontext;
            for(int i = 0; i < r.length; i++)
            {
                r[i] = new Rectangle();
            }

            invalidate();
        }
    }

    /**
     * An instance of VertexRenderer. 
     * Renderer for the class. 
     */
    public static VertexRenderer renderer = new VertexRenderer();
    
    /** 
     * An instance of Rectangle.
     * Reference to the bounds attribute. 
     */
    protected Rectangle bounds;
    
    /** 
     * An instance of Rectangle.
     * Cached bounds of all children if vertex is a group 
     */
    protected Rectangle groupBounds;

    /** 
     * Constructs a vertex view for the specified model object
     * and the specified child views.
     * @param obj  A variable of Object data type.
     * @param jgraph  A variable of JGraph data type.
     * @param cellmapper  A variable of CellMapper data type.
     * @see Object
     * @see JGraph
     * @see CellMapper
     */
    public VertexView(Object obj, JGraph jgraph, CellMapper cellmapper)
    {
        super(obj, jgraph, cellmapper);
        groupBounds = DefaultGraphCell.defaultBounds;
    }

    //
    // CellView Interface
    //

    /**
     * Overrides the parent method to udpate the cached points.
     */
    public void update()
    {
        super.update();
        bounds = GraphConstants.getBounds(allAttributes);
        groupBounds = null;
    }

    /** 
     * Updates the child information.
     */    
    public void childUpdated()
    {
        super.childUpdated();
        groupBounds = null;
    }

    /** 
     * Returns the cached bounds for the vertex.
     * Information is stored in object Rectangle.
     * @return A Rectangle data type.
     * @see Rectangle
     */
    public Rectangle getBounds()
    {
        if(!isLeaf())
        {
            if(groupBounds == null)
            {
                updateGroupBounds();
            }
            return groupBounds;
        } else
        {
            return bounds;
        }
    }

     /** 
      * Returns the cached bounds for the vertex.
      * Information is stored in object Rectangle.
      * @return A Rectangle data type.
      * @see Rectangle
      */     
    public Rectangle getCachedBounds()
    {
        return bounds;
    }

    /** 
     * Sets the cached bounds for the vertex passed.
     * @param rectangle A variable of Rectangle data type.
     * @see Rectangle
     */    
    public void setCachedBounds(Rectangle rectangle)
    {
        bounds = rectangle;
    }

    /** 
     * Updates Cached bounds of all children if vertex
     * is a group.
     */    
    protected void updateGroupBounds()
    {
        // Note: Prevent infinite recursion by removing
	// child edges that point to their parent.
        CellView acellview[] = getChildViews();
        LinkedList linkedlist = new LinkedList();
        for(int i = 0; i < acellview.length; i++)
        {
            if(includeInGroupBounds(acellview[i]))
            {
                linkedlist.add(acellview[i]);
            }
        }

        acellview = new CellView[linkedlist.size()];
        linkedlist.toArray(acellview);
        groupBounds = AbstractCellView.getBounds(acellview);
    }

    /** 
     * Returns a boolean if the cached bounds are
     * included in a Group.
     * @param cellview A variable of CellView data type.
     * @return A boolean data type.
     * @see CellView
     */    
    private boolean includeInGroupBounds(CellView cellview)
    {
        if(cellview instanceof EdgeView)
        {
            GraphModel graphmodel = graph.getModel();
            EdgeView edgeview = (EdgeView)cellview;
            if(edgeview.getCell() instanceof DefaultMutableTreeNode)
            {
                DefaultMutableTreeNode defaultmutabletreenode = (DefaultMutableTreeNode)edgeview.getCell();
                if(graphmodel.getSource(defaultmutabletreenode) instanceof TreeNode)
                {
                    TreeNode treenode = (TreeNode)graphmodel.getSource(defaultmutabletreenode);
                    if(((DefaultMutableTreeNode)treenode.getParent()).isNodeDescendant(defaultmutabletreenode))
                    {
                        return false;
                    }
                }
                if(graphmodel.getTarget(defaultmutabletreenode) instanceof TreeNode)
                {
                    TreeNode treenode1 = (TreeNode)graphmodel.getTarget(defaultmutabletreenode);
                    if(((DefaultMutableTreeNode)treenode1.getParent()).isNodeDescendant(defaultmutabletreenode))
                    {
                        return false;
                    }
                }
            }
        }
        return true;
    }

    /** 
     * Returns a renderer for the class.
     * @return A CellViewRenderer data type.
     * @see CellViewRenderer
     */
    public CellViewRenderer getRenderer()
    {
        return renderer;
    }

    /** 
     * Returns a cell handle for the view, if the graph and the view
     * are sizeable.
     * @param graphcontext A variable of GraphContext data type.
     * @return A CellHandle data type.
     * @see GraphContext
     * @see CellHandle
     */
    public CellHandle getHandle(GraphContext graphcontext)
    {
        if(GraphConstants.isSizeable(getAllAttributes()) && graphcontext.getGraph().isSizeable())
        {
            return new SizeHandle(this, graphcontext);
        } else
        {
            return null;
        }
    }

    //
    // Special Methods
    //

    /** 
     * Returns the center of this vertex.
     * @return A Point data type.
     * @see Point
     */
    public Point getCenterPoint()
    {
        Rectangle rectangle = getBounds();
        return new Point((int)rectangle.getCenterX(), (int)rectangle.getCenterY());
    }

    /** 
     * Returns the intersection of the bounding rectangle and the
     * straight line between the source and the specified point p.
     * The specified point is expected not to intersect the bounds.
     * @param point A variable of Point data type.
     * @param point1 A variable of Point data type.
     * @return A Point data type.
     * @see Point
     */
   
    public Point getPerimeterPoint(Point point, Point point1)
    {
        return renderer.getPerimeterPoint(this, point, point1);
    }

    
    /** 
     * Returns a boolean based on the isConstrainedMoveEvent
     * of a mouseEvent if the userInterface is an instance of
     * BasicGraphUI. Otherwise returns false
     * @param mouseevent A variable of MouseEvent data type.
     * @return A boolean data type.
     * @see MouseEvent
     */    
    public boolean isConstrainedSizeEvent(MouseEvent mouseevent)
    {
        GraphUI graphui = graph.getUI();
        if(graphui instanceof BasicGraphUI)
        {
            return ((BasicGraphUI)graphui).isConstrainedMoveEvent(mouseevent);
        } else
        {
            return false;
        }
    }

}
