/*
 * @(#)ToolsLookAndFeel.java	1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;



/**
 * This class is used to control the look and the feel for the TerpPresent application.
 *
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class WindowLookAndFeel extends AbstractActionRadioButton {

  /**
   * An array of lookAndFeels objects that controls the look and feel of the program.
   */
  protected UIManager.LookAndFeelInfo[] lookAndFeels = null;

  /**
   * Creates a new WindowLookAndFeel object that describes the specified graphpad.
   *
   * @param graphpad The graphpad whose look and feel we want to describe.
   */
  public WindowLookAndFeel(GPGraphpad graphpad) {
    super(graphpad);
    lastActionCommand = UIManager.getLookAndFeel().getClass().getName();
  }

  /**
   * Returns an String array where each element is one of the possible action commands.
   *
   * @return A String array of all possible action commands.
   */
  public String[] getPossibleActionCommands() {
    if (lookAndFeels == null)
      lookAndFeels = UIManager.getInstalledLookAndFeels();
    String[] values = new String[lookAndFeels.length];

    for (int i = 0; i < lookAndFeels.length; i++) {
      UIManager.LookAndFeelInfo laf = lookAndFeels[i];
      values[i] = laf.getClassName();
    }
    return values;
  }

  /**
   * This function updates the look and feel of the graph after an action has been performed on it.
   *
   * @param e The event that was raised when an action was performed.
   */
  public void actionPerformed(ActionEvent e) {
    this.lastActionCommand = e.getActionCommand();
    try {
      UIManager.setLookAndFeel(e.getActionCommand());
      SwingUtilities.updateComponentTreeUI(graphpad.getFrame());
    } catch (Exception ex) {
    };
    update();
  }

  /**
   * Updates all Abstract Buttons, because the acton may have caused a change in them.
   */
  public void update() {
    Enumeration enum = abstractButtons.elements();
    while (enum.hasMoreElements()) {
      AbstractButton button = (AbstractButton) enum.nextElement();
      button.setSelected(isSelected(button.getActionCommand()));
    }
  };

  /**
   * Returns the presentation Text for the action command or null as the default.
   *
   * @param actionCommand String that specifies the action.
   *
   * @return - Either the presentation text for the action command or null.
   */
  public String getPresentationText(String actionCommand) {
    if (actionCommand == null)
      return null;

    for (int i = 0; i < lookAndFeels.length ; i++){
      if (lookAndFeels[i].getClassName().equals(actionCommand)){
        return lookAndFeels[i].getName() ;
      }
    }

    return actionCommand;
  }

}
